!-------------------------------------------------------------------------
!     Last change:  ZK   14 Oct 2018    3:48 pm
! THIS PROGRAM DELETES ALL NONLINEAR TERMS IN THE DEEP 
! WATER   (DE PTH.GT.40M) by introducing cell number PCELL=1 
!in shallow water and PCELL=0 in deep water.
 !Program constructed by Z.K.
!
! Additional Changes made by T. Logan June, 2012.
!	- reordered loops into possible parallelization  
!	
!-------------------------------------------------------------------------

PROGRAM slosh
  INCLUDE "block2d_2min-1"

!-------------------------------------------------------------------------
!       Note -- all units are in CGS (Set density = 1)
!
!          V  |    V |  V  |
!   K+1
!         --  +  --  + --  +      SPACE LOCATION OF SEA LEVEL AND VELOCITY
!         U   Z  U   Z  U  Z
!          J-1     J    J+1       GRID ENUMERATION
!          V  |    V |  V  |
!   K
!         --  +  --  + --  +      SPACE LOCATION OF SEA LEVEL AND VELOCITY
!         U   Z  U   Z  U  Z
!          J-1     J    J+1       GRID ENUMERATION
!-------------------------------------------------------------------------

  REAL, DIMENSION(1:121,1:121)::ETS1
! ETS1 sea level at the source function ( bottom uplift)
       
    DHU=0.;DHV=0.;H=0.
    M=0; CELL=0; VO = 0.0 ;VN=0.;SLNN=0.
    UO=0.;UN=0.;SLO=0.;SLN=0.;D=0.; H=0.
    SLMA=0.
    JM=0;KM=0
    ZMAX=-1.
    T1=0.;T2=0.

    call cpu_time ( t1 )

    open(unit=16,file='surface.dat',form='unformatted',status='unknown')
    open(unit=21,file='ampmax.dat',form='unformatted',status='unknown')
    open(unit=25,file='ale2min.dat',form='unformatted',status='unknown')
    open(unit=27,file='source_2min.dat',status='unknown')

!--ESTABLISH DEPTH AND SEA LEVEL
    CALL DEPTH4_RIGHT
!    PRINT *, 'H(650,600)=',H(650,600),H(2000,600)

    CALL TRIGON

    do k=1,KE
      do j=1,je
        ETS(J,K)=0.
      END DO
    END DO

    do k1=1, 121
      READ(27,*)(ETS1(J1,K1),J1=1,121)
    END DO
! Set a source function grid into the large domain grid

      do k1=1,121
           K=K1+810
           DO J1=1,121
           J=960+J1
           ETS(J,K)=ETS1(J1,K1)
           END DO
           END DO
    DO K=1,KE
      DO J=1,JE
        IF(H(J,K).LE.0.)ETS(J,K)=0.            ! Islands no down
        SLO(J,K)=ETS(J,K)
      END DO
    END DO
    PRINT *, 'SLO(650,600)=',SLO(650,600)

!--ESTABKISH WET-DRY-------------
    DO K=KS,KE
      DO j=js,je
        IF(H(J,K).LT.EPSILON)THEN
            CELL(J,K)=0
        ELSE  IF (H(J,K).GE.EPSILON) THEN
           CELL(J,K)=1
        END IF
      END DO
    END DO
    
!--DELETE NONLINEAR FOR DEPTH G.T. 40 M  by using PCELL number
    DO K=KS,KE
      DO j=js,je
        PCELL(J,K)=0
        IF(H(J,K).LT.4000.)THEN
            PCELL(J,K)=1
        END IF
      END DO
    END DO

!--------------------------------------------------------------------------------
!   Starting main time loop
!--------------------------------------------------------------------------------    
    TIMELOOP: DO M=1, ME    !TIME LOOP
        CALL NORTH_RADIATING
        CALL SOUTH_RADIATING
        CALL RIGHT_RADIATING
        CALL LEFT_RADIATING

	! ONLY ONE SUBROUTINE FOR VELOCITY, BUT FOR BEACH4 IT HAS TO BE MODIFIED
	! FOR RADIATION AT RIGHT END OFFSET DOMAIN
        CALL VELOCITYX
        CALL VELOCITYY
        CALL CONTX
        CALL CONTY
! Change old to new variables
        SLO=SLN
        UO=UN
        VO=VN

        CALL WRITER_BEACH
        DO K=1,KE
          DO J=1,JE
            IF (ABS(SLO(J,K)).GT.80000.AND.CELL(J,K).EQ.1)THEN
              PRINT*, J,K, SLO(J,K)
               GO TO  203
                END IF
               END DO
           END DO
	
	
	! JUST FOR QUICK ORIENTATION
        DO k=KS,KE
          DO j=JS,JE
            IF(SLO(J,K).GT.ZMAX(J,K).and.CELL(J,K).EQ.1)THEN
               ZMAX(J,K)=SLO(J,K)
                   END IF
                   IF(ZMAX(J,K).GT.SLMA)THEN
                   SLMA=ZMAX(J,K)
                JM=J
                KM=K
              END IF
            END DO
           END DO

    END  DO  TIMELOOP ! TIME LOOP
    CALL WRITER_MAX
    
203  CONTINUE

    PRINT*,'SLMAX=',SLMA,'JMAX=',JM, 'KMAX=',KM

    call cpu_time ( t2 )
    write ( *, * ) 'Elapsed CPU time = ', t2 - t1

!END  program SLOSH

!-------------------------------------------------------------------------
  CONTAINS
!-------------------------------------------------------------------------

SUBROUTINE DEPTH4_RIGHT
    READ (25) ((H(J,K),J=1,JE),K=1,KE)
    DO K=KS,KE
      DO J=JS,JE
       H(J,K)=-H(J,K)*100.    !IN CM AND DEPTH ON LAND NEGATIVE
      END DO
    END DO
    DO K=KS,KE
      DO j=js+1,je
      DHU(J,K)=(H(J-1,K)+H(J,K))*0.5
      END DO
    END DO
    DO K=KS,KE
      DHU(JS,K) = DHU(JS+1,K)
    END DO
    DO K=KS,KE-1
      DO J=JS,JE
       DHV(J,K)=(H(J,K+1)+H(J,K))*0.5
!  depth in V  points       
        END DO
    END DO
    DO J=JS,JE
      DHV(J,KE)=DHV(J,KE-1)
    END DO
END SUBROUTINE

SUBROUTINE TRIGON
    HALF=0.5*DF
     BLAT=18.
!        STARTING  IN DEGREES; LATITUDE MINUS IN SOUTHERN HEMISPHERE
     DO K=KS,KE

     RLAT=BLAT+DF*FLOAT(K-1)
! STEPPING IN 1DEG
     A1=RLAT*RAD
      A2=(RLAT+HALF)*RAD
      A3=(RLAT-HALF)*RAD
      SIZ(K)=SIN(A1)
      SIV(K)=SIN(A2)
      CSZ(K)=COS(A1)
      CSV(K)=COS(A2)
      CSV1(K)=COS(A3)
    END DO

    DO K=KS,KE
      CSVZ(K)=CSV(K)/CSZ(K)
      CSVZ1(K)=CSV1(K)/CSZ(K)
    END DO
END SUBROUTINE

SUBROUTINE RIGHT_RADIATING
    DO K=KS+1,KE-1
      HTOT= H(JE-1,K)+SLO(JE-1,K)
      IF(HTOT.LT.0)HTOT=0.
      SLO(JE,K)=UO(JE,K)*SQRT(HTOT/G)
      SLN(JE,K)=SLO(JE,K)
    END DO
END SUBROUTINE

SUBROUTINE LEFT_RADIATING
    DO K=KS,KE
      HTOT= H(JS+1,K)+SLO(JS+1,K)
      IF(HTOT.LT.0)HTOT=0.
      SLO(JS,K)=-UO(JS+1,K)*SQRT(HTOT/G)
      SLN(JS,K)=SLO(JS,K)
    END DO
END SUBROUTINE

SUBROUTINE NORTH_RADIATING
    DO j=JS,JE
      HTOT= H(J,KE)+SLO(J,KE)
      IF(HTOT.LT.0)HTOT=0.
      SLO(J,KE)=VO(J,KE-1)*SQRT(HTOT/G)
      SLN(J,KE)=SLO(J,KE)
    END DO
END SUBROUTINE

SUBROUTINE SOUTH_RADIATING
    DO j=JS,JE
      HTOT= H(J,KS)+SLO(J,KS)
      IF(HTOT.LT.0.)HTOT=0.
      SLO(J,KS)=-VO(J,KS)*SQRT(HTOT/G)
      SLN(J,KS)=SLO(J,KS)
    END DO
END SUBROUTINE

!-------------------------------------------------------------------------
!-------------------------------------------------------------------------
SUBROUTINE VELOCITYX
    REAL, DIMENSION(1:JE,1:KE)::DU
    REAL :: MAG1,MAG
    
    DU=0.

! GENERAL EQUATION
    DO K=KS+1,KE-1
      DO j = JS+2, JE - 1
         IF(CELL(J-1,K).EQ.1.AND.CELL(J,K).EQ.1)THEN
           IF(PCELL(J,K)==1)THEN
             VAU=0.25*(VO(J,K)+VO(J-1,K)+VO(J-1,K-1)+VO(J,K-1))
            VAUP=0.5*(VAU+ABS(VAU))
            VAUN=0.5*(VAU-ABS(VAU))
            MAG1= UO(J,K)*UO(J,K)+VAU*VAU
            MAG=SQRT(MAG1)
            DU(J,K)=DHU(J,K)+0.5*(SLO(J-1,K)+SLO(J,K))

            DDU=DU(J,K)
            IF(DU(J,K).LE.2.)DDU=2.   ! DEFENCE AGAINST ZERO DENOM. IN BOTTOM FRICTION

            UOP=0.5*(UO(j,K)+ABS(UO(j,K)))
            UON=0.5*(UO(J,K)-ABS(UO(J,K)))

            UN(J,K)=UO(J,K) &
              +NL*T*SIZ(K)*VAU*UO(J,K)/(CSZ(K)*R0)&
              -THX*(UOP*(UO(J,K)-UO(J-1,K))+UON*(UO(J+1,K)-UO(J,K)))/CSZ(K) &
              -THY*(VAUP*(UO(J,K)-UO(J,K-1))+VAUN*(UO(J,K+1)-UO(J,K)))&
              +GTHX*(SLO(j-1,K) - SLO(j,K))/CSZ(K)&
              -r*T*UO(j,K)*MAG/DDU
              END IF
          IF(PCELL(J,K)==0)THEN
            UN(J,K)=UO(J,K) + GTHX*(SLO(j-1,K) - SLO(j,K))/CSZ(K)
          END IF
        END IF
      END DO
    END DO
    
! BOUNDARY West (J=JS+1)
    DO K=KS+1,KE
      IF(CELL(JS,K).EQ.1.AND.CELL(JS+1,K).EQ.1)THEN
        DU(JS+1,K)=DHU(JS+1,K)+0.5*(SLO(JS,K)+SLO(JS+1,K))
        IF(DU(JS+1,K).LE.5.)DU(JS+1,K)=5.
        UN(JS+1,K)=UO(JS+1,K)-GTHX*(SLO(JS+1,K)-SLO(JS,K))/CSZ(K)-r*T*UO(JS+1,K)*abs(UO(JS+1,K))/du(JS+1,K)
      END IF
    END DO

! BOUNDARY EAST (J=JE)
    DO K=KS+1,KE-1
      IF(CELL(JE-1,K).EQ.1.AND.CELL(JE,K).EQ.1)THEN
        DU(JE,K)=DHU(JE,K)+0.5*(SLO(JE-1,K)+SLO(JE,K))
        DDU=DU(JE,K)
        IF(DU(JE,K).LE.5.)DDU=5.
        UN(JE,K)=UO(JE,K)+GTHX*(SLO(JE-1,K)-SLO(JE,K))/CSZ(K)-r*T*UO(JE,K)*abs(UO(JE,K))/DDU
      END IF
    END DO

! BOUNDARY SOUTH (K=KS)
    DO j = JS+2, JE - 1
      IF(CELL(J-1,KS).EQ.1.AND.CELL(J,KS).EQ.1)THEN
        VAU=0.5*(VO(J,KS)+VO(J-1,KS))
        VAUP=0.5*(VAU+ABS(VAU))
        VAUN=0.5*(VAU-ABS(VAU))
        MAG1= UO(J,KS)*UO(J,KS)+VAU*VAU
        MAG=SQRT(MAG1)
        DU(J,KS)=DHU(J,KS)+0.5*(SLO(J-1,KS)+SLO(J,KS))

        DDU=DU(J,KS)
        IF(DU(J,KS).LE.2.)DDU=2.   ! DEFENCE AGAINST ZERO DENOM. IN BOTTOM FRICTION

        UOP=0.5*(UO(j,KS)+ABS(UO(j,KS)))
        UON=0.5*(UO(J,KS)-ABS(UO(J,KS)))

        UN(J,KS)=UO(J,KS) &
          +NL*T*SIZ(KS)*VAU*UO(J,KS)/(CSZ(KS)*R0)&
          -THX*(UOP*(UO(J,KS)-UO(J-1,KS))+UON*(UO(J+1,KS)-UO(J,KS)))/CSZ(KS) &
          -THY*VAUN*(UO(J,KS+1)-UO(J,KS))&
          +GTHX*(SLO(j-1,KS) - SLO(j,KS))/CSZ(KS)&
          -r*T*UO(j,KS)*MAG/DDU
      END IF
    END DO
    
! BOUNDARY NORTH (K=KE)
    DO j = JS+2, JE - 1
      IF(CELL(J-1,KE).EQ.1.AND.CELL(J,KE).EQ.1)THEN
        VAU=0.5*(VO(J-1,KE-1)+VO(J,KE-1))
        VAUP=0.5*(VAU+ABS(VAU))
        MAG1= UO(J,KE)*UO(J,KE)+VAU*VAU
        MAG=SQRT(MAG1)
        DU(J,KE)=DHU(J,KE)+0.5*(SLO(J-1,KE)+SLO(J,KE))

        DDU=DU(J,KE)
        IF(DU(J,KE).LE.2.)DDU=2.   ! DEFENCE AGAINST ZERO DENOM. IN BOTTOM FRICTION

        UOP=0.5*(UO(j,KE)+ABS(UO(j,KE)))
        UON=0.5*(UO(J,KE)-ABS(UO(J,KE)))

        UN(J,KE)=UO(J,KE) &
          +NL*T*SIZ(KE)*VAU*UO(J,KE)/(CSZ(KE)*R0)&
          -THX*(UOP*(UO(J,KE)-UO(J-1,KE))+UON*(UO(J+1,KE)-UO(J,KE)))/CSZ(KE) &
          -THY*VAUP*(UO(J,KE)-UO(J,KE-1))&
          +GTHX*(SLO(j-1,KE) - SLO(j,KE))/CSZ(KE)&
          -r*T*UO(j,KE)*MAG/DDU
      END IF
    END DO

  END SUBROUTINE
!-------------------------------------------------------------------------
!-------------------------------------------------------------------------
SUBROUTINE CONTX
    REAL, DIMENSION(1:JE,1:KE):: FLUX
    
    FLUX=0
    DO K=KS+1,KE-1
      DO J = JS+1, JE   ! SET J=JE WHEN RADIATE, OTHERWISE J=JE-1
        IF(CELL(J,K).EQ.1.AND.CELL(J-1,K).EQ.1)THEN    ! THIS CONTROLS ONLY ONE WALL (LEFT)
          ULP=0.5*(UN(J,K)+ABS(UN(J,K)))
          ULN=0.5*(UN(J,K)-ABS(UN(J,K)))
          SP=SLO(J-1,K)
          SN=SLO(J,K)
          FLUX(J,K)=ULP*SP+ULN*SN+DHU(J,K)*UN(J,K)
        END IF
      END DO
    END DO

    DO K= KS+1,KE-1
      DO J = JS+1, JE-1
        SLNN(J,K)=SLO(J,K)- (FLUX(J+1,K)-FLUX(J,K))*THX/CSZ(K)
      END DO
    END DO
END SUBROUTINE

!-------------------------------------------------------------------------
!-------------------------------------------------------------------------
SUBROUTINE CONTY
    REAL, DIMENSION(JS:JE,KS:KE)::  FluxY
!    INTEGER:: KEE
    
    FLUXY=0.
    DO K=KS,KE-1
      DO J = JS,JE
        SP=0.; SN=0.
        IF(CELL(J,K).EQ.1.AND.CELL(J,K+1).EQ.1)THEN    ! THIS CONTROLS ONLY ONE WALL (LEFT)
          VLP=0.5*(VN(J,K)+ABS(VN(J,K)))
          VLN=0.5*(VN(J,K)-ABS(VN(J,K)))
          SP=SLO(J,K)
          SN=SLO(J,K+1)
          FLUXY(J,K)=VLP*SP+VLN*SN+DHV(J,K)*VN(J,K)
          FLUXY(J,K)=FLUXY(J,K)*CSV(K)
        END IF
      END DO
    END DO

    DO K = KS+1, KE-1
      DO J = JS+1, JE-1
        SLN(J,K)=SLNN(J,K)- (FLUXY(J,K)-FLUXY(J,K-1))*THY/CSZ(K)
      END DO
    END DO
END SUBROUTINE

!-------------------------------------------------------------------------
!-------------------------------------------------------------------------
SUBROUTINE VELOCITYY
    REAL, DIMENSION(1:JE,1:KE)::DV
    REAL :: MAG1,MAG
    
    DV=0.

! NORTH BOUNDARY (K=KE-1) IS ACCOUNTED IN GENERAL EQUATION

! BOUNDARY SOUTH  (K=KS)
    DO j = JS, JE-1
      IF(CELL(J,KS).EQ.1.AND.CELL(J,KS+1).EQ.1)THEN
        DV(J,KS)=DHV(J,KS)+0.5*(SLO(J,KS)+SLO(J,KS+1))
        IF(DV(J,KS).LE.5.)DV(J,KS)=5.
        UAV=0.25*(UN(J,KS)+UN(J+1,KS)+UN(J+1,KS+1)+UN(J,KS+1))
        VN(J,KS)=VO(J,KS)&
          -NL*T*UAV*UAV*SIV(K)/(CSV(K)*R0)&
          -GTHY*(SLO(J,KS+1)-SLO(J,KS))-r*T*VO(J,KS)*abs(VO(J,KS))/dV(J,KS)
      END IF
    END DO
    
! BOUNDARY WEST  (J=JS)
    DO K=KS+1,KE-1
      IF(CELL(JS,K).EQ.1.AND.CELL(JS,K+1).EQ.1)THEN
        UAV=0.5*(UO(JS+1,K)+UO(JS+1,K+1))
        UAVP=0.5*(UAV+ABS(UAV))
        UAVN=0.5*(UAV-ABS(UAV))
        MAG1=UAV*UAV+VO(JS,K)*VO(JS,K)
        MAG=SQRT(MAG1)
        DV(Js,K)=DHV(Js,K)+0.5*(SLO(JS,K)+SLO(JS,K+1))
        DDV=DV(JS,K)
        IF(DV(JS,K).LE.2.)DDV=2.   ! DEFENCE AGAINST ZERO DENOM. IN BOTTOM FRICTION
        VOP=0.5*(VO(jS,K)+ABS(VO(jS,K)))
        VON=0.5*(VO(JS,K)-ABS(VO(JS,K)))
        VN(jS,K)=VO(JS,K)&
           -THY*(VOP*(VO(Js,K)-VO(Js,K-1))+VON*(VO(Js,K+1)-VO(Js,K))) &
             -THX*UAVN*(VO(JS+1,K)-VO(JS,K))/CSV(K)&
            +GTHY*(SLO(jS,K) - SLO(jS,K+1))&
            -NL*T*UAV*UAV*SIV(K)/(CSV(K)*R0) &
            -r*T*VO(jS,K)*MAG/DDV
      END IF
    END DO

!BOUNDARY EAST(J=JE)
    DO K=KS+1,KE-1
      IF(CELL(JE,K).EQ.1.AND.CELL(JE,K+1).EQ.1)THEN
        UAV=0.5*(UO(JE,K)+UO(JE,K+1))
        UAVP=0.5*(UAV+ABS(UAV))
        UAVN=0.5*(UAV-ABS(UAV))
        MAG1=UAV*UAV+VO(JE,K)*VO(JE,K)
        MAG=SQRT(MAG1)
        DV(JE,K)=DHV(JE,K)+0.5*(SLO(JE,K)+SLO(JE,K+1))

        DDV=DV(JE,K)
        IF(DV(JE,K).LE.2.)DDV=2.   ! DEFENCE AGAINST ZERO DENOM. IN BOTTOM FRICTION
        VOP=0.5*(VO(jE,K)+ABS(VO(jE,K)))
        VON=0.5*(VO(JE,K)-ABS(VO(JE,K)))

        VN(jE,K)=VO(JE,K)&
          -THY*(VOP*(VO(JE,K)-VO(JE,K-1))+VON*(VO(JE,K+1)-VO(JE,K))) &
            -THX*UAVP*(VO(JE,K)-VO(JE-1,K))/CSV(K)&
           +GTHY*(SLO(jE,K) - SLO(jE,K+1))&
           -NL*T*UAV*UAV*SIV(K)/(CSV(K)*R0) &
           -r*T*VO(jE,K)*MAG/DDV
      END IF
    END DO

! GENERAL EQUATION
    DO K=KS+1,KE-1
      DO j = JS+1, JE-1
        IF(CELL(J,K).EQ.1.AND.CELL(J,K+1).EQ.1)THEN
          IF(PCELL(J,K).EQ.1)THEN
            UAV=0.25*(UO(J,K)+UO(J+1,K)+UO(J+1,K+1)+UO(J,K+1))
            UAVP=0.5*(UAV+ABS(UAV))
            UAVN=0.5*(UAV-ABS(UAV))
            MAG1=UAV*UAV+VO(J,K)*VO(J,K)
            MAG=SQRT(MAG1)
            DV(J,K)=DHV(J,K)+0.5*(SLO(J,K)+SLO(J,K+1))

            DDV=DV(J,K)
            IF(DV(J,K).LE.2.)DDV=2.   ! DEFENCE AGAINST ZERO DENOM. IN BOTTOM FRICTION
            VOP=0.5*(VO(j,K)+ABS(VO(j,K)))
            VON=0.5*(VO(J,K)-ABS(VO(J,K)))

            VN(j,K)=VO(J,K)&
            -THY*(VOP*(VO(J,K)-VO(J,K-1))+VON*(VO(J,K+1)-VO(J,K))) &
               -THX*(UAVP*(VO(J,K)-VO(J-1,K))+UAVN*(VO(J+1,K)-VO(J,K)))/CSV(K)&
               +GTHY*(SLO(j,K) - SLO(j,K+1))&
               -NL*T*UAV*UAV*SIV(K)/(CSV(K)*R0) &
               -r*T*VO(j,K)*MAG/DDV
          END IF
          IF(PCELL(J,K).EQ.0)THEN
            VN(j,K)=VO(J,K) + GTHY*(SLO(j,K) - SLO(j,K+1))
          END IF
        END IF
      END DO
    END DO
END SUBROUTINE

!-------------------------------------------------------------------------
!-------------------------------------------------------------------------
subroutine WRITER_BEACH
    REAL, DIMENSION(JS:JE,KS:KE)::  zwrite
    REAL :: FMA
    INTEGER,PARAMETER::MA=10      ! number of time step for averaging
    
    ZWRITE=-1.
    FMA=FLOAT(MA)
    if(mod(m,4*MA).eq.0)then
      PRINT*,"MAXVALUE= ", MAXVAL(SLO,MASK=(CELL.eq.1)),&
      "TIME STEP= ",M
      PRINT*,"MAX_LOCATION= ", MAXLOC(SLO,MASK=(CELL.eq.1))
    END IF
end subroutine

!-------------------------------------------------------------------------
!-------------------------------------------------------------------------
subroutine WRITER_MAX
    DO K=KS,KE
      DO J=JS,JE
        IF(cell(j,k).eq.0)ZMAX(j,k)=-1
      END DO
    END DO
    write (21) ((ZMAX(J,K),J=1,JE,2),K=1,KE,2)
end subroutine

!-------------------------------------------------------------------------
!-------------------------------------------------------------------------
!-------------------------------------------------------------------------
!-------------------------------------------------------------------------
END  program SLOSH




