!     Last change:  ZK    7 Nov 2018    2:42 pm
!------------------------------------------------------------------------
PROGRAM NEOWAVE2D
!------------------------------------------------------------------------
!                       program NEOWAVE2D.f90
!           hydrostatic and  non-hydrostatic Evolution of Ocean WAVE 2D model
!                       (NEOWAVE2D version 1)
!---------------------------------------------------------------------
!
!   This program is the main program of 2D depth-integrated,hydrostatic and
!    non-hydrostatic dispersive wave model with a semi-implicit, finite
!   -difference method utilizing an upwind flux approximation and
!   momentum conserved advection.
!
!   The numerical formulation of non-hydrostatic model is based on
!   the Technical Report of Kowalik (2006).
!
!   This version impliments the momentum conserved advection
!   approximation with upwind flux scheme of Yamazaki et al. (2008)
!   to handle flow discontinuities, associated with breaking waves
!   and hydraulic jumps.

!   I the present program the calls for  subroutines
!   for non-hydrostatic computations are commented out. Only hydrostatic
!   component is computed. The main purpose is to learn interconnection
!   of the main domain located over North Pacific with finer grid subdomains
!   located in proximity to the Crescent City.
!
!   Z.Kowalik  2006 IMS,      University of Alaska Fairbanks
!   Y.Yamazaki 2007 ORE,      University of Hawaii Manoa
!------------------------------------------------------------------------
!when restartin: be sure:
! 1.  MINI is smaller than Mfin
! 2.  Change in all_input Logical restart from false to true
! 3.  Change file  'Dump_new.bin'  to  'Dump_old.bin'  so it can
!     be read for restart by this program.

      IMPLICIT NONE
      INCLUDE "all_input_kurile"

      INTEGER:: j, k,j1,k1

!... 1min source
!      REAL, DIMENSION(1:241,1:241)::ETS1  ! surface deformation
!... 2min source
      REAL, DIMENSION(1:121,1:121)::ETS1  ! surface deformation

      REAL, DIMENSION(1:je1,1:ke1)::ETS   ! surface deformation
      INTEGER, DIMENSION(2):: IZM1,IZM2,IZM3,IZM4 ! INDEX FOR MAX VALUE
       OPEN ( UNIT=501, FILE='MAX_LOC.dat', STATUS='unknown')

      m = 0
      CELL1=0

      zo1 =0. ; zn1 = 0.
      uo1 =0. ; un1 = 0.
      vo1 =0. ; vn1 = 0.
      eto1 = 0. ; etn1 = 0
      wbo1 = 0. ; wbn1 = 0.
      wso1 = 0. ; wsn1 = 0.
      q1 = 0.

      H1  =0. ; Hb1 = 0.; D1 = 0.
      Hu1 =0. ; Hv1 = 0.

      zmax1 = -1.

      m2=0
      CELL2=0

      zo2 =0. ; zn2 = 0.
      uo2 =0. ; un2 = 0.
      vo2 =0. ; vn2 = 0.
      eto2 = 0. ; etn2 = 0
      wbo2 = 0. ; wbn2 = 0.
      wso2 = 0. ; wsn2 = 0.
      q2 = 0.
      zo2(4,4)=0.01
      zn2(4,4)=0.01

      H2  =0. ; Hb2 = 0.; D2 = 0.
      Hu2 =0. ; Hv2 = 0.

      zmax2 = -1.

      m3=0
      CELL3=0

      zo3 =0. ; zn3 = 0.
      uo3 =0. ; un3 = 0.
      vo3 =0. ; vn3 = 0.
      eto3 = 0. ; etn3 = 0
      wbo3 = 0. ; wbn3 = 0.
      wso3 = 0. ; wsn3 = 0.

      H3  =0. ; Hb3 = 0.; D3 = 0.
      Hu3 =0. ; Hv3 = 0.
        zo3(4,4)= 0.01
        zn3(4,4)=0.01
      zmax3 = -1.

      m4=0
      CELL4=0

      zo4 =0. ; zn4 = 0.
      uo4 =0. ; un4 = 0.
      vo4 =0. ; vn4 = 0.
      eto4 = 0. ; etn4 = 0
      wbo4 = 0. ; wbn4 = 0.
      wso4 = 0. ; wsn4 = 0.

      H4  =0. ; Hb4 = 0.; D4 = 0.
      Hu4 =0. ; Hv4 = 0.
       zo4(4,4)=0.01
       zn4(4,4)=0.01
      zmax4 = -1.

!... OUTPUT file names
!------------------------------------------------------------------------
      CALL OPEN_OUTPUT_FILE_NAME

       
!... water level station x y coordinates
!------------------------------------------------------------------------
!      CALL OPEN_WATER_LEVEL_STATIONS
!      CALL OPEN_WATER_LEVEL_STATIONS_UAF
      CALL OPEN_WATER_LEVEL_STATIONS_AVERAGED

!... Bathymetry SETUP
!------------------------------------------------------------------------
      CALL INPUT_BATHYMETRY

      PRINT *, 'js1=',js1,'je1=',je1
      PRINT *, 'ks1=',ks1,'ke1=',ke1

!... Cal. of paramters for spherical/cartesian coordinate grid
!------------------------------------------------------------------------
      CALL TRIGON ( siz1, csz1, siv1, csv1, ks1, ke1, dlat1, latB1 )
      CALL TRIGON ( siz2, csz2, siv2, csv2, ks2, ke2, dlat2, latB2 )
      CALL TRIGON ( siz3, csz3, siv3, csv3, ks3, ke3, dlat3, latB3 )
      CALL TRIGON ( siz4, csz4, siv4, csv4, ks4, ke4, dlat4, latB4 )


!... Initial condition TYPE 1 (T=0 initiation)
!------------------------------------------------------------------------------

!!      open(unit=27,file='news2_1min.dat',status='unknown') ! 1min
!      open(unit=27,file='source_2min.dat',status='unknown')  ! 2min
!      do k=1,KE1
!        do j=1,je1
!          ETS(J,K)=0.
!        END DO
!      END DO

!-----------------------------------------
!... 1min source
!      do k1=1, 241
!        READ(27,*)(ETS1(J1,K1),J1=1,241)
!      END DO
!
!      do k1=1,241
!        K=K1+1619
!        DO J1=1,241
!          J=1919+J1
!          ETS(J,K)=ETS1(J1,K1)
!        END DO
!      END DO
!-----------------------------------------

!-----------------------------------------
!... 2min source
!      do k1=1, 121
!        READ(27,*)(ETS1(J1,K1),J1=1,121)
!      END DO
!
!      do k1=1,121
!        K=K1+810    ! K=K1+990      !
!        DO J1=1,121
!        J=960+J1     ! J=1095+J1
!          ETS(J,K)=ETS1(J1,K1)
!        END DO
!      END DO
!-----------------------------------------

!      DO K=1,KE1
!        DO J=1,JE1
!          IF(H1(J,K).LE.0.) ETS(J,K)=0.            ! Islands no down
!          zo1(J,K) = ETS(J,K)
!        END DO
!      END DO


! Initial condition TYPE 1 (T=0 initiation)
!---------------------------------------------------------------------
! (1). static surface deformation
      CALL SURFACE_OKADA_REC_1 ( z01_1,zn1,H1,dx1,dy1,csz1,lonL1,latB1,  &
                                 dlon1,dlat1,js1,je1,ks1,ke1,n1smooth )

!      CALL SURFACE_OKADA_REC_2 ( z01_2,zn1,H1,dx1,dy1,csz1,lonL1,latB1,  &
!                                 dlon1,dlat1,js1,je1,ks1,ke1,n1smooth )

!      CALL SURFACE_OKADA_REC_3 ( z01_3,zn1,H1,dx1,dy1,csz1,lonL1,latB1,  &
!                                 dlon1,dlat1,js1,je1,ks1,ke1,n1smooth )
      DO k =ks1,ke1
        DO j=js1,je1
          z01(j,k) = z01_1(j,k) !+z01_2(j,k) +z01_3(j,k)
        END DO
      END DO

      OPEN ( UNIT=151, FILE='z01.dat', FORM='unformatted', STATUS='unknown')
!      WRITE (151) ( (z01(j,k), j=js1,je1, inc_out1 ), k=ks1,ke1, inc_out1 )
      WRITE (151) ( (z01(j,k), j=js1,je1), k=ks1,ke1)

      zo1 = z01
      zn1 = zo1

! (2). static surface deformation obtained from the outer grid's z0

!      CALL DC_INTERP_z0 ( z02,z01,inc12x,inc12y,js1,je1,ks1,ke1,     &
!                          js2,je2,ks2,ke2,js2o,je2o,ks2o,ke2o,jnum2o )

!      CALL DC_INTERP_z0 ( z03,z02,inc23x,inc23y,js2,je2,ks2,ke2,     &
!                          js3,je3,ks3,ke3,js3o,je3o,ks3o,ke3o,jnum3o )

!      CALL DC_INTERP_z0 ( z04,z03,inc34x,inc34y,js3,je3,ks3,ke3,     &
!                          js4,je4,ks4,ke4,js4o,je4o,ks4o,ke4o,jnum4o )

! (2). initial calculation for bottom deformation
!      CALL INIT_BOTTOM_OKADA_REC_VERT( disx1,disy1,disz1,mrise1,dt1, &
!                                       dx1,dy1,csz1,lonL1,latB1,     &
!                                       js1,je1,ks1,ke1 )

!      OPEN ( UNIT=151, FILE='z01.dat',   FORM='unformatted', STATUS='unknown')
!      OPEN ( UNIT=251, FILE='z02.dat',   FORM='unformatted', STATUS='unknown')
!      WRITE (151) ( ( zo1(j,k) ,j=js1,je1 ), k=ks1,ke1 )
!      WRITE (251) ( ( zo2(j,k) ,j=js2,je2 ), k=ks2,ke2 )

!.. H at u v cal points
!------------------------------------------------------------------------
      CALL Hu_Hv_CAL ( Hu1, Hv1, H1, js1, je1, ks1, ke1 )
      CALL Hu_Hv_CAL ( Hu2, Hv2, H2, js2, je2, ks2, ke2 )
      CALL Hu_Hv_CAL ( Hu3, Hv3, H3, js3, je3, ks3, ke3 )
      CALL Hu_Hv_CAL ( Hu4, Hv4, H4, js4, je4, ks4, ke4 )

      DO k=ks1,ke1
        DO j=js1,je1
          IF( H1(j,k) .LT. EPSILON ) THEN
            zo1(j,k)  = -H1(j,k)
          END IF
        END DO
      END DO
      zn1=zo1
      DO k=ks2,ke2
        DO j=js2,je2
          IF( H2(j,k) .LT. EPSILON ) THEN
            zo2(j,k)  = -H2(j,k)
          END IF
        END DO
      END DO
      zn2=zo2

      DO k=ks3,ke3
        DO j=js3,je3
          IF( H3(j,k) .LT. EPSILON ) THEN
            zo3(j,k)  = -H3(j,k)
          END IF
        END DO
      END DO
      zn3=zo3

      DO k=ks4,ke4
        DO j=js4,je4
          IF( H4(j,k) .LT. EPSILON ) THEN
            zo4(j,k)  = -H4(j,k)
          END IF
        END DO
      END DO
      zn4=zo4
!------------------------------------------------------------------------
! RESTART     Made by Juan Horrillo, SFOS, Fairbanks, AK
!------------------------------------------------------------------------
mINI=1;
IF (RESTART) THEN
   OPEN(UNIT=1001,FILE='Dump_old.bin',STATUS='unknown',FORM='unformatted', &
        ACTION='read')
   READ(1001)mINI,zo1,uo1,vo1,wso1,wbo1,eto1, &
                  zo2,uo2,vo2,wso2,wbo2,eto2, &
                  zo3,uo3,vo3,wso3,wbo3,eto3, &
                  zo4,uo4,vo4,wso4,wbo4,eto4, &
                  zmax1,zmax2,zmax3,zmax4
   PRINT*,'Reading Dump cycle',mINI
   CLOSE(1001)
END IF
!------------------------------------------------------------------------
! START TIME LOOP TIDE
!------------------------------------------------------------------------
!    TIMELOOP1: DO m=mINI, mFIN1   !TIME LOOP
!                  DO m2I = 1,inc12t
!                     DO m3I = 1,inc23t
!!                        CALL TIDE_GENERATION()
!                        CALL DOUBLE_COUPLE_D34()
!                     END DO
!                  END DO
!               END  DO  TIMELOOP1
!------------------------------------------------------------------------
! END TIME LOOP TIDE
!------------------------------------------------------------------------
!    mINI=mFIN1+1


!------------------------------------------------------------------------
! START TIME LOOP
!------------------------------------------------------------------------
      TIMELOOP: DO m=mINI, mFIN   !TIME LOOP

!... OPEN BC implimentation for 1st layer
!------------------------------------------------------------------------
      CALL OPEN_BC_LR
      CALL OPEN_BC_BT

!... Initial Condition TYPE 2 (time-series initial condition)
!------------------------------------------------------------------------
  ! Vertical Sea-bottom deformation with rise time Tr


!... D: Total water depth cal.
!------------------------------------------------------------------------
!   Land Height z Cal.,
!   and Wet/Dry CELL Determinations
!   (NOTE: CELL is initially set as DRY (=0) !!
      DO k=ks1,ke1
        DO j=js1,je1
          Hb1(j,k) = H1(j,k) -eto1(j,k)
          D1(j,k) = zo1(j,k) +Hb1(j,k)
          IF( D1(j,k) .LT. EPSILON ) THEN
            zo1(j,k)  = -H1(j,k) +eto1(j,k)
            CELL1(j,k) = 0.
          ELSE
         !ELSE IF ( D1(j,k) .GE. EPSILON ) THEN
            CELL1(j,k) = 1.
          END IF
        END DO
      END DO


!... RUNUP
!------------------------------------------------------------------------
      CALL RUNUP ( CELL1, D1, uo1, vo1,  &
                   zo1, H1, eto1,        &
                   js1, je1, ks1, ke1  )
     !CALL RUNUP_Kowalik ( CELL1, D1, uo1, vo1,  &
     !                     zo1, H1, eto1,        &
     !                     js1, je1, ks1, ke1  )
      ! u and v treatment at wet/dry interface
      CALL WET_DRY_UV_BC ( uo1, vo1,               &
                           zo1, H1, eto1, CELL1,   &
                           js1, je1, ks1, ke1    )
!... save initila OUTPUT (T=0)
!------------------------------------------------------------------------------
      CALL OUTPUT_INIT_TIME_FRAME
      IF ( m .EQ. 1 ) THEN
        CALL OUTPUT_WATER_LEVEL_AVERAGED_INIT_GRID_1
      END IF

!------------------------------------------------------------------------
!  MAIN SOLVER
!  Momentum and Continuity Equations
!------------------------------------------------------------------------

!... wb:Bottom Vertical Velocity calculation using linearized z momentum equation
!      CALL VERT_BOTTOM (wbn1,un1,vn1,eto1,etn1,Hb1,CELL1,csz1,dx1,    &
!                        dy1,dt1,js1,je1,ks1,ke1)

!... 1ST u,v: 1st half-step horizontal velocities cal. using explicit momentum equations

      CALL Dlim_Du_Dv_CAL (Dlim1,Du1,Dv1,zo1,eto1,H1,D1,Hu1,Hv1,js1,  &
                           je1,ks1,ke1)

      CALL XMOMENT_1ST (un1,zo1,uo1,vo1,eto1,H1,Hu1,Du1,CELL1,siz1,   &
                        csz1,FCT1,NL1,dt1,dt_dx1,dt_dy1,gdt_dx1,js1,  &
                        je1, ks1, ke1)

      CALL YMOMENT_1ST (vn1,zo1,un1,vo1,eto1,H1,Hv1,Dv1,CELL1,siv1,   &
                        csv1,FCT1,NL1,dt1,dt_dx1,dt_dy1,gdt_dy1,js1,  &
                        je1,ks1,ke1)

!... q: Non-hydrostatic pressure cal. using linear algorithm solvers
!      CALL NON_HYDRO_SIP_PIVOT ( q1,An1,Bn1,zo1,uo1,vo1,eto1,wbo1,   &
!                                 wbn1,wso1,H1,Dlim1,CELL1,csz1,csv1, &
!                                 dt1,dx1,dy1,js1,je1,ks1,ke1,NJK1)

!... ws: Sea surface velocity cal. using q
!   (comment out for hydrostatic solution)
!      CALL VERT_SURFACE (wsn1,q1,wso1,wbo1,wbn1,Dlim1,CELL1,dt1,js1,  &
!                         je1,ks1,ke1)

!... correction from Q
!... 2ND u,v: 2nd half-step horizontal velocities cal. using q
!   (comment out for hydrostatic solution)
!      CALL XMOMENT_2ND (un1,q1,An1,zo1,eto1,H1,Dlim1,CELL1,csz1,      &
!                        dt_dx1,js1,je1,ks1,ke1)
!      CALL SLIP_BC_U_BT

!      CALL YMOMENT_2ND (vn1,q1,Bn1,zo1,eto1,H1,Dlim1,CELL1,           &
!                        dt_dy1,js1,je1,ks1,ke1)
!      CALL SLIP_BC_V_BT
!... z(zeta): Sea Level cal. using continuity equation
      CALL CONT ( zn1,                          &
                  zo1, etn1, eto1, un1, vn1,    &
                  Hu1, Hv1, CELL1, csz1, csv1,  &
                  dt1, dt_dx1, dt_dy1,          &
                  js1, je1, ks1, ke1          )

!------------------------------------------------------------------------------
! DOUBLE-COUPLE 1ST AND 2ND LAYERS
!        PRINT*,'INTEGRATION STEP,M=',M
    IF(M.GT.m0DC)THEN
     CALL  DOUBLE_COUPLE_D12
!     CALL  DOUBLE_COUPLE_D23 ! moved to  DOUBLE_COUPLE_D12
!     CALL  DOUBLE_COUPLE_D34
     END IF
!------------------------------------------------------------------------------

!... UPDATE variables
!------------------------------------------------------------------------------
      zo1  = zn1
      uo1  = un1
      vo1  = vn1
      wbo1 = wbn1
      wso1 = wsn1
      eto1 = etn1

!... save time-series OUTPUT
!------------------------------------------------------------------------------

!     CALL OUTPUT_TIME_FRAME   !do not call initially
!      CALL OUTPUT_WATER_LEVEL_STATION  ! water level records
!      CALL OUTPUT_WATER_LEVEL_STATION_AVERAGED
      CALL OUTPUT_WATER_LEVEL_AVERAGED_GRID_1

!... AUTO program termination setup
!---------------------------------------------------------------
      DO j=js1,je1
        DO k=ks1,ke1
          IF ( ABS(zo1(j,k))  .GT. 80000       &
                              .AND.            &
                   CELL1(j,k) .EQ. 1     )THEN
          !If ABS(zo(j,k)>800m
            PRINT*, j, k, zo1(j,k)
            GO TO 1004  ! end calculation
          END IF
        END DO
      END DO

!... MAXIMUM zeta search, zmax(j,k)
!---------------------------------------------------------------
      CALL ZMAX_SEARCH ( zmax1, z_max1, jmax1, kmax1,      &
                          zo1, CELL1, js1, je1, ks1, ke1 )
      CALL ZMAX_SEARCH ( zmax2, z_max2, jmax2, kmax2,      &
                          zo2, CELL2, js2, je2, ks2, ke2 )
      CALL ZMAX_SEARCH ( zmax3, z_max3, jmax3, kmax3,      &
                          zo3, CELL3, js3, je3, ks3, ke3 )
      CALL ZMAX_SEARCH ( zmax4, z_max4, jmax4, kmax4,      &
                          zo4, CELL4, js4, je4, ks4, ke4 )


      IF ( MOD(m,mout1) .EQ. 0 ) THEN
       PRINT *,m*dt1     , '(s), zo1(154,31)=',zo1(154,31)  ! sec count
           PRINT*,"MAXVALUE= ", MAXVAL(zO1,MASK=(CELL1.eq.1)),&
         "TIME STEP= ",M
         PRINT*,"MAX_LOCATION= ", MAXLOC(zO1,MASK=(CELL1.eq.1))

         IZM1 = MAXLOC(zO1,MASK=(CELL1.eq.1))
         PRINT*,"DEPTH_AT_MAX_LOCATION(DOMAIN 1)= ", H1( IZM1(1), IZM1(2))
!=========================================================================
  PRINT*, "MAXVALUE IN DOMAIN2=", MAXVAL(zO2,MASK=(CELL2.eq.1))
          IF(MAXVAL(zO2,MASK=(CELL2.eq.1)).LT.0.0) GO TO 30
          PRINT*,"MAX_LOCATION= ", MAXLOC(zO2,MASK=(CELL2.eq.1))
           IZM2 = MAXLOC(zO2,MASK=(CELL2.eq.1))

         PRINT*,"DEPTH_AT_MAX_LOCATION(DOMAIN 2)= ", H2( IZM2(1), IZM2(2))
!==========================================================================
  30       PRINT*, "MAXVALUE IN DOMAIN3=", MAXVAL(zO3,MASK=(CELL3.eq.1))
            IF(MAXVAL(zO3,MASK=(CELL3.eq.1)).LT.0.0) GO TO 31
          PRINT*,"MAX_LOCATION= ", MAXLOC(zO3,MASK=(CELL3.eq.1))
           IZM3 = MAXLOC(zO3,MASK=(CELL3.eq.1))
         PRINT*,"DEPTH_AT_MAX_LOCATION(DOMAIN 3)= ", H3( IZM3(1), IZM3(2))
!==========================================================================

31           PRINT*, "MAXVALUE IN DOMAIN4=", MAXVAL(zO4,MASK=(CELL4.eq.1))
             IF(MAXVAL(zO4,MASK=(CELL4.eq.1)).LT.0.0) GO TO 32
          PRINT*,"MAX_LOCATION= ", MAXLOC(zO4,MASK=(CELL4.eq.1))
           IZM4 = MAXLOC(zO4,MASK=(CELL4.eq.1))
         PRINT*,"DEPTH_AT_MAX_LOCATION(DOMAIN 4)= ", H4( IZM4(1), IZM4(2))
     END if
32   CONTINUE
     IF(M.GT.m0DC)THEN
     IF ( MOD(m,10) .EQ. 0 ) THEN
        WRITE(501,*)"TIME STEP=",M, "MAXVALUE IN DOMAIN3=", MAXVAL(zO3,MASK=(CELL3.eq.1))
          IF(MAXVAL(zO3,MASK=(CELL3.eq.1)).LT.0.0) GO TO 40
        WRITE(501,*)  "MAX_LOCATION= ", MAXLOC(zO3,MASK=(CELL3.eq.1))
          IZM3 = MAXLOC(zO3,MASK=(CELL3.eq.1))
        WRITE(501,*) "DEPTH_AT_MAX_LOCATION(DOMAIN 3)= ", H3( IZM3(1), IZM3(2))
!==========================================================================

40         WRITE(501,*) "MAXVALUE IN DOMAIN4=", MAXVAL(zO4,MASK=(CELL4.eq.1))
             IF(MAXVAL(zO4,MASK=(CELL4.eq.1)).LT.0.0) GO TO 41
         WRITE(501,*) "MAX_LOCATION= ", MAXLOC(zO4,MASK=(CELL4.eq.1))
           IZM4 = MAXLOC(zO4,MASK=(CELL4.eq.1))
          WRITE(501,*)"DEPTH_AT_MAX_LOCATION(DOMAIN 4)= ", H4( IZM4(1), IZM4(2))
 41     CONTINUE
    END IF

    END IF





      END  DO  TIMELOOP
!------------------------------------------------------------------------
! END TIME LOOP
!------------------------------------------------------------------------
 1004 CONTINUE

      CALL OUTPUT_ZMAX1
      CALL OUTPUT_ZMAX2
      CALL OUTPUT_ZMAX3
      CALL OUTPUT_ZMAX4
!------------------------------------------------------------------------
! DUMP
!------------------------------------------------------------------------
GO TO 666
   OPEN(UNIT=1011,FILE='Dump_new.bin',STATUS='unknown',FORM='unformatted', &
        ACTION='write')
   WRITE(1011)m,zo1,uo1,vo1,wso1,wbo1,eto1, &
                zo2,uo2,vo2,wso2,wbo2,eto2, &
                zo3,uo3,vo3,wso3,wbo3,eto3, &
                zo4,uo4,vo4,wso4,wbo4,eto4, &
                zmax1,zmax2,zmax3,zmax4
   PRINT*,'Writing Dump cycle',m
   CLOSE(1011)
666 continue
! END PROGRAM NEOWAVE2D
!------------------------------------------------------------------------
! END OF MAIN PROGRAM
!------------------------------------------------------------------------


CONTAINS
!------------------------------------------------------------------------
!------------------------------------------------------------------------
!                              SUBROUTINES
!------------------------------------------------------------------------
!------------------------------------------------------------------------

! double coupling subroutines
!-----------------------------------------------------------------------
SUBROUTINE  DOUBLE_COUPLE_D12
!------------------------------------------------------------------------
!          Double-coupled solver for 1st and 2nd layers
!------------------------------------------------------------------------
!
!   This program double-couples the 1st and 2nd domains through transfer
!   information between 2 domains.
!
!   Y.Yamazaki 2008 ORE      University of Hawaii Manoa
!------------------------------------------------------------------------
      !USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE

!  (INTERNAL)...
      INTEGER:: j, k, m2I !ms2 ! (m2I=ms2)   m2


!======================================================================
!... interpolate variables along Level-2 grid's boundaries from Leve-1 grid's variables
!------------------------------------------------------------------------

      m2I = 1
     !smallstepd12: DO MS2=1,INT12
      smallstepd12: DO m2I = 1,inc12t ! TIME LOOP with dt2
        m2 = (m-1)*inc12t +m2I

!        CALL BOUND_INTERP_zo_D12 (MS2)
        CALL DC_INTERP ( zn2,zo1,zn1,m2I,inc12x,inc12y,    &
                         inc12t,js1,je1,ks1,ke1,js2,je2,ks2,ke2,   &
                         js2o,je2o,ks2o,ke2o,jnum2o )

!        CALL DC_INTERP_q ( q21,qo1,q1,m2I,inc12x,inc12y,inc12t,    &
!                           js1,je1,ks1,ke1,js2,je2,ks2,ke2,        &
!                           js2o,je2o,ks2o,ke2o,jnum2o )

        CALL DC_BOUND_INTERP_u_LR ( un2,uo1,un1,m2I,inc12x,inc12y,   &
                                    inc12t,js1,je1,ks1,ke1,js2,je2,  &
                                    ks2,ke2,js2o,je2o,ks2o,ke2o )
        CALL DC_BOUND_INTERP_v_BT ( vn2,vo1,vn1,m2I,inc12x,inc12y,   &
                                    inc12t,js1,je1,ks1,ke1,js2,je2,  &
                                    ks2,ke2,js2o,je2o,ks2o,ke2o )
        CALL DC_BOUND_INTERP_u_BT ( un2,uo1,un1,m2I,inc12x,inc12y,   &
                                    inc12t,js1,je1,ks1,ke1,js2,je2,  &
                                    ks2,ke2,js2o,je2o,ks2o,ke2o )
        CALL DC_BOUND_INTERP_v_LR ( vn2,vo1,vn1,m2I,inc12x,inc12y,   &
                                    inc12t,js1,je1,ks1,ke1,js2,je2,  &
                                    ks2,ke2,js2o,je2o,ks2o,ke2o )

         un21( js2+1, ks2+1:ke2 ) = un2( js2+1, ks2+1:ke2 )
         un21(  je2 , ks2+1:ke2 ) = un2(  je2 , ks2+1:ke2 )
         un21( js2+1:je2, ks2 )   = un2( js2+1:je2, ks2)
         un21( js2+1:je2, ke2 )   = un2( js2+1:je2, ke2)

         vn21( js2, ks2:ke2-1 ) = vn2( js2, ks2:ke2-1 )
         vn21( je2, ks2:ke2-1 ) = vn2( je2, ks2:ke2-1 )
         vn21( js2:je2,  ks2  ) = vn2( js2:je2,  ks2  )
         vn21( js2:je2, ke2-1 ) = vn2( js2:je2, ke2-1 )
!======================================================================

 !      PRINT*,' IN DOUBLE COUPLE MS2=',MS2,'int12=', INT12
!... Initial Condition TYPE 2 (time-series initial condition)
!------------------------------------------------------------------------
  ! Vertical Sea-bottom deformation with rise time Tr


!... D: Total water depth cal.
!------------------------------------------------------------------------
!   Land Height z Cal.,
!   and Wet/Dry CELL Determinations
!   (NOTE: CELL is initially set as DRY (=0) !!
      DO k=ks2,ke2
        DO j=js2,je2
          Hb2(j,k) = H2(j,k) -eto2(j,k)
          D2(j,k) = zo2(j,k) +Hb2(j,k)
          IF( D2(j,k) .LT. EPSILON ) THEN
            zo2(j,k)  = -H2(j,k) +eto2(j,k)
            CELL2(j,k) = 0.
          ELSE
         !ELSE IF ( D2(j,k) .GE. EPSILON ) THEN
            CELL2(j,k) = 1.
          END IF
        END DO
      END DO

!... RUNUP
!------------------------------------------------------------------------
!======================================================================
        CALL RUNUP ( CELL2,D2,uo2,vo2,zo2,H2,eto2,js2,je2,ks2,ke2 )        ! This is new RUNUP
!======================================================================
       !CALL RUNUP_Kowalik (CELL2,D2,uo2,vo2,zo2,H2,eto2,js2,je2,ks2,ke2) ! original Kowalik 2008 RUNUP

!======================================================================
! correct H1 into H2 at Dry Cell obtained from zn1 !
! NOTE: Not sure if this is nessesary for Partial Radiation B.C.
        DO k=ks2,ke2
          DO j=js2,je2
            IF( CELL2(j,k) .EQ. 0  ) THEN
              zn2(j,k)  = -H2(j,k) +etn2(j,k)
            END IF
          END DO
        END DO
!======================================================================

      ! u and v treatment at wet/dry interface
      CALL WET_DRY_UV_BC (uo2,vo2,zo2,H2,eto2,CELL2,js2,je2,ks2,ke2)

!... save initial OUTPUT (T=0)
!---------------------------------------------------------------------
        IF ( m2 .EQ. m20DC ) THEN
          CALL OUTPUT_WATER_LEVEL_AVERAGED_INIT_GRID_2
        END IF


!------------------------------------------------------------------------
!  MAIN SOLVER
!  Momentum and Continuity Equations
!------------------------------------------------------------------------
!... wb
!      CALL VERT_BOTTOM (wbn2,un2,vn2,eto2,etn2,Hb2,CELL2,csz2,dx2,    &
!                        dy2,dt2,js2,je2,ks2,ke2)

      CALL Dlim_Du_Dv_CAL (Dlim2,Du2,Dv2,zo2,eto2,H2,D2,Hu2,Hv2,js2,  &
                           je2,ks2,ke2)

!... 1ST u,v
      CALL XMOMENT_1ST (un2,zo2,uo2,vo2,eto2,H2,Hu2,Du2,   &
                        CELL2,siz2,csz2,FCT2,NL2,          &
                        dt2,dt_dx2,dt_dy2,gdt_dx2,         &
                        js2,je2,ks2,ke2                  )
      CALL YMOMENT_1ST (vn2,zo2,un2,vo2,eto2,H2,Hv2,Dv2,   &
                        CELL2,siv2,csv2,FCT2,NL2,          &
                        dt2,dt_dx2,dt_dy2,gdt_dy2,         &
                        js2,je2,ks2,ke2                  )

!... q
!======================================================================
!      CALL NON_HYDRO_SIP_PIVOT_DC ( q2,q21,An2,Bn2,zo2,uo2,vo2,     &
!                                    eto2,wbo2,wbn2,wso2,H2,Dlim2,   &
!                                    CELL2,csz2,csv2,dt2,dx2,dy2,    &
!                                    js2,je2,ks2,ke2,NJK2,m2,dm_out2)
!======================================================================
!... ws
!      CALL VERT_SURFACE (wsn2,q2,wso2,wbo2,wbn2,Dlim2,CELL2,dt2,js2,  &
!                         je2,ks2,ke2)


!... 2ND u,v  correction from Q
!      CALL XMOMENT_2ND (un2,q2,An2,zo2,eto2,H2,Dlim2,CELL2,csz2,      &
!                           dt_dx2,js2,je2,ks2,ke2)
!      CALL YMOMENT_2ND (vn2,q2,Bn2,zo2,eto2,H2,Dlim2,CELL2,           &
!                           dt_dy2,js2,je2,ks2,ke2)


      CALL CONT (zn2,zo2,etn2,eto2,un2,vn2,Hu2,Hv2,CELL2,    &
                 csz2,csv2,dt2,dt_dx2,dt_dy2,js2,je2,ks2,ke2)

      CALL DC_BOUND_SHORELINE_z ( zn1,zn2,H1,H2,Hb2,etn2,D2,   &
                                  CELL1,CELL2,js1,je1,ks1,ke1, &
                                  js2,je2,ks2,ke2,js2o,je2o,   &
                                  ks2o,ke2o,inc12x,inc12y,m2 )

      zn21( js2, ks2:ke2 ) = zn2( js2, ks2:ke2 )
      zn21( je2, ks2:ke2 ) = zn2( je2, ks2:ke2 )
      zn21( js2:je2, ks2 ) = zn2( js2:je2, ks2 )
      zn21( js2:je2, ke2 ) = zn2( js2:je2, ke2 )

      CALL DC_BOUND_RADIATION ( zn2,un2,vn2,zn21,un21,vn21,CELL2,   &
                                H2,Hu2,Hv2,etn2,js2,je2,ks2,ke2 )


! OUTPUT (FEEDBACK) of Double-Coupling
!---------------------------------------------------------------------
!  extract zn1 from zn2
!        IF( m2I.EQ.inc12t)THEN
!          CALL DC_EXTRACT_zn ( zn1,zn2,H1,H2,CELL1,CELL2,       &
!                               js1,je1,ks1,ke1,js2,je2,ks2,ke2, &
!                               js2o,ks2o,inc12x,inc12y,m2 )
!
!        END IF

!---------------------------------------------------------------------
! Double-Couples 2nd and 3rd Grids
!---------------------------------------------------------------------
     CALL  DOUBLE_COUPLE_D23
!---------------------------------------------------------------------

!... UPDATE variables
!------------------------------------------------------------------------------
      zo2  = zn2
      uo2  = un2
      vo2  = vn2
      wbo2 = wbn2
      wso2 = wsn2
      eto2 = etn2

!... fill zn2 along domain boundary using zn1
!    This is not nesessary for computation. only add zn2 along the 2nd
!    grid boundary (but not good, need more TEST)
!      CALL BOUND_INTERP_zo_D12(MS2)

! save time-series OUTPUT
!---------------------------------------------------------------------
      CALL OUTPUT_WATER_LEVEL_AVERAGED_GRID_2


      END  DO  smallstepd12

      RETURN

END SUBROUTINE

!-----------------------------------------------------------------------
SUBROUTINE  DOUBLE_COUPLE_D23
!------------------------------------------------------------------------
!          Double-coupled solver for 2nd and 3rd layers
!------------------------------------------------------------------------
!
!   This program double-couples the 2nd and 3rd domains through transfer
!   information between 2 domains.
!
!   Y.Yamazaki 2008 ORE      University of Hawaii Manoa
!------------------------------------------------------------------------
      !USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE

!  (INTERNAL)...
      INTEGER:: j, k,m3I !ms3 ! (m3I=ms3)


!======================================================================
!... interpolate variables along Level-3 grid's boundaries from Leve-2 grid's variables
!------------------------------------------------------------------------

      m3I = 1
     !smallstepd23: DO MS3=1,INT23
      smallstepd23: DO m3I = 1,inc23t ! TIME LOOP with dt3
        m3 = (m2-1)*inc23t +m3I

!        CALL BOUND_INTERP_zo_D23 (MS3)
        CALL DC_INTERP ( zn3,zo2,zn2,m3I,inc23x,inc23y,    &
                         inc23t,js2,je2,ks2,ke2,js3,je3,ks3,ke3,   &
                         js3o,je3o,ks3o,ke3o,jnum3o )

!        CALL DC_INTERP_q ( q32,qo2,q2,m3I,inc23x,inc23y,inc23t,    &
!                           js2,je2,ks2,ke2,js3,je3,ks3,ke3,        &
!                           js3o,je3o,ks3o,ke3o,jnum3o )

        CALL DC_BOUND_INTERP_u_LR ( un3,uo2,un2,m3I,inc23x,inc23y,   &
                                    inc23t,js2,je2,ks2,ke2,js3,je3,  &
                                    ks3,ke3,js3o,je3o,ks3o,ke3o )
        CALL DC_BOUND_INTERP_v_BT ( vn3,vo2,vn2,m3I,inc23x,inc23y,   &
                                    inc23t,js2,je2,ks2,ke2,js3,je3,  &
                                    ks3,ke3,js3o,je3o,ks3o,ke3o )
        CALL DC_BOUND_INTERP_u_BT ( un3,uo2,un2,m3I,inc23x,inc23y,   &
                                    inc23t,js2,je2,ks2,ke2,js3,je3,  &
                                    ks3,ke3,js3o,je3o,ks3o,ke3o )
        CALL DC_BOUND_INTERP_v_LR ( vn3,vo2,vn2,m3I,inc23x,inc23y,   &
                                    inc23t,js2,je2,ks2,ke2,js3,je3,  &
                                    ks3,ke3,js3o,je3o,ks3o,ke3o )

         un32( js3+1, ks3+1:ke3 ) = un3( js3+1, ks3+1:ke3 )
         un32(  je3 , ks3+1:ke3 ) = un3(  je3 , ks3+1:ke3 )
         un32( js3+1:je3, ks3 )   = un3( js3+1:je3, ks3)
         un32( js3+1:je3, ke3 )   = un3( js3+1:je3, ke3)

         vn32( js3, ks3:ke3-1 ) = vn3( js3, ks3:ke3-1 )
         vn32( je3, ks3:ke3-1 ) = vn3( je3, ks3:ke3-1 )
         vn32( js3:je3,  ks3  ) = vn3( js3:je3,  ks3  )
         vn32( js3:je3, ke3-1 ) = vn3( js3:je3, ke3-1 )
!======================================================================

 !      PRINT*,' IN DOUBLE COUPLE MS3=',MS3,'int23=', INT23
!... Initial Condition TYPE 2 (time-series initial condition)
!------------------------------------------------------------------------
  ! Vertical Sea-bottom deformation with rise time Tr


!... D: Total water depth cal.
!------------------------------------------------------------------------
!   Land Height z Cal.,
!   and Wet/Dry CELL Determinations
!   (NOTE: CELL is initially set as DRY (=0) !!
      DO k=ks3,ke3
        DO j=js3,je3
          Hb3(j,k) = H3(j,k) -eto3(j,k)
          D3(j,k) = zo3(j,k) +Hb3(j,k)
          IF( D3(j,k) .LT. EPSILON ) THEN
            zo3(j,k)  = -H3(j,k) +eto3(j,k)
            CELL3(j,k) = 0.
          ELSE
         !ELSE IF ( D3(j,k) .GE. EPSILON ) THEN
            CELL3(j,k) = 1.
          END IF
        END DO
      END DO

!... RUNUP
!------------------------------------------------------------------------
!======================================================================
        CALL RUNUP ( CELL3,D3,uo3,vo3,zo3,H3,eto3,js3,je3,ks3,ke3 ) ! This is new RUNUP
!======================================================================
       !CALL RUNUP_Kowalik (CELL3,D3,uo3,vo3,zo3,H3,eto3,js3,je3,ks3,ke3) ! original RUNUP

!======================================================================
! correct H2 into H3 at Dry Cell obtained from zn2 !
! NOTE: Not sure if this is nessesary for Partial Radiation B.C.
        DO k=ks3,ke3
          DO j=js3,je3
            IF( CELL3(j,k) .EQ. 0  ) THEN
              zn3(j,k)  = -H3(j,k) +etn3(j,k)
            END IF
          END DO
        END DO
!======================================================================

      ! u and v treatment at wet/dry interface
      CALL WET_DRY_UV_BC (uo3,vo3,zo3,H3,eto3,CELL3,js3,je3,ks3,ke3)

! save initial OUTPUT (T=0)
!---------------------------------------------------------------------
        IF ( m3 .EQ. m30DC ) THEN
          CALL OUTPUT_WATER_LEVEL_AVERAGED_INIT_GRID_3
        END IF



!------------------------------------------------------------------------
!  MAIN SOLVER
!  Momentum and Continuity Equations
!------------------------------------------------------------------------
!... wb
!      CALL VERT_BOTTOM (wbn3,un3,vn3,eto3,etn3,Hb3,CELL3,csz3,dx3,    &
!                        dy3,dt3,js3,je3,ks3,ke3)

      CALL Dlim_Du_Dv_CAL (Dlim3,Du3,Dv3,zo3,eto3,H3,D3,Hu3,Hv3,js3,  &
                           je3,ks3,ke3)

!... 1ST u,v
      CALL XMOMENT_1ST (un3,zo3,uo3,vo3,eto3,H3,Hu3,Du3,   &
                        CELL3,siz3,csz3,FCT3,NL3,          &
                        dt3,dt_dx3,dt_dy3,gdt_dx3,         &
                        js3,je3,ks3,ke3                  )
      CALL YMOMENT_1ST (vn3,zo3,un3,vo3,eto3,H3,Hv3,Dv3,   &
                        CELL3,siv3,csv3,FCT3,NL3,          &
                        dt3,dt_dx3,dt_dy3,gdt_dy3,         &
                        js3,je3,ks3,ke3                  )

!... q
!======================================================================
!      CALL NON_HYDRO_SIP_PIVOT_DC ( q3,q32,An3,Bn3,zo3,uo3,vo3,     &
!                                    eto3,wbo3,wbn3,wso3,H3,Dlim3,   &
!                                    CELL3,csz3,csv3,dt3,dx3,dy3,    &
!                                    js3,je3,ks3,ke3,NJK3,m3,dm_out3)
!======================================================================
!... ws
!      CALL VERT_SURFACE (wsn3,q3,wso3,wbo3,wbn3,Dlim3,CELL3,dt3,js3,  &
!                         je3,ks3,ke3)


!... 2ND u,v  correction from Q
!      CALL XMOMENT_2ND (un3,q3,An3,zo3,eto3,H3,Dlim3,CELL3,csz3,      &
!                           dt_dx3,js3,je3,ks3,ke3)
!      CALL YMOMENT_2ND (vn3,q3,Bn3,zo3,eto3,H3,Dlim3,CELL3,           &
!                           dt_dy3,js3,je3,ks3,ke3)


      CALL CONT (zn3,zo3,etn3,eto3,un3,vn3,Hu3,Hv3,CELL3,    &
                 csz3,csv3,dt3,dt_dx3,dt_dy3,js3,je3,ks3,ke3)

      CALL DC_BOUND_SHORELINE_z ( zn2,zn3,H2,H3,Hb3,etn3,D3,   &
                                  CELL2,CELL3,js2,je2,ks2,ke2, &
                                  js3,je3,ks3,ke3,js3o,je3o,   &
                                  ks3o,ke3o,inc23x,inc23y,m3 )

      zn32( js3, ks3:ke3 ) = zn3( js3, ks3:ke3 )
      zn32( je3, ks3:ke3 ) = zn3( je3, ks3:ke3 )
      zn32( js3:je3, ks3 ) = zn3( js3:je3, ks3 )
      zn32( js3:je3, ke3 ) = zn3( js3:je3, ke3 )
!      znt(  js3, ks3:ke3 ) = 50.*SIN(3.1416

      CALL DC_BOUND_RADIATION ( zn3,un3,vn3,zn32,un32,vn32,CELL3,   &
                                H3,Hu3,Hv3,etn3,js3,je3,ks3,ke3 )


! OUTPUT (FEEDBACK) of Double-Coupling
!---------------------------------------------------------------------
!  extract zn2 from zn3
!        IF( m3I.EQ.inc23t)THEN
!          CALL DC_EXTRACT_zn ( zn2,zn3,H2,H3,CELL2,CELL3,       &
!                               js2,je2,ks2,ke2,js3,je3,ks3,ke3, &
!                               js3o,ks3o,inc23x,inc23y,m3 )
!
!        END IF

!---------------------------------------------------------------------
! Double-Couples 3rd and 4th Grids
!---------------------------------------------------------------------
      CALL  DOUBLE_COUPLE_D34
!---------------------------------------------------------------------

!... UPDATE variables
!------------------------------------------------------------------------------
      zo3  = zn3
      uo3  = un3
      vo3  = vn3
      wbo3 = wbn3
      wso3 = wsn3
      eto3 = etn3

!... fill zn3 along domain boundary using zn2
!    This is not nesessary for computation. only add zn3 along the 3rd
!    grid boundary (but not good, need more TEST)
!      CALL BOUND_INTERP_zo_D23(MS3)

! save time-series OUTPUT
!---------------------------------------------------------------------
      CALL OUTPUT_WATER_LEVEL_AVERAGED_GRID_3


      END  DO  smallstepd23

      RETURN

END SUBROUTINE

!-----------------------------------------------------------------------
SUBROUTINE  DOUBLE_COUPLE_D34
!------------------------------------------------------------------------
!          Double-coupled solver for 3nd and 4rd layers
!------------------------------------------------------------------------
!
!   This program double-couples the 3nd and 4rd domains through transfer
!   information between 2 domains.
!
!   Y.Yamazaki 2008 ORE      University of Hawaii Manoa
!------------------------------------------------------------------------
      !USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE

!  (INTERNAL)...
      INTEGER:: j, k,m4I !ms4 ! (m4I=ms4)


!======================================================================
!... interpolate variables along Level-4 grid's boundaries from Leve-3 grid's variables
!------------------------------------------------------------------------

      m4I = 1
     !smallstepd34: DO MS4=1,INT34
      smallstepd34: DO m4I = 1,inc34t ! TIME LOOP with dt4
        m4 = (m3-1)*inc34t +m4I

!        CALL BOUND_INTERP_zo_D34 (MS4)
        CALL DC_INTERP ( zn4,zo3,zn3,m4I,inc34x,inc34y,    &
                         inc34t,js3,je3,ks3,ke3,js4,je4,ks4,ke4,   &
                         js4o,je4o,ks4o,ke4o,jnum4o )

!        CALL DC_INTERP_q ( q43,qo3,q3,m4I,inc34x,inc34y,inc34t,    &
!                           js3,je3,ks3,ke3,js4,je4,ks4,ke4,        &
!                           js4o,je4o,ks4o,ke4o,jnum4o )

        CALL DC_BOUND_INTERP_u_LR ( un4,uo3,un3,m4I,inc34x,inc34y,   &
                                    inc34t,js3,je3,ks3,ke3,js4,je4,  &
                                    ks4,ke4,js4o,je4o,ks4o,ke4o )
        CALL DC_BOUND_INTERP_v_BT ( vn4,vo3,vn3,m4I,inc34x,inc34y,   &
                                    inc34t,js3,je3,ks3,ke3,js4,je4,  &
                                    ks4,ke4,js4o,je4o,ks4o,ke4o )
        CALL DC_BOUND_INTERP_u_BT ( un4,uo3,un3,m4I,inc34x,inc34y,   &
                                    inc34t,js3,je3,ks3,ke3,js4,je4,  &
                                    ks4,ke4,js4o,je4o,ks4o,ke4o )
        CALL DC_BOUND_INTERP_v_LR ( vn4,vo3,vn3,m4I,inc34x,inc34y,   &
                                    inc34t,js3,je3,ks3,ke3,js4,je4,  &
                                    ks4,ke4,js4o,je4o,ks4o,ke4o )

         un43( js4+1, ks4+1:ke4 ) = un4( js4+1, ks4+1:ke4 )
         un43(  je4 , ks4+1:ke4 ) = un4(  je4 , ks4+1:ke4 )
         un43( js4+1:je4, ks4 )   = un4( js4+1:je4, ks4)
         un43( js4+1:je4, ke4 )   = un4( js4+1:je4, ke4)

         vn43( js4, ks4:ke4-1 ) = vn4( js4, ks4:ke4-1 )
         vn43( je4, ks4:ke4-1 ) = vn4( je4, ks4:ke4-1 )
         vn43( js4:je4,  ks4  ) = vn4( js4:je4,  ks4  )
         vn43( js4:je4, ke4-1 ) = vn4( js4:je4, ke4-1 )
!======================================================================

 !      PRINT*,' IN DOUBLE COUPLE MS4=',MS4,'int34=', INT34
!... Initial Condition TYPE 2 (time-series initial condition)
!------------------------------------------------------------------------
  ! Vertical Sea-bottom deformation with rise time Tr


!... D: Total water depth cal.
!------------------------------------------------------------------------
!   Land Height z Cal.,
!   and Wet/Dry CELL Determinations
!   (NOTE: CELL is initially set as DRY (=0) !!
      DO k=ks4,ke4
        DO j=js4,je4
          Hb4(j,k) = H4(j,k) -eto4(j,k)
          D4(j,k) = zo4(j,k) +Hb4(j,k)
          IF( D4(j,k) .LT. EPSILON ) THEN
            zo4(j,k)  = -H4(j,k) +eto4(j,k)
            CELL4(j,k) = 0.
          ELSE
         !ELSE IF ( D4(j,k) .GE. EPSILON ) THEN
            CELL4(j,k) = 1.
          END IF
        END DO
      END DO

!... RUNUP
!------------------------------------------------------------------------
!======================================================================
        CALL RUNUP ( CELL4,D4,uo4,vo4,zo4,H4,eto4,js4,je4,ks4,ke4 ) ! This is new RUNUP
!======================================================================
       !CALL RUNUP_Kowalik (CELL4,D4,uo4,vo4,zo4,H4,eto4,js4,je4,ks4,ke4) ! original RUNUP

!======================================================================
! correct H3 into H4 at Dry Cell obtained from zn3 !
! NOTE: Not sure if this is nessesary for Partial Radiation B.C.
        DO k=ks4,ke4
          DO j=js4,je4
            IF( CELL4(j,k) .EQ. 0  ) THEN
              zn4(j,k)  = -H4(j,k) +etn4(j,k)
            END IF
          END DO
        END DO
!======================================================================

      ! u and v treatment at wet/dry interface
      CALL WET_DRY_UV_BC (uo4,vo4,zo4,H4,eto4,CELL4,js4,je4,ks4,ke4)


! save initial OUTPUT (T=0)
!---------------------------------------------------------------------
        IF ( m4 .EQ. m40DC ) THEN
          CALL OUTPUT_WATER_LEVEL_AVERAGED_INIT_GRID_4
        END IF


!------------------------------------------------------------------------
!  MAIN SOLVER
!  Momentum and Continuity Equations
!------------------------------------------------------------------------
!... wb
!      CALL VERT_BOTTOM (wbn4,un4,vn4,eto4,etn4,Hb4,CELL4,csz4,dx4,    &
!                        dy4,dt4,js4,je4,ks4,ke4)

      CALL Dlim_Du_Dv_CAL (Dlim4,Du4,Dv4,zo4,eto4,H4,D4,Hu4,Hv4,js4,  &
                           je4,ks4,ke4)

!... 1ST u,v
      CALL XMOMENT_1ST (un4,zo4,uo4,vo4,eto4,H4,Hu4,Du4,   &
                        CELL4,siz4,csz4,FCT4,NL4,          &
                        dt4,dt_dx4,dt_dy4,gdt_dx4,         &
                        js4,je4,ks4,ke4                  )
      CALL YMOMENT_1ST (vn4,zo4,un4,vo4,eto4,H4,Hv4,Dv4,   &
                        CELL4,siv4,csv4,FCT4,NL4,          &
                        dt4,dt_dx4,dt_dy4,gdt_dy4,         &
                        js4,je4,ks4,ke4                  )

!... q
!======================================================================
!      CALL NON_HYDRO_SIP_PIVOT_DC ( q4,q43,An4,Bn4,zo4,uo4,vo4,     &
!                                    eto4,wbo4,wbn4,wso4,H4,Dlim4,   &
!                                    CELL4,csz4,csv4,dt4,dx4,dy4,    &
!                                    js4,je4,ks4,ke4,NJK4,m4,dm_out4)
!======================================================================
!... ws
!      CALL VERT_SURFACE (wsn4,q4,wso4,wbo4,wbn4,Dlim4,CELL4,dt4,js4,  &
!                         je4,ks4,ke4)


!... 2ND u,v  correction from Q
!      CALL XMOMENT_2ND (un4,q4,An4,zo4,eto4,H4,Dlim4,CELL4,csz4,      &
!                           dt_dx4,js4,je4,ks4,ke4)
!      CALL YMOMENT_2ND (vn4,q4,Bn4,zo4,eto4,H4,Dlim4,CELL4,           &
!                           dt_dy4,js4,je4,ks4,ke4)


      CALL CONT (zn4,zo4,etn4,eto4,un4,vn4,Hu4,Hv4,CELL4,    &
                 csz4,csv4,dt4,dt_dx4,dt_dy4,js4,je4,ks4,ke4)

      CALL DC_BOUND_SHORELINE_z ( zn3,zn4,H3,H4,Hb4,etn4,D4,   &
                                  CELL3,CELL4,js3,je3,ks3,ke3, &
                                  js4,je4,ks4,ke4,js4o,je4o,   &
                                  ks4o,ke4o,inc34x,inc34y,m4 )

      zn43( js4, ks4:ke4 ) = zn4( js4, ks4:ke4 )
      zn43( je4, ks4:ke4 ) = zn4( je4, ks4:ke4 )
      zn43( js4:je4, ks4 ) = zn4( js4:je4, ks4 )
      zn43( js4:je4, ke4 ) = zn4( js4:je4, ke4 )

      CALL DC_BOUND_RADIATION ( zn4,un4,vn4,zn43,un43,vn43,CELL4,   &
                                H4,Hu4,Hv4,etn4,js4,je4,ks4,ke4 )


! OUTPUT (FEEDBACK) of Double-Coupling
!---------------------------------------------------------------------
!  extract zn3 from zn4
!        IF( m4I.EQ.inc34t)THEN
!          CALL DC_EXTRACT_zn ( zn3,zn4,H3,H4,CELL3,CELL4,       &
!                               js3,je3,ks3,ke3,js4,je4,ks4,ke4, &
!                               js4o,ks4o,inc34x,inc34y,m4 )
!
!        END IF

!---------------------------------------------------------------------
! Double-Couples 4th and 5th Grids
!---------------------------------------------------------------------
!      CALL  DOUBLE_COUPLE_D45
!---------------------------------------------------------------------

!... UPDATE variables
!------------------------------------------------------------------------------
      zo4  = zn4
      uo4  = un4
      vo4  = vn4
      wbo4 = wbn4
      wso4 = wsn4
      eto4 = etn4

!... fill zn4 along domain boundary using zn3
!    This is not nesessary for computation. only add zn4 along the 4rd
!    grid boundary (but not good, need more TEST)
!      CALL BOUND_INTERP_zo_D34(MS4)

! save time-series OUTPUT
!---------------------------------------------------------------------
      CALL OUTPUT_WATER_LEVEL_AVERAGED_GRID_4

      END  DO  smallstepd34

      RETURN

END SUBROUTINE

!---------------------------------------------------------------------
SUBROUTINE DC_BOUND_INTERP_H ( H_I, H_O, incx, incy,    &
                               jsO, jeO, ksO, keO,      &
                               jsI, jeI, ksI, keI,      &
                               jsIO, jeIO, ksIO, keIO )
!---------------------------------------------------------------------
!     H Interpolation from OUTER grid along Inner Domain Boundary
!                   (INPUT of Double-Coupling)
!---------------------------------------------------------------------
!
!  This program obtain H_I (H of the INNER grid) along the INNER
!  (fine grid) domain's bounries by interpolating H_O (H of the
!  OUTER grid) to provide smooth transition of depth between H_I and
!  H_O.
!
!
!   Z.Kowalik  2008  IMS, University of Alaska Fairbanks
!   Y.Yamazaki 2008  ORE, University of Hawaii Manoa
!---------------------------------------------------------------------
!      USE ALL_INPUTS
      IMPLICIT NONE

!  (INOUT)...
      REAL, DIMENSION(jsI:jeI,ksI:keI):: H_I ! inner domain H

!  (IN)...
      REAL, DIMENSION(jsO:jeO,ksO:keO):: H_O ! outer domain H

      INTEGER:: incx                         ! dxO/dxI
      INTEGER:: incy                         ! dyO/dyI

      INTEGER:: jsO, jeO, ksO, keO
      INTEGER:: jsI, jeI, ksI, keI

      INTEGER:: jsIO, jeIO, ksIO, keIO       ! j, k indice at 4 corners
                                             ! of inner domain respect
                                             ! to outer grids
!  (INTERNAL)...
      REAL:: LC,RC,BC,TC
      INTEGER:: j1, k1, j2, k2, jj, kk, mnum

!---------------------------------------------------------------------
! interpolate LEFT Boundary
      j1=jsIO
      DO k1=ksIO,keIO-1
        j2 = (j1-jsIO)*incx +1
        k2 = (k1-ksIO)*incy +1

        DO kk = 1,incy
            BC = FLOAT( ( incy -(kk-1) ) ) /FLOAT(incy)
            TC = 1-BC
            H_I( j2 , k2+kk-1) = BC*H_O(j1,k1)    &
                                +TC*H_O(j1,k1+1)
        END DO
      END DO


! interpolate RIGHT Boundary
      j1=jeIO
      DO k1=ksIO,keIO-1
        j2 = (j1-jsIO)*incx +1
        k2 = (k1-ksIO)*incy +1

        DO kk = 1,incy
            BC = FLOAT( ( incy -(kk-1) ) ) /FLOAT(incy)
            TC = 1-BC
            H_I( j2 , k2+kk-1) = BC*H_O(j1,k1)    &
                                +TC*H_O(j1,k1+1)
        END DO
      END DO


! interpolate BOTTOM boundary
      k1=ksIO
      DO j1=jsIO,jeIO-1
        j2 = (j1-jsIO)*incx +1
        k2 = (k1-ksIO)*incy +1

        DO jj = 1,incx
            LC = FLOAT( ( incx -(jj-1) ) ) /FLOAT(incx)
            RC = 1-LC
            H_I( j2+jj-1 ,k2 ) = LC*H_O(j1,k1)    &
                                +RC*H_O(j1+1,k1)
        END DO
      END DO

! interpolate TOP boundary
      k1=keIO
      DO j1=jsIO,jeIO-1
        j2 = (j1-jsIO)*incx +1
        k2 = (k1-ksIO)*incy +1

        DO jj = 1,incx
            LC = FLOAT( ( incx -(jj-1) ) ) /FLOAT(incx)
            RC = 1-LC
            H_I( j2+jj-1 ,k2 ) = LC*H_O(j1,k1)    &
                                +RC*H_O(j1+1,k1)
        END DO
      END DO


! fill RIGHT-TOP coner point, H_I(jeI,keI)
        H_I(jeI,keI) = H_O(jeIO,keIO)

! Boundary Treatment
!    set H_I at js+1, je-1, ks+1, ke-1 equal to js, je, ks, ke, repsectively
      H_I( jsI+1, ksI+1:keI-1 ) = H_I( jsI, ksI+1:keI-1 )
      H_I( jeI-1, ksI+1:keI-1 ) = H_I( jeI, ksI+1:keI-1 )

      H_I( jsI+1:jeI-1, ksI+1 ) = H_I( jsI+1:jeI-1, ksI )
      H_I( jsI+1:jeI-1, keI-1 ) = H_I( jsI+1:jeI-1, keI )



  ! CORNERS
!      H_I(jsI+1,ksI+1) = 0.5*( H_I(jsI,ksI+1) + H_I(jsI+1,ksI) )
!      H_I(jsI+1,keI-1) = 0.5*( H_I(jsI,keI-1) + H_I(jsI+1,keI) )

!      H_I(jeI-1,ksI+1) = 0.5*( H_I(jeI,ksI+1) + H_I(jeI-1,ksI) )
!      H_I(jeI-1,keI-1) = 0.5*( H_I(jeI,keI-1) + H_I(jeI-1,keI) )

      H_I(jsI+1,ksI+1) = 0.25*( H_I(jsI,ksI+1) +H_I(jsI+1,ksI) +H_I(jsI+1,ksI+2) +H_I(jsI+2,ksI+1) )
      H_I(jsI+1,keI-1) = 0.25*( H_I(jsI,keI-1) +H_I(jsI+1,keI) +H_I(jsI+2,keI-1) +H_I(jsI+1,keI-2) )

      H_I(jeI-1,ksI+1) = 0.25*( H_I(jeI,ksI+1) +H_I(jeI-1,ksI) +H_I(jeI-2,ksI+1) +H_I(jeI-1,ksI+2) )
      H_I(jeI-1,keI-1) = 0.25*( H_I(jeI,keI-1) +H_I(jeI-1,keI) +H_I(jeI-2,keI-1) +H_I(jeI-1,keI-2) )




! LEFT

      j1=jsIO
      DO k2=ksI+2,keI-2
        mnum = incx*2
        DO j2=jsI+2,jsI+mnum
            LC = 1 -FLOAT( j2+1 -(jsI+2) ) /FLOAT( mnum -1 )
            RC = 1-LC
            H_I( j2 , k2) = LC*H_I( jsI+1    , k2)   &
                          +RC*H_I( jsI+mnum , k2)
        END DO
      END DO

!RIGHT

      j1=jsIO
      DO k2=ksI+2,keI-2
        mnum = incx*2
        DO j2 = jeI-mnum ,jeI-2
            LC = 1-FLOAT( j2 -(jeI-mnum) ) /FLOAT( mnum -1 )
            RC = 1-LC
            H_I( j2 , k2) = LC*H_I( jeI-mnum , k2)   &
                          +RC*H_I( jeI-1    , k2)
        END DO
      END DO

!BOTTOM

      DO j2=jsI+2,jeI-2
        mnum = incy*2
        DO k2=ksI+2,ksI+mnum
            BC = 1 -FLOAT( k2+1 -(ksI+2) ) /FLOAT( mnum -1 )
            TC = 1-BC
            H_I( j2 , k2) = BC*H_I( j2, ksI+1    )   &
                          +TC*H_I( j2, ksI+mnum )
        END DO
      END DO

!TOP
      DO j2=jsI+2,jeI-2
        mnum = incy*2
        DO k2= keI-mnum, keI-2
            BC = 1 -FLOAT( k2 -(keI-mnum) ) /FLOAT( mnum -1 )
            TC = 1 -BC
            H_I( j2 , k2) = BC*H_I( j2, keI-mnum )   &
                          +TC*H_I( j2, keI-1)
        END DO
      END DO


END SUBROUTINE

!-----------------------------------------------------------------------
SUBROUTINE DC_INTERP (vn_I, vo_O, vn_O, mI, incx, incy,     &
                      inct, jsO, jeO, ksO, keO, jsI, jeI, ksI, keI,  &
                      jsIO, jeIO, ksIO, keIO, jnumIO )
!-----------------------------------------------------------------------
!        Cell-centered variable Interpolation of the OUTER grid
!                   (INPUT of Double-Coupling)
!------------------------------------------------------------------------
!
!  This program interpolates the cell-centered variables (ex, z, eta, D)
!  in the OUTER grid(domain), v_O, to obtain the INNER grid variables,
!  vn_I.
!
!   Z.Kowalik  2009 IMS,      University of Alaska Fairbanks
!   Y.Yamazaki 2009 ORE,      University of Hawaii Manoa
!------------------------------------------------------------------------
!      USE ALL_INPUTS
      IMPLICIT NONE

!  (OUT)...
      REAL, DIMENSION(jsI:jeI,ksI:keI) :: vn_I ! inner grid variables

!  (IN)...
      REAL, DIMENSION(jsO:jeO,ksO:keO) :: vo_O ! outer grid variables
      REAL, DIMENSION(jsO:jeO,ksO:keO) :: vn_O ! outer grid variables


      INTEGER:: incx                          ! dxO/dxI
      INTEGER:: incy                          ! dyO/dyI
      INTEGER:: inct                          ! dtO/dtI

      INTEGER:: jsO, jeO, ksO, keO
      INTEGER:: jsI, jeI, ksI, keI

      INTEGER:: jsIO, jeIO, ksIO, keIO        ! j, k indice at 4 corners
                                              ! of inner domain respect
                                              ! to outer grids
      INTEGER:: jnumIO
      INTEGER:: mI                            ! inner grid time step

!  (INTERNAL)...

      REAL, DIMENSION(jsO:jeO,ksO:keO) :: v_O 
      REAL, DIMENSION(1:jnumIO,ksI:keI):: vy_I
      REAL, DIMENSION(jsI:jeI,ksI:keI) :: vI_O

      REAL:: OC,NC                                 ! TIME STEP  weighting coefficients
      REAL:: LC,RC,BC,TC                           ! GRID SPACE weighting coefficients

      INTEGER:: j1, k1, j2, k2, jj, kk

      v_O  = 0.0
      vy_I = 0.0
      vI_O = 0.0
!      vn_I = 0.0

!---------------------------------------------------------------------
! TIME STEP Interpolation
!---------------------------------------------------------------------
! Time Weigthing Coefficients
      OC = FLOAT(inct-mI) /FLOAT(inct)            ! old time step weight = PM1/PM3
      NC = 1-OC                                   ! new time step weight = PM2/PM3

      DO k1=ksIO,keIO
        DO j1=jsIO,jeIO
         v_O(j1,k1) = OC*vo_O(j1,k1) +NC*vn_O(j1,k1)
        END DO
      END DO

!---------------------------------------------------------------------
! Vertical Interpolation
!---------------------------------------------------------------------
      DO j1=jsIO,jeIO
        DO k1=ksIO,keIO-1
          k2 = (k1-ksIO)*incy +1
          DO kk = 1,incy                      !For incy = 3
          !BOTTOM weighting coeff.            ! kk =  1    2    3
           BC = 1 -FLOAT( kk-1 ) /FLOAT(incy) ! BC =  1   2/3   1/3
          !TOP weighting coeff.               ! TC =  0   1/3   2/3
           TC = 1-BC
          !INTERPOLATION
           vy_I( j1-jsIO+1 , k2+kk-1) = BC*v_O(j1, k1 )  &
                                       +TC*v_O(j1,k1+1)
          END DO
        END DO
      END DO

      vy_I(1:jnumIO, keI) = v_O(jsIO:jeIO, keIO)

!---------------------------------------------------------------------
! Horizontal Interpolation
!---------------------------------------------------------------------

      DO k2=ksI,keI
        DO j1=1,jnumIO-1
          j2 = (j1-1)*incx +1
          DO jj = 1,incx                       !For incx = 3
          !LEFT weighting coeff.               ! jj =  1    2    3
            LC = 1 -FLOAT( jj-1 ) /FLOAT(incx) ! LC =  1   2/3   1/3
          !RIGHT weighting coeff.              ! RC =  0   1/3   2/3
            RC = 1-LC
          !INTERPOLATION
            vI_O( j2+jj-1 ,k2 ) = LC*vy_I( j1 ,k2)    &
                                 +RC*vy_I(j1+1,k2)
          END DO
        END DO
      END DO

      vI_O(jeI,ksI:keI) = vy_I(jnumIO,ksI:keI)

!---------------------------------------------------------------------
! Set the inner grid variables, vn_I, to the interpolated outer grid
! varaiables, vI_O, at CELL, CELLI_O = 1
!---------------------------------------------------------------------
!      DO k2 = ksI, keI
!        DO j2 = jsI, jeI
!          IF ( CELLI_O(j2,k2) .EQ. 1 ) THEN
!            vn_I(j2,k2) = vI_O(j2,k2)
!          END IF
!        END DO
!      END DO

      vn_I(  jsI  ,ksI:keI) = vI_O(  jsI  ,ksI:keI) ! LEFT
      vn_I(  jeI  ,ksI:keI) = vI_O(  jeI  ,ksI:keI) ! RIGHT
      vn_I(jsI:jeI,  ksI  ) = vI_O(jsI:jeI,  ksI  ) ! BOTTOM
      vn_I(jsI:jeI,  keI  ) = vI_O(jsI:jeI,  keI  ) ! TOP


END SUBROUTINE

!-----------------------------------------------------------------------
SUBROUTINE DC_INTERP_q (vn_I, vo_O, vn_O, mI, incx, incy, inct,   &
                        jsO, jeO, ksO, keO, jsI, jeI, ksI, keI,   &
                        jsIO, jeIO, ksIO, keIO, jnumIO )
!-----------------------------------------------------------------------
!        Cell-centered variable Interpolation of the OUTER grid
!                   (INPUT of Double-Coupling)
!------------------------------------------------------------------------
!
!  This program interpolates the cell-centered variables (ex, z, eta, D)
!  in the OUTER grid(domain), v_O, to obtain the INNER grid variables,
!  vn_I.
!
!   Z.Kowalik  2009 IMS,      University of Alaska Fairbanks
!   Y.Yamazaki 2009 ORE,      University of Hawaii Manoa
!------------------------------------------------------------------------
!      USE ALL_INPUTS
      IMPLICIT NONE

!  (INOUT)...
      REAL(dp), DIMENSION(jsI:jeI,ksI:keI) :: vn_I ! inner grid variables

!  (IN)...
      REAL(dp), DIMENSION(jsO:jeO,ksO:keO) :: vo_O ! outer grid variables
      REAL(dp), DIMENSION(jsO:jeO,ksO:keO) :: vn_O ! outer grid variables

      INTEGER:: incx                               ! dxO/dxI
      INTEGER:: incy                               ! dyO/dyI
      INTEGER:: inct                               ! dtO/dtI

      INTEGER:: jsO, jeO, ksO, keO
      INTEGER:: jsI, jeI, ksI, keI

      INTEGER:: jsIO, jeIO, ksIO, keIO             ! j, k indice at 4 corners
                                                   ! of inner domain respect
                                                   ! to outer grids
      INTEGER:: jnumIO
      INTEGER:: mI                                 ! inner grid time step

!  (INTERNAL)...
      REAL(dp), DIMENSION(jsO:jeO,ksO:keO) :: v_O  ! time step interpolated outer grid variables
      REAL(dp), DIMENSION(1:jnumIO,ksI:keI):: vy_I !

      REAL:: OC,NC                                 ! TIME STEP  weighting coefficients
      REAL:: LC,RC,BC,TC                           ! GRID SPACE weighting coefficients

      INTEGER:: j1, k1, j2, k2, jj, kk

      v_O  = 0.0
      vy_I = 0.0
      vn_I = 0.0
      vn_I = 0.0

!---------------------------------------------------------------------
! TIME STEP Interpolation
!---------------------------------------------------------------------
! Time Weigthing Coefficients
      OC = FLOAT(inct-mI) /FLOAT(inct)            ! old time step weight = PM1/PM3
      NC = 1-OC                                   ! new time step weight = PM2/PM3

      DO k1=ksIO,keIO
        DO j1=jsIO,jeIO
         v_O(j1,k1) = OC*vo_O(j1,k1) +NC*vn_O(j1,k1)
        END DO
      END DO

!---------------------------------------------------------------------
! Vertical Interpolation
!---------------------------------------------------------------------
      DO j1=jsIO,jeIO
        DO k1=ksIO,keIO-1
          k2 = (k1-ksIO)*incy +1
          DO kk = 1,incy                      !For incy = 3
          !BOTTOM weighting coeff.            ! kk =  1    2    3
           BC = 1 -FLOAT( kk-1 ) /FLOAT(incy) ! BC =  1   2/3   1/3
          !TOP weighting coeff.               ! TC =  0   1/3   2/3
           TC = 1-BC
          !INTERPOLATION
           vy_I( j1-jsIO+1 , k2+kk-1) = BC*v_O(j1, k1 )  &
                                       +TC*v_O(j1,k1+1)
          END DO
        END DO
      END DO

      vy_I(1:jnumIO, keI) = v_O(jsIO:jeIO, keIO)

!---------------------------------------------------------------------
! Horizontal Interpolation
!---------------------------------------------------------------------

      DO k2=ksI,keI
        DO j1=1,jnumIO-1
          j2 = (j1-1)*incx +1
          DO jj = 1,incx                       !For incx = 3
          !LEFT weighting coeff.               ! jj =  1    2    3
            LC = 1 -FLOAT( jj-1 ) /FLOAT(incx) ! LC =  1   2/3   1/3
          !RIGHT weighting coeff.              ! RC =  0   1/3   2/3
            RC = 1-LC
          !INTERPOLATION
            vn_I( j2+jj-1 ,k2 ) = LC*vy_I( j1 ,k2)    &
                                +RC*vy_I(j1+1,k2)
          END DO
        END DO
      END DO

      vn_I(jeI,ksI:keI) = vy_I(jnumIO,ksI:keI)

END SUBROUTINE

!---------------------------------------------------------------------
SUBROUTINE DC_BOUND_INTERP_u_BT ( un_I, uo_O, un_O, mI,    &
                                  incx, incy, inct,        &
                                  jsO, jeO, ksO, keO,      &
                                  jsI, jeI, ksI, keI,      &
                                  jsIO, jeIO, ksIO, keIO )
!---------------------------------------------------------------------
!      un Interpolation of BOTTOM and TOP INNER Domain Boundary
!                  (INPUT of Double-Coupling)
!---------------------------------------------------------------------
!
!  This program obtain un_I (un of the INNER grid) along the BOTTOM
!  and TOP INNER (fine grid) domain's bounries by interpolating uo_O
!  and un_O (uo and un of the OUTER grid) in both space (incx,incy)
!  and time (inct).
!
!
!   Z.Kowalik  2008  IMS, University of Alaska Fairbanks
!   Y.Yamazaki 2008  ORE, University of Hawaii Manoa
!---------------------------------------------------------------------
!      USE ALL_INPUTS
      IMPLICIT NONE

!  (INOUT)...
      REAL, DIMENSION(jsI:jeI,ksI:keI):: un_I ! inner domain uo

!  (IN)...
      REAL, DIMENSION(jsO:jeO,ksO:keO):: uo_O ! outer domain uo
      REAL, DIMENSION(jsO:jeO,ksO:keO):: un_O ! outer domain un

      INTEGER:: incx                          ! dxO/dxI
      INTEGER:: incy                          ! dyO/dyI
      INTEGER:: inct                          ! dtO/dtI

      INTEGER:: jsO, jeO, ksO, keO
      INTEGER:: jsI, jeI, ksI, keI

      INTEGER:: jsIO, jeIO, ksIO, keIO        ! j, k indice at 4 corners
                                              ! of inner domain respect
                                              ! to outer grids

      INTEGER:: mI                            ! inner grid time step

!  (INTERNAL)...
      REAL, DIMENSION(jsO:jeO):: uon_O

      REAL:: OC,NC  ! TIME STEP  weighting coefficients

      REAL:: LC,RC  ! GRID SPACE weighting coefficients

      INTEGER:: j1, k1, j2, k2, jj, kk
      INTEGER:: jjs, jje


! Time Weigthing Coefficients
      OC = FLOAT(inct-mI) /FLOAT(inct) ! old time step weight = PM1/PM3
      NC = 1-OC                        ! new time step weight = PM2/PM3

!------------------------------------------------------------------------
! BOTTOM (k2=ksI)
!------------------------------------------------------------------------
      k1 = ksIO
      k2 = ksI

!... TIME_STEP interpolation
      uon_O = 0.0
!      DO j1=jsIO,jeIO+1
         DO j1=jsIO,jeIO
        uon_O(j1) = OC*uo_O(j1,k1) +NC*un_O(j1,k1)
      END DO

!... LEFT END CELLS
      j1 = jsIO
      j2 = jsI

      IF ( MOD(incy, 2 ) .EQ. 1 ) THEN           ! odd inc
!        jjs= (incx+1)/2+1
        jjs= (incx+1)/2
        DO jj = jjs+1,incx
!          IF ( CELL1(j1,k1)   .EQ. 1       &
!                              .AND.        &
!               CELL1(j1+1,k1) .EQ. 1) THEN
           !LC = FLOAT( ( incx -(jj-1) ) ) /FLOAT(incx)  
            LC = 1 -FLOAT( jj-1 ) /FLOAT(incx)
            RC = 1 -LC

           !un_I( j2+jj-1   ,k2 ) = LC*uon_O(j1,k1)    &
            un_I( j2+jj-jjs ,k2 ) = LC*uon_O(j1)    &
                                   +RC*uon_O(j1+1)
!          ELSE
!            un_I( j2+jj-jjs ,k2 ) = 0.0
!          END IF
        END DO

      ELSE != IF ( MOD(incy, 2 ) .EQ. 0 ) THEN   ! even inc
!       jjs= incx/2+1
        jjs= incx/2
        DO jj = jjs+1,incx
!          IF ( CELL1(j1,k1)   .EQ. 1       &
!                              .AND.        &
!               CELL1(j1+1,k1) .EQ. 1) THEN
            LC = 1 -FLOAT( jj*2-1 ) /FLOAT( incx*2 )
            RC = 1-LC
                                                 
           !un_I( j2+jj-1   ,k2 ) = LC*uon_O(j1,k1)    &
            un_I( j2+jj-jjs ,k2 ) = LC*uon_O(j1)    &
                                   +RC*uon_O(j1+1)



!          ELSE
!            un_I( j2+jj-jjs ,k2 ) = 0.0
!          END IF
        END DO
      END IF

!... INNER CELLS
      DO j1=jsIO+1,jeIO-1
!        j2 = (j1-jsIO)*incx -( incx -jjs )
        j2 = (j1-(jsIO+1))*incx +(incx-jjs+1) +1
        DO jj = 1,incx                                    
!          IF ( CELL1(j1,k1)   .EQ. 1       &
!                              .AND.        &
!               CELL1(j1+1,k1) .EQ. 1) THEN
            IF ( MOD(incx, 2 ) .EQ. 1 ) THEN           ! odd inc 
              LC = 1 -FLOAT( jj-1 ) /FLOAT(incx)
            ELSE                                         ! even inc
              LC = 1 -FLOAT( jj*2-1 ) /FLOAT( incx*2 )
            END IF
            RC = 1-LC                                       

            un_I( j2+jj-1 ,k2 ) = LC*uon_O(j1)    &
                                +RC*uon_O(j1+1)
!          ELSE
!            un_I( j2+jj-1 ,k2 ) = 0.0
!          END IF
        END DO
      END DO

!... RIGHT END CELLS
!      j1 = jeIO
      j1= jeIO-1
     ! j2 = jeI-1  -( incx -jjs )
     !j2 = jeI-1

      IF ( MOD(incy, 2 ) .EQ. 1 ) THEN           ! odd inc

        jje= (incx+1)/2
        j2 = jeI-(jje-1)
        DO jj = 1,jje
!          IF ( CELL1(j1,k1)   .EQ. 1       &
!                              .AND.        &
!               CELL1(j1+1,k1) .EQ. 1) THEN
           !LC = FLOAT( ( incx -(jj-1) ) ) /FLOAT(incx)  
            LC = 1 -FLOAT( jj-1 ) /FLOAT(incx)
            RC = 1-LC

            un_I( j2+jj-1 ,k2 ) = LC*uon_O(j1)    &
                                +RC*uon_O(j1+1)
!          ELSE
!            un_I( j2+jj-1 ,k2 ) = 0.0
!          END IF
        END DO

      ELSE != IF ( MOD(incy, 2 ) .EQ. 0 ) THEN   ! even inc

        jje= incx/2
        j2 = jeI-(jje-1)
        DO jj = 1,jje
!          IF ( CELL1(j1,k1)   .EQ. 1       &
!                              .AND.        &
!               CELL1(j1+1,k1) .EQ. 1) THEN
            LC = 1 -FLOAT( jj*2-1 ) /FLOAT( incx*2 )
            RC = 1-LC

            un_I( j2+jj-1 ,k2 ) = LC*uon_O(j1)    &
                                +RC*uon_O(j1+1)
!          ELSE
!            un_I( j2+jj-1 ,k2 ) = 0.0
!          END IF
        END DO

      END IF



!------------------------------------------------------------------------
! TOP (k2=keI)
!------------------------------------------------------------------------
      k1 = keIO
      k2 = keI

!... TIME_STEP interpolation
      uon_O = 0.0
!...  DO j1=jsIO,jeIO+1
      DO j1=jsIO,jeIO
        uon_O(j1) = OC*uo_O(j1,k1) +NC*un_O(j1,k1)
      END DO

!... LEFT END CELLS
      j1 = jsIO
      j2 = jsI

      IF ( MOD(incy, 2 ) .EQ. 1 ) THEN           ! odd inc
!        jjs= (incx+1)/2+1
        jjs= (incx+1)/2
        DO jj = jjs+1,incx
            LC = 1 -FLOAT( jj-1 ) /FLOAT(incx)
            RC = 1 -LC

            un_I( j2+jj-jjs ,k2 ) = LC*uon_O(j1)    &
                                   +RC*uon_O(j1+1)
        END DO

      ELSE != IF ( MOD(incy, 2 ) .EQ. 0 ) THEN   ! even inc
!       jjs= incx/2+1
        jjs= incx/2
        DO jj = jjs+1,incx
            LC = 1 -FLOAT( jj*2-1 ) /FLOAT( incx*2 )
            RC = 1-LC
                     
           !un_I( j2+jj-1   ,k2 ) = LC*uon_O(j1,k1)    &
            un_I( j2+jj-jjs ,k2 ) = LC*uon_O(j1)    &
                                   +RC*uon_O(j1+1)
        END DO
      END IF

!... INNER CELLS
      DO j1=jsIO+1,jeIO-1
!        j2 = (j1-jsIO)*incx -( incx -jjs )
        j2 = (j1-(jsIO+1))*incx +(incx-jjs+1) +1
        DO jj = 1,incx                                    
!          IF ( CELL1(j1,k1)   .EQ. 1       &
!                              .AND.        &
!               CELL1(j1+1,k1) .EQ. 1) THEN
            IF ( MOD(incx, 2 ) .EQ. 1 ) THEN           ! odd inc
              LC = 1 -FLOAT( jj-1 ) /FLOAT(incx)
            ELSE                                         ! even inc
              LC = 1 -FLOAT( jj*2-1 ) /FLOAT( incx*2 )
            END IF
            RC = 1-LC                                       

            un_I( j2+jj-1 ,k2 ) = LC*uon_O(j1)    &
                                 +RC*uon_O(j1+1)
!          ELSE
!            un_I( j2+jj-1 ,k2 ) = 0.0
!          END IF
        END DO
      END DO

!... RIGHT END CELLS
      j1 = jeIO-1
!..   j1 = jeIO
     !j2 = jeI-1  -( incx -jjs )
     !j2 = jeI-1


      IF ( MOD(incy, 2 ) .EQ. 1 ) THEN           ! odd inc

        jje= (incx+1)/2
        j2 = jeI-(jje-1)
        DO jj = 1,jje
!          IF ( CELL1(j1,k1)   .EQ. 1       &
!                              .AND.        &
!               CELL1(j1+1,k1) .EQ. 1) THEN
           !LC = FLOAT( ( incx -(jj-1) ) ) /FLOAT(incx)  
            LC = 1 -FLOAT( jj-1 ) /FLOAT(incx)
            RC = 1-LC

            un_I( j2+jj-1 ,k2 ) = LC*uon_O(j1)    &
                                 +RC*uon_O(j1+1)
!          ELSE
!            un_I( j2+jj-1 ,k2 ) = 0.0
!          END IF
        END DO

      ELSE != IF ( MOD(incy, 2 ) .EQ. 0 ) THEN   ! even inc

        jje= incx/2
        j2 = jeI-(jje-1)
        DO jj = 1,jje
!          IF ( CELL1(j1,k1)   .EQ. 1       &
!                              .AND.        &
!               CELL1(j1+1,k1) .EQ. 1) THEN
            LC = 1 -FLOAT( jj*2-1 ) /FLOAT( incx*2 )
            RC = 1-LC

            un_I( j2+jj-1 ,k2 ) = LC*uon_O(j1)    &
                                 +RC*uon_O(j1+1)
!          ELSE
!            un_I( j2+jj-1 ,k2 ) = 0.0
!          END IF
        END DO

      END IF

END SUBROUTINE

!---------------------------------------------------------------------
SUBROUTINE DC_BOUND_INTERP_u_LR ( un_I, uo_O, un_O, mI,    &
                                  incx, incy, inct,        &
                                  jsO, jeO, ksO, keO,      &
                                  jsI, jeI, ksI, keI,      &
                                  jsIO, jeIO, ksIO, keIO )
!---------------------------------------------------------------------
!      un Interpolation of LEFT and RIGHT INNER Domain Boundary
!                  (INPUT of Double-Coupling)
!---------------------------------------------------------------------
!
!  This program obtain un_I (un of the INNER grid) along the LEFT and
!  RIGHT INNER (fine grid) domain's bounries by interpolating uo_O
!  and un_O (uo and un of the OUTER grid) in both space (incx,incy)
!  and time (inct).
!
!
!   Z.Kowalik  2008  IMS, University of Alaska Fairbanks
!   Y.Yamazaki 2008  ORE, University of Hawaii Manoa
!---------------------------------------------------------------------
!      USE ALL_INPUTS
      IMPLICIT NONE

!  (INOUT)...
      REAL, DIMENSION(jsI:jeI,ksI:keI):: un_I ! inner domain uo

!  (IN)...
      REAL, DIMENSION(jsO:jeO,ksO:keO):: uo_O ! outer domain uo
      REAL, DIMENSION(jsO:jeO,ksO:keO):: un_O ! outer domain un

      INTEGER:: incx                          ! dxO/dxI
      INTEGER:: incy                          ! dyO/dyI
      INTEGER:: inct                          ! dtO/dtI

      INTEGER:: jsO, jeO, ksO, keO
      INTEGER:: jsI, jeI, ksI, keI

      INTEGER:: jsIO, jeIO, ksIO, keIO        ! j, k indice at 4 corners
                                              ! of inner domain respect
                                              ! to outer grids

      INTEGER:: mI                            ! inner grid time step

!  (INTERNAL)...
      REAL, DIMENSION(ksO:keO):: uL
      REAL, DIMENSION(ksO:keO):: uR
      REAL, DIMENSION(ksO:keO):: uon_O

      REAL:: OC,NC         ! TIME STEP  weighting coefficients

      REAL:: LC,RC,BC,TC   ! GRID SPACE weighting coefficients

      INTEGER:: j1, k1, j2, k2, jj, kk

! Time Weigthing Coefficients
      OC = FLOAT(inct-mI) /FLOAT(inct) ! old time step weight = PM1/PM3
      NC = 1-OC                        ! new time step weight = PM2/PM3

!------------------------------------------------------------------------
! LEFT (j2=jsI+1)
!------------------------------------------------------------------------
      uL = 0.0
      uR = 0.0
      uon_O = 0.0
     !j1=jsIO
     !j2 = jsI+1

! TIME_STEP interpolation

      DO k1=ksIO,keIO
        uL(k1) = OC*uo_O(jsIO,  k1) +NC*un_O(jsIO,  k1)
        uR(k1) = OC*uo_O(jsIO+1,k1) +NC*un_O(jsIO+1,k1)
       !ZA1    = (PM1/PM3)*zo1(j1,k1) +(PM2/PM3)*zn1(j1,k1)
      END DO


! LEFT-RIGHT horizontal interpolation
      DO k1=ksIO,keIO

!... LEFT
        IF ( MOD(incx, 2 ) .EQ. 1 ) THEN           ! odd inc
          LC = FLOAT( (incx-1) /2 ) /FLOAT(  incx  )
        ELSE != IF ( MOD(incx, 2 ) .EQ. 0 ) THEN   ! even inc
          LC = FLOAT(  incx-1     ) /FLOAT( incx*2 )
        END IF
!... RIGHT
        RC = 1-LC

        uon_O(k1) = LC*uL(k1) +RC*uR(k1)
!        uon_O(k1) = LC*un_O(jsIO,k1) +RC*un_O(jsIO+1,k1)
      END DO

! BOTTOM-TOP vertical interpolation
      DO k1=ksIO,keIO-1
        k2 = (k1-ksIO)*incy +1
        DO kk = 1,incy
!... BOTTOM
          BC = 1 -FLOAT( kk-1 ) /FLOAT(incy)
!... TOP
          TC = 1-BC

          un_I(jsI+1,k2-1+kk) = BC*uon_O(k1) +TC*uon_O(k1+1)

        END DO
      END DO

      un_I(jsI+1,ksI) = 0.0  ! for LR interp only

!------------------------------------------------------------------------
! RIGHT (j2=jeI)
!------------------------------------------------------------------------
!      uon_O = 0.0
      uL = 0.0
      uR = 0.0
      uon_O = 0.0
     !j1=jeIO
     !j2 = jeI


! TIME_STEP interpolation

      DO k1=ksIO,keIO
!        uL(k1) = OC*uo_O(jeIO,  k1) +NC*un_O(jeIO,  k1)
!        uR(k1) = OC*uo_O(jeIO+1,k1) +NC*un_O(jeIO+1,k1)
        uL(k1) = OC*uo_O(jeIO-1,k1)+NC*un_O(jeIO-1,k1)
        uR(k1) = OC*uo_O(jeIO,k1) +NC*un_O(jeIO,k1)

       !ZA1    = (PM1/PM3)*zo1(j1,k1) +(PM2/PM3)*zn1(j1,k1)
      END DO
!        uL(keIO)=uL(keIO-1)
!        uR(keIO)=uR(keIO-1)
! LEFT-RIGHT horizontal interpolation
      DO k1=ksIO,keIO

!... LEFT
        IF ( MOD(incx, 2 ) .EQ. 1 ) THEN           ! odd inc
          LC = FLOAT( (incx+1) /2 ) /FLOAT(  incx  )
        ELSE != IF ( MOD(incx, 2 ) .EQ. 0 ) THEN   ! even inc
          LC = FLOAT(  incx+1     ) /FLOAT( incx*2 )
        END IF
!... RIGHT
        RC = 1-LC
        uon_O(k1) = LC*uL(k1) +RC*uR(k1)
!        uon_O(k1) = LC*un_O(jeIO,k1) +RC*un_O(jeIO+1,k1)

      END DO

      DO k1=ksIO,keIO-1
        k2 = (k1-ksIO)*incy +1
        DO kk = 1,incy
!... BOTTOM
          BC = 1 -FLOAT( kk-1 ) /FLOAT(incy)
!... TOP
          TC = 1-BC

          un_I(jeI,k2-1+kk) = BC*uon_O(k1) +TC*uon_O(k1+1)
        END DO
      END DO

      un_I(jeI,ksI) = 0.0 ! for LR interp only

END SUBROUTINE

!-----------------------------------------------------------------------
SUBROUTINE DC_BOUND_INTERP_v_BT ( vn_I, vo_O, vn_O, mI,    &
                                  incx, incy, inct,        &
                                  jsO, jeO, ksO, keO,      &
                                  jsI, jeI, ksI, keI,      &
                                  jsIO, jeIO, ksIO, keIO )
!---------------------------------------------------------------------
!      vn Interpolation of BOTTOM and TOP INNER Domain Boundary
!                  (INPUT of Double-Coupling)
!---------------------------------------------------------------------
!
!  This program obtain vn_I (vn of the INNER grid) along the BOTTOM
!  and TOP INNER (fine grid) domain's bounries by interpolating vo_O
!  and vn_O (vo and vn of the OUTER grid) in both space (incx,incy)
!  and time (inct).
!
!
!   Z.Kowalik  2008  IMS, University of Alaska Fairbanks
!   Y.Yamazaki 2008  ORE, University of Hawaii Manoa
!---------------------------------------------------------------------
!      USE ALL_INPUTS
      IMPLICIT NONE

!  (INOUT)...
      REAL, DIMENSION(jsI:jeI,ksI:keI):: vn_I ! inner domain vn

!  (IN)...
      REAL, DIMENSION(jsO:jeO,ksO:keO):: vo_O ! outer domain vo
      REAL, DIMENSION(jsO:jeO,ksO:keO):: vn_O ! outer domain vn

      INTEGER:: incx                          ! dxO/dxI
      INTEGER:: incy                          ! dyO/dyI
      INTEGER:: inct                          ! dtO/dtI

      INTEGER:: jsO, jeO, ksO, keO
      INTEGER:: jsI, jeI, ksI, keI

      INTEGER:: jsIO, jeIO, ksIO, keIO        ! j, k indice at 4 corners
                                              ! of inner domain respect
                                              ! to outer grids

      INTEGER:: mI                            ! inner grid time step

!  (INTERNAL)...
      REAL, DIMENSION(jsO:jeO):: vB
      REAL, DIMENSION(jsO:jeO):: vT
      REAL, DIMENSION(jsO:jeO):: von_O

      REAL:: OC,NC         ! TIME STEP  weighting coefficients

      REAL:: LC,RC,BC,TC   ! GRID SPACE weighting coefficients

      INTEGER:: j1, k1, j2, k2, jj, kk
      INTEGER:: jjs, jje

! Time Weigthing Coefficients
      OC = FLOAT(inct-mI) /FLOAT(inct) ! old time step weight = PM1/PM3
      NC = 1-OC                        ! new time step weight = PM2/PM3

!------------------------------------------------------------------------
! BOTTOM (k2=ksI)
!------------------------------------------------------------------------
      vB = 0.0
      vT = 0.0
      von_O = 0.0
!      k1 = ksIO
!      k2 = ksI

!... TIME_STEP interpolation

      DO j1=jsIO,jeIO
        vB(j1) = OC*vo_O(j1,ksIO-1) +NC*vn_O(j1,ksIO-1)
        vT(j1) = OC*vo_O(j1,ksIO)   +NC*vn_O(j1,ksIO)
      END DO

! BOTTOM-TOP vertical interpolation
      DO j1=jsIO,jeIO

!... BOTTOM
        IF ( MOD(incy, 2 ) .EQ. 1 ) THEN           ! odd inc
          BC = FLOAT( (incy-1) /2 ) /FLOAT(  incy  )
        ELSE != IF ( MOD(incy, 2 ) .EQ. 0 ) THEN   ! even inc
          BC = FLOAT(  incy-1     ) /FLOAT( incy*2 )
        END IF
!... TOP
        TC = 1-BC
        von_O(j1) = BC*vB(j1) +TC*vT(j1)
       !von_O(j1) = BC*vn_O(j1,ksIO-1) +TC*vn_O(j1,ksIO)
      END DO

      DO j1=jsIO,jeIO-1
        j2 = (j1-jsIO)*incx+1

        DO jj = 1,incx                                    
!          IF ( CELL1(j1,ksIO)   .EQ. 1       &
!                              .AND.        &
!               CELL1(j1+1,ksIO) .EQ. 1) THEN
            LC = 1 -FLOAT( jj-1 ) /FLOAT(incx)
            RC = 1-LC

            vn_I( j2+jj-1 ,ksI ) = LC*von_O(j1)    &
                                  +RC*von_O(j1+1)
!          ELSE
!            vn_I( j2+jj-1 ,ksI ) = 0.0
!          END IF
        END DO
      END DO
      vn_I( jsI ,ksI ) = 0.0




!------------------------------------------------------------------------
! TOP (k2=keI)
!------------------------------------------------------------------------
!      k1 = keIO
!      k2 = keI-1

      vB = 0.0
      vT = 0.0
      von_O = 0.0

!... TIME_STEP interpolation

      DO j1=jsIO,jeIO
        vB(j1) = OC*vo_O(j1,keIO-1) +NC*vn_O(j1,keIO-1)
        vT(j1) = OC*vo_O(j1,keIO)   +NC*vn_O(j1,keIO)
      END DO

! BOTTOM-TOP vertical interpolation
      DO j1=jsIO,jeIO

!... BOTTOM
        IF ( MOD(incy, 2 ) .EQ. 1 ) THEN           ! odd inc
          BC = FLOAT( (incy+1) /2 ) /FLOAT(  incy  )
        ELSE != IF ( MOD(incy, 2 ) .EQ. 0 ) THEN   ! even inc
          BC = FLOAT(  incy+1     ) /FLOAT( incy*2 )
        END IF
!... TOP
        TC = 1-BC
        von_O(j1) = BC*vB(j1) +TC*vT(j1)
!        von_O(j1) = BC*vn_O(j1,keIO-1) +TC*vn_O(j1,keIO)

      END DO

      DO j1=jsIO,jeIO-1
        j2 = (j1-jsIO)*incx+1

        DO jj = 1,incx                                    
!          IF ( CELL1(j1,keIO)   .EQ. 1       &
!                              .AND.        &
!               CELL1(j1+1,keIO) .EQ. 1) THEN
            LC = 1 -FLOAT( jj-1 ) /FLOAT(incx)
            RC = 1-LC

            vn_I( j2+jj-1 ,keI-1 ) = LC*von_O(j1)    &
                                   +RC*von_O(j1+1)
!          ELSE
!            vn_I( j2+jj-1 ,keI-1 ) = 0.0
!          END IF
        END DO
      END DO
      vn_I( jsI ,keI-1 ) = 0.0


END SUBROUTINE

!-----------------------------------------------------------------------
SUBROUTINE DC_BOUND_INTERP_v_LR ( vn_I, vo_O, vn_O, mI,    &
                                  incx, incy, inct,        &
                                  jsO, jeO, ksO, keO,      &
                                  jsI, jeI, ksI, keI,      &
                                  jsIO, jeIO, ksIO, keIO )
!-----------------------------------------------------------------------
!     vn Interpolation from OUTER grid along Inner Domain Boundary
!                   (INPUT of Double-Coupling)
!---------------------------------------------------------------------
!
!  This program obtain vn_I (vn of the INNER grid) along the LEFT and
!  RIGHT INNER (fine grid) domain's bounries by interpolating vo_O
!  and vn_O (vo and vn of the OUTER grid) in both space (incx,incy)
!  and time (inct).
!
!
!   Z.Kowalik  2008  IMS, University of Alaska Fairbanks
!   Y.Yamazaki 2008  ORE, University of Hawaii Manoa
!---------------------------------------------------------------------
!      USE ALL_INPUTS
      IMPLICIT NONE

!  (INOUT)...
      REAL, DIMENSION(jsI:jeI,ksI:keI):: vn_I ! inner domain vn

!  (IN)...
      REAL, DIMENSION(jsO:jeO,ksO:keO):: vo_O ! outer domain vo
      REAL, DIMENSION(jsO:jeO,ksO:keO):: vn_O ! outer domain vn

      INTEGER:: incx                          ! dxO/dxI
      INTEGER:: incy                          ! dyO/dyI
      INTEGER:: inct                          ! dtO/dtI

      INTEGER:: jsO, jeO, ksO, keO
      INTEGER:: jsI, jeI, ksI, keI

      INTEGER:: jsIO, jeIO, ksIO, keIO        ! j, k indice at 4 corners
                                              ! of inner domain respect
                                              ! to outer grids

      INTEGER:: mI                            ! inner grid time step

!  (INTERNAL)...
      REAL, DIMENSION(ksO:keO):: von_O

      REAL:: OC,NC  ! TIME STEP  weighting coefficients

      REAL:: BC,TC  ! GRID SPACE weighting coefficients

      INTEGER:: j1, k1, j2, k2, jj, kk
      INTEGER:: kks, kke


! Time Weigthing Coefficients
      OC = FLOAT(inct-mI) /FLOAT(inct)        ! old time step weight = PM1/PM3
      NC = 1-OC                               ! new time step weight = PM2/PM3

!------------------------------------------------------------------------
! LEFT (j2=jsI)
!------------------------------------------------------------------------
     !j1=jsIO
     !j2 = jsI

!... TIME_STEP interpolation
      von_O = 0.0
!      DO k1=ksIO,keIO
      DO k1=ksIO-1,keIO
        von_O(k1) = OC*vo_O(jsIO,k1) +NC*vn_O(jsIO,k1)
      END DO

!... BOTTOM END CELLS
      k1 = ksIO
      k2 = ksI
      IF ( MOD(incy, 2 ) .EQ. 1 ) THEN           ! odd inc
        kks= (incy+1)/2                          ! 3
        DO kk = kks,incy
!          IF ( CELL1(jsIO,k1)   .EQ. 1       &
!                                .AND.        &
!               CELL1(jsIO,k1-1) .EQ. 1) THEN
            BC = 1 -FLOAT( kk ) /FLOAT(incy)
            TC = 1-BC

           !vn_I( jsI, k2+kk-1 )   = BC*von_O(jsIO,k1-1)    &
            vn_I( jsI, k2+kk-kks ) = BC*von_O(k1-1)    &
                                    +TC*von_O(k1)
!          ELSE
!            vn_I( jsI, k2+kk-kks ) = 0.0
!          END IF
        END DO

      ELSE ! IF ( MOD(incy, 2 ) .EQ. 0 ) THEN    ! even inc
        kks= incx/2+1
        DO kk = kks,incy
!          IF ( CELL1(jsIO,k1)   .EQ. 1       &
!                                .AND.        &
!               CELL1(jsIO,k1-1) .EQ. 1) THEN
            BC = 1 -FLOAT( kk*2-1 ) /FLOAT( incy*2 )
            TC = 1-BC
           !vn_I( jsI, k2+kk-1 )   = BC*von_O(jsIO,k1-1)    &
            vn_I( jsI, k2+kk-kks ) = BC*von_O(k1-1)    &
                                    +TC*von_O(k1)
!          ELSE
!            vn_I( jsI, k2+kk-kks ) = 0.0
!          END IF
        END DO

      END IF


!... INNER CELLS
      DO k1=ksIO+1,keIO-1
       !k2 = (k1-ksIO)*incy
        IF ( MOD(incy, 2 ) .EQ. 1 ) THEN           ! odd inc
          k2 = (k1-(ksIO+1))*incy +kks+1
        ELSE ! IF ( MOD(incy, 2 ) .EQ. 0 ) THEN    ! even inc
          k2 = (k1-(ksIO+1))*incy +kks
        END IF

        DO kk = 1,incy
!          IF ( CELL1(jsIO,k1)   .EQ. 1       &
!                                .AND.        &
!               CELL1(jsIO,k1-1) .EQ. 1) THEN
            IF ( MOD(incy, 2 ) .EQ. 1 ) THEN           ! odd inc
              BC = 1 -FLOAT( kk ) /FLOAT(incy)
            ELSE ! IF ( MOD(incy, 2 ) .EQ. 0 ) THEN    ! even inc
              BC = 1 -FLOAT( kk*2-1 ) /FLOAT( incy*2 )
            END IF
            TC = 1-BC

            vn_I( jsI, k2+kk-1 ) = BC*von_O(k1-1)    &
                                 +TC*von_O(k1)
!          ELSE
!            vn_I( jsI, k2+kk-1 ) = 0.0
!          END IF
        END DO
      END DO

!... TOP END CELLS
      k1=keIO
!      k2 = (k1-keIO)*incy
      IF ( MOD(incy, 2 ) .EQ. 1 ) THEN           ! odd inc

        kke= (incy-1)/2                          ! (5-1)/2=2
        k2 = keI-kke
        DO kk = 1,kke
!          IF ( CELL1(jsIO,k1)   .EQ. 1       &
!                                .AND.        &
!               CELL1(jsIO,k1-1) .EQ. 1) THEN
            BC = 1 -FLOAT( kk ) /FLOAT(incy)
            TC = 1-BC

            vn_I( jsI, k2+kk-1 ) = BC*von_O(k1-1)    &
                                 +TC*von_O(k1)
!          ELSE
!            vn_I( jsI, k2+kk-1 ) = 0.0
!          END IF
        END DO

      ELSE ! IF ( MOD(incy, 2 ) .EQ. 0 ) THEN    ! even inc

        kke= incx/2                               ! 2
        k2 = keI-kke
        DO kk = 1,kke
!          IF ( CELL1(jsIO,k1)   .EQ. 1       &
!                                .AND.        &
!               CELL1(jsIO,k1-1) .EQ. 1) THEN
            BC = 1 -FLOAT( kk*2-1 ) /FLOAT( incy*2 )
            TC = 1-BC

            vn_I( jsI, k2+kk-1 ) = BC*von_O(k1-1)    &
                                 +TC*von_O(k1)
!          ELSE
!            vn_I( jsI, k2+kk-1 ) = 0.0
!          END IF
        END DO

      END IF

!------------------------------------------------------------------------
! RIGHT (j2=jeI)
!------------------------------------------------------------------------
     !j1=jeIO
     !j2 = jeI

!... TIME_STEP interpolation
      von_O = 0.0
!      DO k1=ksIO,keIO
      DO k1=ksIO-1,keIO
        von_O(k1) = OC*vo_O(jeIO,k1) +NC*vn_O(jeIO,k1)
      END DO

!... BOTTOM END CELLS
      k1 = ksIO
      k2 = ksI
      IF ( MOD(incy, 2 ) .EQ. 1 ) THEN           ! odd inc
        kks= (incy+1)/2                          ! 3
        DO kk = kks,incy
!          IF ( CELL1(jeIO,k1)   .EQ. 1       &
!                                .AND.        &
!               CELL1(jeIO,k1-1) .EQ. 1) THEN
            BC = 1 -FLOAT( kk ) /FLOAT(incy)
            TC = 1-BC

           !vn_I( jeI, k2+kk-1 )   = BC*von_O(jeIO,k1-1)    &
            vn_I( jeI, k2+kk-kks ) = BC*von_O(k1-1)    &
                                   +TC*von_O(k1)
!          ELSE
!            vn_I( jeI, k2+kk-kks ) = 0.0
!          END IF
        END DO

      ELSE ! IF ( MOD(incy, 2 ) .EQ. 0 ) THEN    ! even inc
        kks= incx/2+1
        DO kk = kks,incy
!          IF ( CELL1(jeIO,k1)   .EQ. 1       &
!                                .AND.        &
!               CELL1(jeIO,k1-1) .EQ. 1) THEN
            BC = 1 -FLOAT( kk*2-1 ) /FLOAT( incy*2 )
            TC = 1-BC
           !vn_I( jeI, k2+kk-1 )   = BC*von_O(jeIO,k1-1)    &
            vn_I( jeI, k2+kk-kks ) = BC*von_O(k1-1)    &
                                   +TC*von_O(k1)
!          ELSE
!            vn_I( jeI, k2+kk-kks ) = 0.0
!          END IF
        END DO

      END IF


!... INNER CELLS
      DO k1=ksIO+1,keIO-1
       !k2 = (k1-ksIO)*incy
        IF ( MOD(incy, 2 ) .EQ. 1 ) THEN           ! odd inc
          k2 = (k1-(ksIO+1))*incy +kks+1
        ELSE ! IF ( MOD(incy, 2 ) .EQ. 0 ) THEN    ! even inc
          k2 = (k1-(ksIO+1))*incy +kks
        END IF

        DO kk = 1,incy
!          IF ( CELL1(jeIO,k1)   .EQ. 1       &
!                                .AND.        &
!               CELL1(jeIO,k1-1) .EQ. 1) THEN
            IF ( MOD(incy, 2 ) .EQ. 1 ) THEN           ! odd inc
              BC = 1 -FLOAT( kk ) /FLOAT(incy)
            ELSE ! IF ( MOD(incy, 2 ) .EQ. 0 ) THEN    ! even inc
              BC = 1 -FLOAT( kk*2-1 ) /FLOAT( incy*2 )
            END IF
            TC = 1-BC

            vn_I( jeI, k2+kk-1 ) = BC*von_O(k1-1)    &
                                 +TC*von_O(k1)
!          ELSE
!            vn_I( jeI, k2+kk-1 ) = 0.0
!          END IF
        END DO
      END DO

!... TOP END CELLS
      k1=keIO
!      k2 = (k1-keIO)*incy
      IF ( MOD(incy, 2 ) .EQ. 1 ) THEN           ! odd inc
        kke= (incy-1)/2                          ! (5-1)/2=2
        k2 = keI-kke

        DO kk = 1,kke
!          IF ( CELL1(jeIO,k1)   .EQ. 1       &
!                                .AND.        &
!               CELL1(jeIO,k1-1) .EQ. 1) THEN
            BC = 1 -FLOAT( kk ) /FLOAT(incy)
            TC = 1-BC

            vn_I( jeI, k2+kk-1 ) = BC*von_O(k1-1)    &
                               +TC*von_O(k1)
!          ELSE
!            vn_I( jeI, k2+kk-1 ) = 0.0
!          END IF
        END DO

      ELSE ! IF ( MOD(incy, 2 ) .EQ. 0 ) THEN    ! even inc
        kke= incx/2                               ! 2
        k2 = keI-kke
        DO kk = 1,kke

!          IF ( CELL1(jeIO,k1)   .EQ. 1       &
!                                .AND.        &
!               CELL1(jeIO,k1-1) .EQ. 1) THEN
            BC = 1 -FLOAT( kk*2-1 ) /FLOAT( incy*2 )
            TC = 1-BC

            vn_I( jeI, k2+kk-1 ) = BC*von_O(k1-1)    &
                                 +TC*von_O(k1)
!          ELSE
!            vn_I( jeI, k2+kk-1 ) = 0.0
!          END IF
        END DO

      END IF

END SUBROUTINE

!---------------------------------------------------------------------
SUBROUTINE DC_EXTRACT_zn ( zn_O, zn_I, H_O, H_I, CELL_O, CELL_I,   &
                           jsO, jeO, ksO, keO, jsI, jeI, ksI, keI, &
                           jsIO, ksIO, incx, incy, mI )
!---------------------------------------------------------------------
!     OUTTER level-grid's z Extraction from INNER level-grid's z
!             (Feedback process for Double-Coupling solver)
!---------------------------------------------------------------------
!
!  This program obtains the OUTTER level-grid's zeta, zn_O, extracted
!  from the INNER level-grid's zeta, zn_I. zn_I over the outer grid
!  space is averaged and feedbacked to update zn_O.
!
!   Z.Kowalik  2008  IMS, University of Alaska Fairbanks
!   Y.Yamazaki 2008  ORE, University of Hawaii Manoa
!---------------------------------------------------------------------
!      USE ALL_INPUTS
      IMPLICIT NONE

!  (INOUT)...
      REAL, DIMENSION(jsO:jeO,ksO:keO):: zn_O         ! outer grid zn

!  (IN)...
      REAL, DIMENSION(jsI:jeI,ksI:keI ):: zn_I        ! inner grid zn
      REAL, DIMENSION(jsI:jeI,ksI:keI ):: H_I         ! inner grid H
      REAL, DIMENSION(jsO:jeO,ksO:keO ):: H_O         ! outer grid zn

      INTEGER*2, DIMENSION(jsI:jeI,ksI:keI):: CELL_I
      INTEGER*2, DIMENSION(jsO:jeO,ksO:keO):: CELL_O
      INTEGER*2, DIMENSION(jsI:jeI,ksI:keI):: CELLI_O ! ANG marker

      INTEGER:: incx                                  ! dxO/dxI
      INTEGER:: incy                                  ! dyO/dyI
      INTEGER:: mI

      INTEGER:: jsO, jeO, ksO, keO
      INTEGER:: jsI, jeI, ksI, keI

      INTEGER:: jsIO, jeIO, ksIO, keIO

!  (INTERNAL)...
      INTEGER:: nfill, nhalf
      REAL:: zsum
      INTEGER:: incLR, incBT
      INTEGER:: znum
      INTEGER:: j1, k1, j2, k2, jj, kk


      IF ( MOD(incx, 2 ) .EQ. 1 ) THEN           ! odd inc
        incLR = (incx -1) /2
        incBT = (incy -1) /2
      ELSE != IF ( MOD(incx, 2 ) .EQ. 0 ) THEN   ! even inc
        incLR = incx /2
        incBT = incy /2
      END IF

      nfill  = (incLR*2+1)*(incBT*2+1)
      nhalf  = nfill /2

      IF ( mI .EQ. 1 ) THEN
        PRINT *,'incx=' ,incx
        PRINT *,'incy=' ,incy
        PRINT *,'incLR=',incLR
        PRINT *,'incBT=',incBT
        PRINT *,'nfill=',nfill
        PRINT *,'nhalf=',nhalf
      END IF

!...  extract z

      DO j2=(jsI+incx),(jeI-incx),incx     ! exclude z at boundaries
        DO k2=(ksI+incy),(keI-incy),incy   ! exclude z at boundaries
!          IF( CELLI_O(j2,k2) .EQ. 0 ) THEN
            j1 = (jsIO+1) +( ( j2 -(jsI+incx) ) /incx )
            k1 = (ksIO+1) +( ( k2 -(ksI+incy) ) /incy )

            ! sum INNER level-grid's z
            zsum = 0.0
            znum = 0
            DO jj=j2-incLR,j2+incLR
               DO kk=k2-incBT,k2+incBT
                 IF( CELL_I(jj,kk) .EQ. 1 ) THEN
!                 IF( CELLI_O(jj,kk) .EQ. 0        &
!                                    .AND.         &
!                      CELL_I(jj,kk) .EQ. 1 ) THEN
                   znum = znum+1
                 END IF
                 zsum = zsum +zn_I(jj,kk)
              END DO
            END DO

            ! feed back averaged z into OUTTER level-grid
            IF ( znum .EQ. nfill) THEN
              zn_O(j1,k1) = zsum /nfill  ! averaged z
             !zn_O(j1,k1) = zn_I(j2,k2)  ! exact    z
            END IF

!          END IF
        END DO
      END DO

!---------------------------------------------------------------------
! Feedback Options
!---------------------------------------------------------------------
!
!! COMCOT version
!            IF ( H_O(j1,k1) .GT. 0.0         &
!                            .AND.            &
!                       znum .GT. nhalf) THEN

!! Yamazaki verison 1
!            IF ( CELL_O(j1,k1) .EQ. 1           &
!                               .AND.            &
!                          znum .GT. nhalf) THEN

!! Yamazaki verison 2: only feedback to H_O when all z_I are wet and inside the Adaptive grid
!!                     use when H_O and H_I is consistent
!            IF ( znum .EQ. nfill) THEN
!
!! Yamazaki verison 3: only feedback to H_O when all z_I are wet and inside the Adaptive grid
!!                     use when H_O and H_I is not consistent
!            IF ( CELL_O(j1,k1) .EQ. 1           &
!                               .AND.            &
!                          znum .EQ. nfill) THEN
!              zn_O(j1,k1) = zsum /nfill  ! averaged z
!            END IF

END SUBROUTINE

!---------------------------------------------------------------------
SUBROUTINE DC_BOUND_SHORELINE_z ( z_O, z_I, H_O, H_I, Hb_I, eta_I, &
                                  D_I, CELL_O, CELL_I,             &
                                  jsO, jeO, ksO, keO,              &
                                  jsI, jeI, ksI, keI,              &
                                  jsIO, jeIO, ksIO, keIO,          &
                                  incx, incy, mI )
!---------------------------------------------------------------------
!     OUTTER level-grid's z Extraction from INNER level-grid's z
!             (Feedback process for Double-Coupling solver)
!---------------------------------------------------------------------
!
!  This program obtains the OUTTER level-grid's zeta, z_O, extracted
!  from the INNER level-grid's zeta, z_I. z_I over the outer grid
!  space is averaged and feedbacked to update z_O.
!
!   Z.Kowalik  2008  IMS, University of Alaska Fairbanks
!   Y.Yamazaki 2008  ORE, University of Hawaii Manoa
!---------------------------------------------------------------------
!      USE ALL_INPUTS
      IMPLICIT NONE

!  (INOUT)...
      REAL, DIMENSION(jsO:jeO,ksO:keO):: z_O         ! outer grid z

!  (IN)...
      REAL, DIMENSION(jsI:jeI,ksI:keI ):: z_I        ! inner grid z
      REAL, DIMENSION(jsI:jeI,ksI:keI ):: H_I        ! inner grid H
      REAL, DIMENSION(jsI:jeI,ksI:keI ):: Hb_I        ! inner grid H
      REAL, DIMENSION(jsI:jeI,ksI:keI ):: eta_I       ! inner grid H
      REAL, DIMENSION(jsI:jeI,ksI:keI ):: D_I        ! inner grid H

      REAL, DIMENSION(jsO:jeO,ksO:keO ):: H_O        ! outer grid z

      INTEGER*2, DIMENSION(jsI:jeI,ksI:keI):: CELL_I
      INTEGER*2, DIMENSION(jsO:jeO,ksO:keO):: CELL_O

      INTEGER:: incx                                  ! dxO/dxI
      INTEGER:: incy                                  ! dyO/dyI
      INTEGER:: mI

      INTEGER:: jsO, jeO, ksO, keO
      INTEGER:: jsI, jeI, ksI, keI

      INTEGER:: jsIO, jeIO, ksIO, keIO

!  (INTERNAL)...
      REAL:: dztmpx, dztmpy
      INTEGER:: j1, k1, j2, k2, jj2, kk2




!... LEFT
      j1 = jsIO
      j2 = jsI

      DO k1 = ksIO, keIO-1
        IF ( CELL_O( j1 ,k1) .EQ. 1          &
                             .AND.           &
             CELL_O(j1,k1+1) .EQ. 0) THEN

          kk2 = ksI+(k1-ksIO)*incy
          DO k2 = (kk2+1), (kk2+(incy-1))
            IF (CELL_I(j2+1,k2) .EQ. 1) THEN
              dztmpx     = z_I(j2+1,k2) -z_I(j2+2,k2)
              z_I(j2,k2) = z_I(j2+1,k2) +dztmpy
              IF (j2 .EQ.(kk2+1)) THEN
                dztmpx  = z_I(j2,k2-1) -z_I(j2,k2-2)
                z_I(j2,k2) = z_I(j2,k2-1) +dztmpy
              END IF

              Hb_I(j2,k2) = H_I(j2,k2) -eta_I(j2,k2)
              D_I(j2,k2) = z_I(j2,k2) +Hb_I(j2,k2)

              IF( D_I(j2,k2) .LT. EPSILON ) THEN
                z_I(j2,k2)  = -H_I(j2,k2) +eta_I(j2,k2)
                CELL_I(j2,k2) = 0
              ELSE
                CELL_I(j2,k2) = 1
              END IF

            END IF
          END DO

        END IF
      END DO

      DO k1 = keIO,ksIO+1 ,-1
        IF ( CELL_O(j1, k1 ) .EQ. 1          &
                             .AND.           &
             CELL_O(j1,k1-1) .EQ. 0) THEN

          kk2 = keI-(keIO-k1)*incy
          DO k2 = (kk2-1), (kk2-(incy-1)), -1
            IF (CELL_I(j2+1,k2) .EQ. 1) THEN
              dztmpx     = z_I(j2+1,k2) -z_I(j2+2,k2)
              z_I(j2,k2) = z_I(j2+1,k2) +dztmpx
              IF (j2 .EQ.(jj2-1)) THEN
                dztmpy  = z_I(j2,k2+1) -z_I(j2,k2+2)
                z_I(j2,k2) = z_I(j2,k2+1) +dztmpy
              END IF

              Hb_I(j2,k2) = H_I(j2,k2) -eta_I(j2,k2)
              D_I(j2,k2) = z_I(j2,k2) +Hb_I(j2,k2)

              IF( D_I(j2,k2) .LT. EPSILON ) THEN
                z_I(j2,k2)  = -H_I(j2,k2) +eta_I(j2,k2)
                CELL_I(j2,k2) = 0
              ELSE
                CELL_I(j2,k2) = 1
              END IF

            END IF
          END DO

        END IF
      END DO



!... RIGHT
      j1 = jeIO
      j2 = jeI

      DO k1 = ksIO, keIO-1
        IF ( CELL_O( j1 ,k1) .EQ. 1          &
                             .AND.           &
             CELL_O(j1,k1+1) .EQ. 0) THEN

          kk2 = ksI+(k1-ksIO)*incy
          DO k2 = (kk2+1), (kk2+(incy-1))
            IF (CELL_I(j2-1,k2) .EQ. 1) THEN
              dztmpx     = z_I(j2-1,k2) -z_I(j2-2,k2)
              z_I(j2,k2) = z_I(j2-1,k2) +dztmpy
              IF (j2 .EQ.(kk2+1)) THEN
                dztmpx  = z_I(j2,k2-1) -z_I(j2,k2-2)
                z_I(j2,k2) = z_I(j2,k2-1) +dztmpy
              END IF

              Hb_I(j2,k2) = H_I(j2,k2) -eta_I(j2,k2)
              D_I(j2,k2) = z_I(j2,k2) +Hb_I(j2,k2)

              IF( D_I(j2,k2) .LT. EPSILON ) THEN
                z_I(j2,k2)  = -H_I(j2,k2) +eta_I(j2,k2)
                CELL_I(j2,k2) = 0
              ELSE
                CELL_I(j2,k2) = 1
              END IF

            END IF
          END DO

        END IF
      END DO

      DO k1 = keIO,ksIO+1 ,-1
        IF ( CELL_O(j1, k1 ) .EQ. 1          &
                             .AND.           &
             CELL_O(j1,k1-1) .EQ. 0) THEN

          kk2 = keI-(keIO-k1)*incy
          DO k2 = (kk2-1), (kk2-(incy-1)), -1
            IF (CELL_I(j2-1,k2) .EQ. 1) THEN
              dztmpx     = z_I(j2-1,k2) -z_I(j2-2,k2)
              z_I(j2,k2) = z_I(j2-1,k2) +dztmpx
              IF (j2 .EQ.(jj2-1)) THEN
                dztmpy  = z_I(j2,k2+1) -z_I(j2,k2+2)
                z_I(j2,k2) = z_I(j2,k2+1) +dztmpy
              END IF

              Hb_I(j2,k2) = H_I(j2,k2) -eta_I(j2,k2)
              D_I(j2,k2) = z_I(j2,k2) +Hb_I(j2,k2)

              IF( D_I(j2,k2) .LT. EPSILON ) THEN
                z_I(j2,k2)  = -H_I(j2,k2) +eta_I(j2,k2)
                CELL_I(j2,k2) = 0
              ELSE
                CELL_I(j2,k2) = 1
              END IF

            END IF
          END DO

        END IF
      END DO








!... BOTTOM
      k1 = ksIO
      k2 = ksI

      DO j1 = jsIO, jeIO-1
        IF ( CELL_O( j1 ,k1) .EQ. 1          &
                             .AND.           &
             CELL_O(j1+1,k1) .EQ. 0) THEN

          jj2 = jsI+(j1-jsIO)*incx
          DO j2 = (jj2+1), (jj2+(incx-1))
            IF (CELL_I(j2,k2+1) .EQ. 1) THEN
              dztmpy     = z_I(j2,k2+1) -z_I(j2,k2+2)
              z_I(j2,k2) = z_I(j2,k2+1) +dztmpy
              IF (j2 .EQ.(jj2+1)) THEN
                dztmpx  = z_I(j2-1,k2) -z_I(j2-2,k2)
                z_I(j2,k2) = z_I(j2-1,k2) +dztmpx
              END IF

              Hb_I(j2,k2) = H_I(j2,k2) -eta_I(j2,k2)
              D_I(j2,k2) = z_I(j2,k2) +Hb_I(j2,k2)

              IF( D_I(j2,k2) .LT. EPSILON ) THEN
                z_I(j2,k2)  = -H_I(j2,k2) +eta_I(j2,k2)
                CELL_I(j2,k2) = 0
              ELSE
                CELL_I(j2,k2) = 1
              END IF

            END IF            
          END DO

        END IF
      END DO

      DO j1 = jeIO, jsIO+1 ,-1
        IF ( CELL_O( j1 ,k1) .EQ. 1          &
                             .AND.           &
             CELL_O(j1-1,k1) .EQ. 0) THEN

          jj2 = jeI-(jeIO-j1)*incx
          DO j2 = (jj2-1), (jj2-(incx-1)), -1
            IF (CELL_I(j2,k2+1) .EQ. 1) THEN
              dztmpy     = z_I(j2,k2+1) -z_I(j2,k2+2)
              z_I(j2,k2) = z_I(j2,k2+1) +dztmpy
              IF (j2 .EQ.(jj2-1)) THEN
                dztmpx  = z_I(j2+1,k2) -z_I(j2+2,k2)
                z_I(j2,k2) = z_I(j2+1,k2) +dztmpx
              END IF


              Hb_I(j2,k2) = H_I(j2,k2) -eta_I(j2,k2)
              D_I(j2,k2) = z_I(j2,k2) +Hb_I(j2,k2)

              IF( D_I(j2,k2) .LT. EPSILON ) THEN
                z_I(j2,k2)  = -H_I(j2,k2) +eta_I(j2,k2)
                CELL_I(j2,k2) = 0
              ELSE
                CELL_I(j2,k2) = 1
              END IF


            END IF
          END DO

        END IF
      END DO

!... TOP
      k1 = keIO
      k2 = keI

      DO j1 = jsIO, jeIO-1
        IF ( CELL_O( j1 ,k1) .EQ. 1          &
                             .AND.           &
             CELL_O(j1+1,k1) .EQ. 0) THEN

          jj2 = jsI+(j1-jsIO)*incx
          DO j2 = (jj2+1), (jj2+(incx-1))
            IF (CELL_I(j2,k2-1) .EQ. 1) THEN
              dztmpy     = z_I(j2,k2-1) -z_I(j2,k2-2)
              z_I(j2,k2) = z_I(j2,k2-1) +dztmpy
              IF (j2 .EQ.(jj2+1)) THEN
                dztmpx  = z_I(j2-1,k2) -z_I(j2-2,k2)
                z_I(j2,k2) = z_I(j2-1,k2) +dztmpx
              END IF

              Hb_I(j2,k2) = H_I(j2,k2) -eta_I(j2,k2)
              D_I(j2,k2) = z_I(j2,k2) +Hb_I(j2,k2)

              IF( D_I(j2,k2) .LT. EPSILON ) THEN
                z_I(j2,k2)  = -H_I(j2,k2) +eta_I(j2,k2)
                CELL_I(j2,k2) = 0
              ELSE
                CELL_I(j2,k2) = 1
              END IF

            END IF
          END DO

        END IF
      END DO

      DO j1 = jeIO, jsIO+1 ,-1
        IF ( CELL_O( j1 ,k1) .EQ. 1          &
                             .AND.           &
             CELL_O(j1-1,k1) .EQ. 0) THEN

          jj2 = jeI-(jeIO-j1)*incx
          DO j2 = (jj2-1), (jj2-(incx-1)), -1
            IF (CELL_I(j2,k2-1) .EQ. 1) THEN
              dztmpy     = z_I(j2,k2-1) -z_I(j2,k2-2)
              z_I(j2,k2) = z_I(j2,k2-1) +dztmpy
              IF (j2 .EQ.(jj2-1)) THEN
                dztmpx  = z_I(j2+1,k2) -z_I(j2+2,k2)
                z_I(j2,k2) = z_I(j2+1,k2) +dztmpx
              END IF

              Hb_I(j2,k2) = H_I(j2,k2) -eta_I(j2,k2)
              D_I(j2,k2) = z_I(j2,k2) +Hb_I(j2,k2)

              IF( D_I(j2,k2) .LT. EPSILON ) THEN
                z_I(j2,k2)  = -H_I(j2,k2) +eta_I(j2,k2)
                CELL_I(j2,k2) = 0
              ELSE
                CELL_I(j2,k2) = 1
              END IF

            END IF
          END DO

        END IF
      END DO





END SUBROUTINE

!---------------------------------------------------------------------
SUBROUTINE DC_BOUND_RADIATION ( zn, un, vn, znIO, unIO, vnIO, CELL,  &
                                H, Hu, Hv, etn, js, je, ks, ke )
!---------------------------------------------------------------------
!           Partial Radiation B.C. for Inner-Grid Boundary
!---------------------------------------------------------------------
!
!  This program implements the partial radiation boundary condition
!  along the innner grid boundaries.
!
!   Z.Kowalik  2008  IMS, University of Alaska Fairbanks
!   Y.Yamazaki 2009  ORE, University of Hawaii Manoa
!   L.Ning     2009  ORE, University of Hawaii Manoa
!---------------------------------------------------------------------
!      USE ALL_INPUTS
      IMPLICIT NONE

!  (INOUT)...
      REAL, DIMENSION(js:je,ks:ke):: zn
      REAL, DIMENSION(js:je,ks:ke):: un
      REAL, DIMENSION(js:je,ks:ke):: vn

!  (IN)...
      REAL, DIMENSION(js:je,ks:ke):: znIO      ! z interpolated from outer-grid
      REAL, DIMENSION(js:je,ks:ke):: unIO      ! u interpolated from outer-grid
      REAL, DIMENSION(js:je,ks:ke):: vnIO      ! v interpolated from outer-grid
      REAL, DIMENSION(js:je,ks:ke):: eto, etn
      REAL, DIMENSION(js:je,ks:ke):: H, Hu, Hv

      INTEGER*2, DIMENSION(js:je,ks:ke):: CELL

      INTEGER:: js, je, ks, ke

!  (INTERNAL)...
      REAL:: Dtmp
      INTEGER:: j, k

!... LEFT Partial Radiation B.C. at j = js
      DO k = ks,ke
        IF ( CELL(js,k) .EQ. 1 ) THEN
          Dtmp = H(js+1,k) -etn(js+1,k) +zn(js+1,k)             ! D (js+1,k)
!          Dtmp = Hu(js+1,k) +0.5*(  zn(js,k)  +zn(js+1,k) )  & ! Du(js+1,k)
!                            -0.5*( etn(js,k) +etn(js+1,k) )
          IF( Dtmp .LT. 0. ) THEN
            Dtmp=0.0
          END IF
          zn(js,k) = znIO(js,k)                                &
                    -( un(js+1,k) -unIO(js+1,k) )*SQRT(Dtmp/g)
        END IF
      END DO

!... RIGHT Partial Radiation B.C. at j = je
      DO k = ks,ke
        IF ( CELL(je,k) .EQ. 1 ) THEN
          Dtmp = H(je-1,k) -etn(je-1,k) +zn(je-1,k)           ! D (je,k)
!          Dtmp = Hu(je,k) +0.5*(  zn(je-1,k)  +zn(je,k) )  & ! Du(je,k)
!                          -0.5*( etn(je-1,k) +etn(je,k) )
          IF( Dtmp .LT. 0. ) THEN
            Dtmp=0.0
          END IF
          zn(je,k) = znIO(je,k)                               &
                    +( un(je,k) -unIO(je,k) )*SQRT(Dtmp/g)
        END IF
      END DO

!... BOTTOMT Partial Radiation B.C. at k = ks
      DO j=js,je
        IF ( CELL(j,ks) .EQ. 1 ) THEN
          Dtmp = H(j,ks+1) -etn(j,ks+1) +zn(j,ks+1)          ! D (j,ks)
!          Dtmp = Hv(j,ks) +0.5*(  zn(j,ks)  +zn(j,ks+1) ) & ! Dv(j,ks)
!                          -0.5*( etn(j,ks) +etn(j,ks+1) )
          IF( Dtmp .LT. 0. ) THEN
            Dtmp=0.
          END IF
          zn(j,ks) = znIO(j,ks)                               &
                    -( vn(j,ks) -vnIO(j,ks) )*SQRT(Dtmp/g)
        END IF
      END DO

!... TOP Partial Radiation B.C. at k = ks
      DO j=js,je
        IF ( CELL(j,ke) .EQ. 1 ) THEN
          Dtmp = H(j,ke-1) -etn(j,ke-1) +zn(j,ke-1)
!          Dtmp = Hv(j,ke-1) +0.5*(  zn(j,ke-1)  +zn(j,ke) ) & ! D (j,ke-1)
!                            -0.5*( etn(j,ke-1) +etn(j,ke) )   ! Dv(j,ke-1)
          IF( Dtmp .LT. 0. ) THEN
            Dtmp=0.
          END IF
          zn(j,ke) = znIO(j,ke)                               &
                    +( vn(j,ke-1) -vnIO(j,ke-1) )*SQRT(Dtmp/g)
        END IF
      END DO

END SUBROUTINE

!-----------------------------------------------------------------------
SUBROUTINE DC_INTERP_z0 (z0_I, z0_O, incx, incy,                  &
                        jsO, jeO, ksO, keO, jsI, jeI, ksI, keI,   &
                        jsIO, jeIO, ksIO, keIO, jnumIO )
!-----------------------------------------------------------------------
!        Cell-centered variable Interpolation of the OUTER grid
!                   (INPUT of Double-Coupling)
!------------------------------------------------------------------------
!
!  This program interpolates the cell-centered variables (ex, z, eta, D)
!  in the OUTER grid(domain), z0_O, to obtain the INNER grid variables,
!  z0_I.
!
!   Z.Kowalik  2009 IMS,      University of Alaska Fairbanks
!   Y.Yamazaki 2009 ORE,      University of Hawaii Manoa
!------------------------------------------------------------------------
!      USE ALL_INPUTS
      IMPLICIT NONE

!  (INOUT)...
      REAL, DIMENSION(jsI:jeI,ksI:keI) :: z0_I ! inner grid variables

!  (IN)...
      REAL, DIMENSION(jsO:jeO,ksO:keO) :: z0_O ! outer grid variables

      INTEGER:: incx                          ! dxO/dxI
      INTEGER:: incy                          ! dyO/dyI

      INTEGER:: jsO, jeO, ksO, keO
      INTEGER:: jsI, jeI, ksI, keI

      INTEGER:: jsIO, jeIO, ksIO, keIO        ! j, k indice at 4 corners
                                              ! of inner domain respect
                                              ! to outer grids
      INTEGER:: jnumIO

!  (INTERNAL)...
      REAL, DIMENSION(1:jnumIO,ksI:keI):: z0y_I

      REAL:: LC,RC,BC,TC
      INTEGER:: j1, k1, j2, k2, jj, kk

      z0y_I = 0.0
      z0_I  = 0.0

!---------------------------------------------------------------------
! Vertical Interpolation
!---------------------------------------------------------------------
      DO j1=jsIO,jeIO
        DO k1=ksIO,keIO-1
          k2 = (k1-ksIO)*incy +1
          DO kk = 1,incy                      !For incy = 3
          !BOTTOM weighting coeff.            ! kk =  1    2    3
           BC = 1 -FLOAT( kk-1 ) /FLOAT(incy) ! BC =  1   2/3   1/3
          !TOP weighting coeff.               ! TC =  0   1/3   2/3
           TC = 1-BC
          !INTERPOLATION
           z0y_I( j1-jsIO+1 , k2+kk-1) = BC*z0_O(j1, k1 )  &
                                        +TC*z0_O(j1,k1+1)
          END DO
        END DO
      END DO

      z0y_I(1:jnumIO, keI) = z0_O(jsIO:jeIO, keIO)

!---------------------------------------------------------------------
! Horizontal Interpolation
!---------------------------------------------------------------------

      DO k2=ksI,keI
        DO j1=1,jnumIO-1
          j2 = (j1-1)*incx +1
          DO jj = 1,incx                         !For incx = 3
             !LEFT weighting coeff.              ! jj =  1    2    3
              LC = 1 -FLOAT( jj-1 ) /FLOAT(incx) ! LC =  1   2/3   1/3
             !RIGHT weighting coeff.             ! RC =  0   1/3   2/3
              RC = 1-LC
             !INTERPOLATION
              z0_I( j2+jj-1 ,k2 ) = LC*z0y_I( j1 ,k2)    &
                                   +RC*z0y_I(j1+1,k2)
          END DO
        END DO
      END DO

      z0_I(jeI,ksI:keI) = z0y_I(jnumIO,ksI:keI)


END SUBROUTINE






! governing equations subrouitnes
!-----------------------------------------------------------------------

! 1. Hydrostatic Solvers (Non-linear Shallowa Water solution)

!------------------------------------------------------------------------
SUBROUTINE XMOMENT_1ST ( un,                          &
                         zo, uo, vo, eto, H, Hu, Du,  &
                         CELL, siz, csz, FCT, NL,     &
                         dt, dt_dx, dt_dy, gdt_dx,    &
                         js, je, ks, ke)
!------------------------------------------------------------------------
!
!   This program calculates u using the momentum equation considering
!   bottom dislocation Hb.
!
!
!   Z.Kowalik  2006 IMS,      University of Alaska Fairbanks
!   W.Knight   2007 NOAA/NWS, West Coast & Alaska Tsunami Warning Center
!   Y.Yamazaki 2007 ORE,      University of Hawaii Manoa
!------------------------------------------------------------------------
      !USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE

!  (OUT)...
      REAL, DIMENSION(js:je,ks:ke):: un  ! new zeta

!  (IN)...
      REAL, DIMENSION(js:je,ks:ke):: zo
      REAL, DIMENSION(js:je,ks:ke):: uo
      REAL, DIMENSION(js:je,ks:ke):: vo
      REAL, DIMENSION(js:je,ks:ke):: eto
      REAL, DIMENSION(js:je,ks:ke):: H, Hu
      REAL, DIMENSION(js:je,ks:ke):: Du

      INTEGER*2, DIMENSION(js:je,ks:ke):: CELL

      REAL, DIMENSION(ks:ke):: siz, csz

      REAL :: FCT, NL

      REAL::    dt
      REAL::    dt_dx, dt_dy, gdt_dx

      INTEGER:: js, je, ks, ke

!  (INTERNAL)...
      REAL, DIMENSION(js:je,ks:ke):: utmp
      REAL :: vbar, uv,DDU
      REAL :: U_P, U_N, V_P, V_N
      INTEGER:: j, k
!-----------------------------------------------------------------------
! LEFT (WEST) Boundary Condition (NO advective and Colioris terms)
! ( un at j = js+1 )
!-----------------------------------------------------------------------
!.. Radiation B.C. (OPEN B.C.)
      DO k= ks+1, ke-1
       DDU=DU(JS+1,K)
        IF(DU(Js+1,K).LE.2.)DDU=2.
        un(js+1,k) = uo(js+1,k)                                   &
                     -gdt_dx*( zo(js+1,k) -zo(js,k) )             &
                     -( g*n**2. / ( DDU**(1./3.) ) )       &
                       *dt*uo(js+1,k)*ABS(uo(js+1,k)) /DDU
!                    -f*dt*uo(js+1,k)*ABS(uo(js+1,k)) /Du(js+1,k)
      END DO

!-----------------------------------------------------------------------
! un calculation (Main LOOP)
!-----------------------------------------------------------------------

      DO k=ks+1,ke-1
        DO j=js+2,je-1
          IF ( CELL(j-1,k) .EQ. 1        &
                           .AND.         &
               CELL( j ,k) .EQ. 1 ) THEN

            U_P = 0.5*( uo(j,k) +ABS(uo(j,k)) )
            U_N = 0.5*( uo(j,k) -ABS(uo(j,k)) )

            vbar = 0.25*( vo(j,k) +vo(j-1,k) +vo(j-1,k-1) +vo(j,k-1) )
            V_P = 0.5*( vbar +ABS(vbar) )
            V_N = 0.5*( vbar -ABS(vbar) )

            uv = uo(j,k)*uo(j,k) +vbar*vbar

       ! MAG1= uv
       ! MAG=SQRT(uv)
       !  DU(J,K)=DHU(J,K)+0.5*(SLO(J-1,K)+SLO(J,K))&
       !  -0.5*(ETO(J-1,K)+ETO(J,K))
        DDU=DU(J,K)
        IF(DU(J,K).LE.2.)DDU=2.
         
            un(j,k) = uo(j,k)                                            &
                     +FCT*siz(k)*vbar                                    & ! Coliolis
                     +NL*dt*siz(k)*vbar*uo(j,k)/(csz(k)*R0)              & ! Coliolis
                     -dt_dx*( U_P*( uo( j , k ) -uo(j-1, k ) )           & ! advective term
                             +U_N*( uo(j+1, k ) -uo( j , k ) ) ) /csz(k) & !  u(du/dx)
                     -dt_dy*( V_P*( uo( j , k ) -uo( j ,k-1) )           & ! advective term
                             +V_N*( uo( j ,k+1) -uo( j , k ) ) )         & !  v(du/dy)
                     +gdt_dx*( zo(j-1,k) -zo(j,k) ) / csz(k)             & ! hydro-static term
                     -( g*n**2./( Ddu**(1./3.)))                     & ! n bottom friction
                       *dt*uo(j,k)*SQRT(uv) /Ddu
                    !-f*dt*uo(j,k)*MAG/Ddu                                 ! f bottom friction

          END IF

        END DO
      END DO

!-----------------------------------------------------------------------
! BOTTOM (SOUTH) Boundary Condition (NO advective term)
! ( un at k=ks )
!-----------------------------------------------------------------------
!.. Radiation B.C. (OPEN B.C.)
      DO j = js+1, je ! include LEFT and RIGHT corners
       DDU=DU(J,KS)
        IF(DU(J,KS).LE.2.)DDU=2.
        un(j,ks) = uo(j,ks)                               &
                   -gdt_dx*( zo(j,ks) -zo(j-1,ks) )       &
                   -( g*n**2. /( DDU**(1./3.) ) )    &
                     *dt*uo(j,ks)*ABS(uo(j,ks)) /DDU
!                  -f*dt*uo(j,ks)*ABS(uo(j,ks)) /Du(j,ks)
      END DO

!-----------------------------------------------------------------------
! TOP (NORTH) Boundary Condition (NO advective term)
! ( un at k=ke )
!-----------------------------------------------------------------------
!.. Radiation B.C. (OPEN B.C.)
      DO j = js+1, je ! include LEFT and RIGHT corners
       DDU=DU(J,KE)
        IF(DU(J,KE).LE.2.)DDU=2.
        un(j,ke) = uo(j,ke)                               &
                   -gdt_dx*( zo(j,ke) -zo(j-1,ke) )       &
                   -( g*n**2./( DDU**(1./3.) ) )     &
                     *dt*uo(j,ke)*ABS(uo(j,ke)) /DDU
!                  -f*dt*uo(j,ke)*ABS(uo(j,ke)) / Du(j,ke)
      END DO

!-----------------------------------------------------------------------
! RIGHT (EAST) Boundary Condition (NO advective term)
! ( un at j = je )
!-----------------------------------------------------------------------
!.. Radiation B.C. (OPEN B.C.)
      DO k=ks+1,ke-1
       DDU=DU(JE,K)
        IF(DU(JE,K).LE.2.)DDU=2.
        un(je,k) = uo(je,k)                               &
                   -gdt_dx*( zo(je,k) -zo(je-1,k) )       &
                   -( g*(n**2.)/( DDU**(1./3.) ) )   &
                     *dt*uo(je,k)*ABS(uo(je,k)) /DDU
!                  -f*dt*uo(je,k)*ABS(uo(je,k)) /Du(je,k)
      END DO


!-----------------------------------------------------------------------
! Space Filtering of un
!-----------------------------------------------------------------------
      utmp = un

      DO k=2,ke
        DO j=js+2,je-1
          un(j,k) = (1.-UALPHA)*utmp(j,k)               &
                       +UALPHA*0.25*(    utmp(j-1,k)    &
                                     +2.*utmp( j ,k)    &
                                        +utmp(j+1,k)  )
        END DO
      END DO

!-----------------------------------------------------------------------
! un Treatment at DRY-DRY CELL Interface
!-----------------------------------------------------------------------

      DO k=ks+1,ke-1

!     DRY - DRT CELL case
        DO j=js+1,je
          IF ( CELL( j ,k) .EQ. 0         &
                           .AND.          &
               CELL(j-1,k) .EQ. 0 ) THEN
            un(j,k) = 0.
          END IF
        END DO

!-----------------------------------------------------------------------
! un Treatment at DRY-WET CELL Interface
!-----------------------------------------------------------------------
!.. main treatment

!     WET - DRT CELL case
        DO j=js+1,je-1        ! CHANGE 1  TO 2 FOR RADIATION
          IF ( CELL( j ,k) .EQ. 1        &
                           .AND.         &
               CELL(j+1,k) .EQ. 0 ) THEN

            !     sea level - land height is (-)
            IF ( (zo(j,k) +H(j+1,k) -eto(j+1,k) ) .LE. EPSILON ) THEN
              un(j+1,k) = 0.
            END IF

          END IF

!     DRY - WET CELL case
          IF ( CELL(j-1,k) .EQ. 0        &
                          .AND.          &
               CELL( j ,k) .EQ. 1 ) THEN

            !     sea level - land height is (-)
            IF ( (zo(j,k) +H(j-1,k) -eto(j-1,k) ) .LE. EPSILON ) THEN
              un(j,k) = 0.
            END IF

          END IF
        END DO

      END DO


END SUBROUTINE


!------------------------------------------------------------------------
SUBROUTINE YMOMENT_1ST ( vn,                         &
                         zo, un, vo, eto, H, Hv, Dv, &
                         CELL, siv, csv, FCT, NL,    &
                         dt, dt_dx, dt_dy, gdt_dy,   &
                         js, je, ks, ke)
!------------------------------------------------------------------------
!
!   This program calculates u using the momentum equation considering
!   bottom dislocation Hb.
!
!
!   Z.Kowalik  2006 IMS,      University of Alaska Fairbanks
!   W.Knight   2007 NOAA/NWS, West Coast & Alaska Tsunami Warning Center
!   Y.Yamazaki 2007 ORE,      University of Hawaii Manoa
!------------------------------------------------------------------------
      !USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE

!  (OUT)...
      REAL, DIMENSION(js:je,ks:ke):: vn

!  (IN)...
      REAL, DIMENSION(js:je,ks:ke):: zo
      REAL, DIMENSION(js:je,ks:ke):: un
      REAL, DIMENSION(js:je,ks:ke):: vo
      REAL, DIMENSION(js:je,ks:ke):: eto
      REAL, DIMENSION(js:je,ks:ke):: H, Hv
      REAL, DIMENSION(js:je,ks:ke):: Dv

      INTEGER*2, DIMENSION(js:je,ks:ke):: CELL

      REAL, DIMENSION(ks:ke):: siv, csv

      REAL :: FCT, NL

      REAL::    dt
      REAL::    dt_dx, dt_dy, gdt_dy

      INTEGER:: js, je, ks, ke

!  (INTERNAL)...

      REAL, DIMENSION(js:je,ks:ke):: vtmp
      REAL :: ubar, uv,DDV
      REAL :: U_P, U_N, V_P, V_N
      INTEGER:: j, k



!-----------------------------------------------------------------------
! BOTTOM (SOUTH) Boundary Condition (NO advective term)
! ( vn at k=ks )
!-----------------------------------------------------------------------
!.. Radiation B.C. (OPEN B.C.)
      DO j=js,je
          DDV=DV(J,KS)
   	    IF(DV(J,KS).LE.2.)DDV=2.   ! DEFENCE AGAINST ZERO DENOM. IN BOTTOM FRICTION
          vn(j,ks)=vo(j,ks)                               &
                   -gdt_dy*( zo(j,ks+1) -zo(j,ks) )       &
                   -( g*n**2. /( DDV**(1./3.) ) )    &
                     *dt*vo(j,ks)*ABS(vo(j,ks)) /DDV
!                   -f*dt*vo(j,ks)*ABS(vo(j,ks)) /Dv(j,ks)
      END DO

!-----------------------------------------------------------------------
! vn calculation (Main LOOP)
!-----------------------------------------------------------------------
      DO j= js+1,je-1
        DO k=ks+1,ke-2
       !DO k=ks+1,ke-1  ! Kowalik & Knight 2007 version
          IF ( CELL(j, k ) .EQ. 1        &
                           .AND.         &
               CELL(j,k+1) .EQ. 1 ) THEN

            ubar = 0.25*( un(j,k) +un(j+1,k) +un(j+1,k+1) +un(j,k+1) )
            U_P = 0.5*( ubar +ABS(ubar) )
            U_N = 0.5*( ubar -ABS(ubar) )

            V_P=0.5*(vo(j,k)+ABS(vo(j,k)))
            V_N=0.5*(vo(j,k)-ABS(vo(j,k)))

            uv=ubar*ubar+vo(j,k)*vo(j,k)
            DDV=DV(J,K)
   	    IF(DV(J,K).LE.2.)DDV=2.   ! DEFENCE AGAINST ZERO DENOM. IN BOTTOM FRICTION
            vn(j,k) = vo(j,k)                                        &
                     -FCT*siv(k)*ubar                                &
                     -NL*dt*ubar*ubar*siv(k)/(csv(k)*R0)             &
                     -dt_dx*( U_P*( vo( j ,k) -vo(j-1,k) )           &
                             +U_N*( vo(j+1,k) -vo( j ,k) ) ) /csv(k) &
                     -dt_dy*( V_P*( vo(j, k ) -vo(j,k-1) )           &
                             +V_N*( vo(j,k+1) -vo(j, k ) ) )         &
                     +gdt_dy*( zo(j,k) -zo(j,k+1) )                  &
                     -( g*n**2./( DDV**(1./3.) ) )               &
                       *dt*vo(j,k)*SQRT(uv) /DDV
!                     -f*dt*vo(j,k)*MAG/Ddv
          END IF
        END DO
      END DO

!-----------------------------------------------------------------------
! TOP (NORTH) Boundary Condition (NO advective and Colioris terms)
! ( vn at k=ke-1 )
!-----------------------------------------------------------------------
!..  Radiation B.C. (OPEN B.C.)
      DO j = js, je
        IF ( CELL(j,ke-1) .EQ. 1       &
                          .AND.        &
             CELL(j,ke  ) .EQ. 1 ) THEN
                   DDV=DV(J,KE-1)
   	    IF(DV(J,Ke-1).LE.2.)DDV=2.   ! DEFENCE AGAINST ZERO DENOM. IN BOTTOM FRICTI
          vn(j,ke-1) = vo(j,ke-1)                                    &
                      -gdt_dy*( zo(j,ke) -zo(j,ke-1) )               &
                      -( g*n**2. /( DDV**(1./3.) ) )          &
                        *dt*vo(j,ke-1)*ABS(vo(j,ke-1)) /DDV
!                     -f*dt*vo(j,ke-1)*ABS(vo(j,ke-1))/Dv(j,ke-1)
        END IF
      END DO

!-----------------------------------------------------------------------
! LEFT (WEST) Boundary Condition (NO advective and Colioris terms)
! ( vn at j = js )
!-----------------------------------------------------------------------
!..  Radiation B.C. (OPEN B.C.)
      DO k=ks+1,ke-2 ! not cover TOP and BOTTOM corners
        IF ( CELL(js, k ) .EQ. 1         &
                          .AND.          &
             CELL(js,k+1) .EQ. 1 ) THEN
                 DDV=DV(JS,K)
   	    IF(DV(JS,K).LE.2.)DDV=2.   ! DEFENCE AGAINST ZERO DENOM. IN BOTTOM FRICTION
          vn(js,k) = vo(js,k)                              &
                    -gdt_dy*( zo(js,k+1) -zo(js,k) )       &
                    -( g*n**2./( DDV**(1./3.) ) )     &
                      *dt*vo(js,k)*ABS(vo(js,k)) /DDV
!                   -f*dt*vo(js,k)*ABS(vo(js,k))/Dv(js,k)
        END IF
      END DO

!-----------------------------------------------------------------------
! RIGHT (EAST) Boundary Condition (NO advective and Colioris terms)
! ( vn at j = je )
!-----------------------------------------------------------------------
!..  Radiation B.C. (OPEN B.C.)
      DO k=ks+1,ke-2  ! not cover TOP and BOTTOM corners
        IF ( CELL(je, k ) .EQ. 1        &
                          .AND.         &
             CELL(je,k+1) .EQ. 1 ) THEN
            DDV=DV(JE,K)
   	    IF(DV(JE,K).LE.2.)DDV=2.   ! DEFENCE AGAINST ZERO DENOM. IN BOTTOM FRICTION
          vn(je,k) = vo(je,k)                              &
                    -gdt_dy*( zo(je,k+1) -zo(je,k) )       &
                    -( g*n**2./( DDV**(1./3.) ) )     &
                      *dt*vo(je,k)*ABS(vo(je,k))/DDV
!                   -f*dt*vo(je,k)*ABS(vo(je,k))/Dv(je,k)
        END IF
      END DO


!-----------------------------------------------------------------------
! Space Filtering of vn
!-----------------------------------------------------------------------

     vtmp=vn

     DO j = js, je
       DO k=ks+1,ke-1
         vn(j,k) = (1.-UALPHA)*vtmp(j,k)               &
                      +UALPHA*0.25*(    vtmp(j,k-1)    &
                                    +2.*vtmp(j, k )    &
                                      +vtmp(j,k+1)  )
       END DO
     END DO

!-----------------------------------------------------------------------
! vn Treatment at DRY-DRY CELL Interface
!-----------------------------------------------------------------------
      DO j=js,je

!     DRY - DRT CELL case
        DO k=ks+1,ke-1
          IF ( CELL(j,k  ) .EQ. 0        &
                           .AND.         &
               CELL(j,k+1) .EQ. 0) THEN
            vn(j,k) = 0.
          END IF
        END DO

!-----------------------------------------------------------------------
! vn Treatment at DRY-WET CELL Interface
!-----------------------------------------------------------------------

!.. main treatment

!     WET - DRT CELL case
        DO k=ks+1,ke-1
          IF ( CELL(j,k  ) .EQ. 1        &
                           .AND.         &
               CELL(j,k+1) .EQ. 0 ) THEN

           !    sea level - land height is (-)
            IF ((zo(j,k) +H(j,k+1) -eto(j,k+1) ) .LE. EPSILON ) THEN
              vn(j,k) = 0.   ! v=0 at TOP WALL
            END IF
          END IF

!     DRY - WET CELL case
          IF ( CELL(j,k-1) .EQ. 0        &
                           .AND.         &
               CELL(j,k  ) .EQ. 1 ) THEN
           !    sea level - land height is (-)
            IF ((zo(j,k) +H(j,k-1) -eto(j,k-1) ) .LE. EPSILON ) THEN
              vn(j,k-1) = 0. ! v=0 at BOTTOM WALL
            END IF
          END IF
        END DO
      END DO


END SUBROUTINE

!------------------------------------------------------------------------
SUBROUTINE CONT ( zn,                      &
                  zo, etn, eto, un, vn,    &
                  Hu, Hv, CELL, csz, csv,  &
                  dt, dt_dx, dt_dy,        &
                  js, je, ks, ke         )
!------------------------------------------------------------------------
!      Continuity Equation solver with Upwind Flux Scheme
!      in Spherical Coordinate System
!------------------------------------------------------------------------
!
!   This program calculates z using the continuity equation aided by
!   the upwind flux scheme (Kowalik et al. 2005, or Yamazaki et al. 2008).
!
!
!   Z.Kowalik  2006  IMS, University of Alaska Fairbanks
!   Y.Yamazaki 2007  ORE, University of Hawaii Manoa
!------------------------------------------------------------------------
      !USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE
!  (OUT)...
      REAL, DIMENSION(js:je,ks:ke):: zn  ! new zeta

!  (IN)...
      REAL, DIMENSION(js:je,ks:ke):: zo
      REAL, DIMENSION(js:je,ks:ke):: un
      REAL, DIMENSION(js:je,ks:ke):: vn
      REAL, DIMENSION(js:je,ks:ke):: eto, etn
      REAL, DIMENSION(js:je,ks:ke):: Hu, Hv

      REAL, DIMENSION(ks:ke):: csz, csv

      REAL::    dt
      REAL::    dt_dx, dt_dy

      INTEGER*2, DIMENSION(js:je,ks:ke):: CELL

      INTEGER:: js, je, ks, ke

!  (INTERNAL)...
      REAL, DIMENSION(js:je,ks:ke):: xFLUX, yFLUX
      REAL:: U_P, U_N
      REAL:: V_P, V_N
      INTEGER:: j, k

      xFLUX=0.; yFLUX=0.
      U_P = 0. ; U_N = 0.
      V_P = 0. ; V_N = 0.

!------------------------------------------------------------------------
! FLUX along x with Upwind Flux scheme
!------------------------------------------------------------------------
      DO k = ks+1, ke-1
        DO j = js+1, je    ! for RIGHT OPEN BC
!        DO j = js+1, je-1 ! for other BC
          IF( CELL( j ,k) .EQ. 1        &
                          .AND.         &
              CELL(j-1,k) .EQ. 1 ) THEN

                U_P = 0.5*( un(j,k) +ABS( un(j,k) ) )
                U_N = 0.5*( un(j,k) -ABS( un(j,k) ) )

                xFLUX(j,k) = U_P* zo(j-1,k) +U_N *zo(j,k)  &
                            -U_P*eto(j-1,k) -U_N*eto(j,k)  &
                            +un(j,k)*Hu(j,k)

          END IF
        END DO
      END DO

!------------------------------------------------------------------------
! FLUX along y with Upwind Flux scheme
!------------------------------------------------------------------------
      DO j = js, je
        DO k = ks, ke-1
          IF( CELL(j, k ) .EQ. 1        &
                          .AND.         &
              CELL(j,k+1) .EQ. 1 ) THEN

                V_P = 0.5*( vn(j,k) +ABS( vn(j,k) ) )
                V_N = 0.5*( vn(j,k) -ABS( vn(j,k) ) )

                yFLUX(j,k) = V_P* zo(j,k) +V_N* zo(j,k+1)  &
                            -V_P*eto(j,k) -V_N*eto(j,k+1)  &
                            +vn(j,k)*Hv(j,k)

                yFLUX(j,k) = yFLUX(j,k)*csv(k) !try not use csv(k)

          END IF
        END DO
      END DO

!------------------------------------------------------------------------
! zn calculation (Main LOOP)
!------------------------------------------------------------------------
      DO k = ks+1, ke-1
        DO j = js+1, je-1

          zn(j,k)= zo(j,k)                                        &
                   +etn(j,k) -eto(j,k)                            &
                   -dt_dx*( xFLUX(j+1,k) -xFLUX(j, k ) ) /csz(k)  &
                   -dt_dy*( yFLUX( j ,k) -yFLUX(j,k-1) ) /csz(k)

        END DO
      END DO

END SUBROUTINE


! 2. Non-hydrostatic Correction Solvers

!------------------------------------------------------------------------
SUBROUTINE VERT_BOTTOM (wbn,                   &
                        un, vn, eto, etn, Hb,  &
                        CELL, csz, dx, dy, dt, &
                        js, je, ks, ke       )
!------------------------------------------------------------------------
!  Linialized z-Momentum Equation solver in Spherical coordinates
!------------------------------------------------------------------------
!
!  This program calculates vertical velocity at sea bottom, wb, using
!  the linialized z-momentum equation.
!
!   Z.Kowalik  2006  IMS, University of Alaska Fairbanks
!   Y.Yamazaki 2007  ORE, University of Hawaii Manoa
!------------------------------------------------------------------------
      !USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE

!  (OUT)...
      REAL, DIMENSION(js:je,ks:ke):: wbn

!  (IN)...
      REAL, DIMENSION(js:je,ks:ke):: un
      REAL, DIMENSION(js:je,ks:ke):: vn
      REAL, DIMENSION(js:je,ks:ke):: etn, eto
      REAL, DIMENSION(js:je,ks:ke):: Hb

      INTEGER*2, DIMENSION(js:je,ks:ke):: CELL

      REAL, DIMENSION(ks:ke):: csz

      REAL::    dx, dy, dt
      INTEGER:: js, je, ks, ke

!  (INTERNAL)...
      REAL :: ubar, vbar
      REAL :: U_P, U_N, V_P, V_N
      INTEGER :: j, k


      DO k =ks+1,ke-1
        DO j=js+1,je-1
!------------------------------------------------------------------------
!     UPOS and UNEG for Eq.(44)
!------------------------------------------------------------------------
          IF(CELL(j,k).EQ.1.AND.CELL(j+1,k).EQ.1)THEN
          !IF(CELL(j,k).EQ.1.AND.CELL(j-1,k).EQ.1)THEN  ! original Dr. Kowalik code
            ubar   = 0.5*( un(j,k) +un(j+1,k) )           ! ask which is correct
            U_P  = 0.5*( ubar +ABS(ubar) )
            U_N  = 0.5*( ubar -ABS(ubar) )
          END IF
!------------------------------------------------------------------------
!     VPOS and VNEG for Eq.(44)
!------------------------------------------------------------------------
          IF(CELL(j,k).EQ.1.AND.CELL(j,k-1).EQ.1)THEN
            vbar   = 0.5*( vn(j,k) +vn(j,k-1) )

            V_P  = 0.5*( vbar +ABS(vbar) )
            V_N  = 0.5*( vbar -ABS(vbar) )
          END IF

!------------------------------------------------------------------------
!     wb, Bottom Verical Velocity calculation using Eq.(44)
!------------------------------------------------------------------------
          IF( CELL(j,k) .EQ. 1 ) THEN
!          IF( CELL( j , k )  .EQ.  1      & This BC allows to handle vertical wall
!                            .AND.         & But need more test
!             CELL(j+1, k )  .EQ.  1       &
!                            .AND.         &
!             CELL(j-1, k )  .EQ.  1       &
!                            .AND.         &
!             CELL( j ,k+1 ) .EQ.  1       &
!                            .AND.         &
!             CELL( j ,k-1 ) .EQ.  1 )THEN
            IF ( ABS( etn(j,k) -eto(j,k) ) .GE. EPSILON ) THEN
!             wbn(j,k) = -( etn(j,k) - eto(j,k) ) /dt                      &
              wbn(j,k) = ( etn(j,k) - eto(j,k) ) /dt                       &
                         -U_P*( Hb(j  ,k  ) - Hb(j-1,k  ) )/( dx*csz(k) )  &
                         -U_N*( Hb(j+1,k  ) - Hb(j  ,k  ) )/( dx*csz(k) )  &
                         -V_P*( Hb(j  ,k  ) - Hb(j  ,k-1) )/dy             &
                         -V_N*( Hb(j  ,k+1) - Hb(j  ,k  ) )/dy
            ELSE
              wbn(j,k) = -U_P*( Hb(j  ,k  ) - Hb(j-1,k  ) )/( dx*csz(k) )  &
                         -U_N*( Hb(j+1,k  ) - Hb(j  ,k  ) )/( dx*csz(k) )  &
                         -V_P*( Hb(j  ,k  ) - Hb(j  ,k-1) )/dy             &
                         -V_N*( Hb(j  ,k+1) - Hb(j  ,k  ) )/dy
            END IF
          END IF

        END DO
      END DO

END SUBROUTINE

!------------------------------------------------------------------------
SUBROUTINE VERT_SURFACE (wsn,                     &
                         q, wso, wbo, wbn, Dlim,  &
                         CELL, dt, js, je, ks, ke)
!------------------------------------------------------------------------
!  Linialized z-Momentum Equation solver in Spherical coordinates
!------------------------------------------------------------------------
!
!  This program calculates vertical velocity at sea surface, ws, using
!  the non-hydrostatic pressure, q.
!
!
!   Z.Kowalik  2006 IMS University of Alaska Fairbanks
!   Y.Yamazaki 2007 ORE University of Hawaii Manoa
!------------------------------------------------------------------------
      !USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE

!  (OUT)...
      REAL, DIMENSION(js:je,ks:ke):: wsn

!  (IN)...
      REAL(dp), DIMENSION(js:je,ks:ke):: q
      REAL, DIMENSION(js:je,ks:ke):: wso, wbo, wbn
      REAL, DIMENSION(js:je,ks:ke):: Dlim
      INTEGER*2, DIMENSION(js:je,ks:ke):: CELL

      REAL::    dt

      INTEGER:: js, je, ks, ke

!  (INTERNAL)...
      INTEGER :: j, k

      DO k =ks+1,ke-1
        DO j =js+1,je-1
          IF (CELL(j, k ) .EQ. 1 ) THEN
            wsn(j,k) =  wso(j,k)                   &
                       +wbo(j,k)                   &
                       -wbn(j,k)                   &
                       +2.*dt*q(j,k) /Dlim(j,k)

            !wsn(j,k) =  wso(j,k)                   &
            !           +wbo(j,k)                   &
            !           -wbn(j,k)                   &
            !           +2.*t*q(j,k) /( rho*D(j,k) )
          END IF
        END DO
      END DO

END SUBROUTINE

!-------------------------------------------------------------------------
SUBROUTINE XMOMENT_2ND ( un,                               &
                         q, A1, zo, eto, H, Dlim,          &
                         CELL, csz, dt_dx, js, je, ks, ke)
!-------------------------------------------------------------------------
!  2nd-half timestep x-Momentum Equation solver in Spherical coordinates
!-------------------------------------------------------------------------
!
!  This program calculates 2nd-half timestep x-direction velocity, un,
!  using the 2nd-half timestep x-momentum equation.
!
!
!   Z.Kowalik  2006 IMS,      University of Alaska Fairbanks
!   Y.Yamazaki 2007 ORE,      University of Hawaii Manoa
!------------------------------------------------------------------------
      !USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE

!  (OUT)...
      REAL, DIMENSION(js:je,ks:ke):: un

!  (IN)...
      REAL(dp), DIMENSION(js:je,ks:ke):: q
      REAL(dp), DIMENSION(js:je,ks:ke):: A1
      REAL, DIMENSION(js:je,ks:ke):: zo
      REAL, DIMENSION(js:je,ks:ke):: eto
      REAL, DIMENSION(js:je,ks:ke):: H
      REAL, DIMENSION(js:je,ks:ke):: Dlim

      INTEGER*2, DIMENSION(js:je,ks:ke):: CELL

      REAL, DIMENSION(ks:ke):: csz

      REAL::    dt_dx

      INTEGER:: js, je, ks, ke

!  (INTERNAL)...
      REAL, DIMENSION(js:je,ks:ke):: uh
      REAL, DIMENSION(js:je,ks:ke):: utmp
      INTEGER :: j, k

      uh(js:je,ks:ke) = un(js:je,ks:ke)

      DO k=ks+1,ke-1
        DO j=js+1,je-1
          IF ( CELL( j ,k) .EQ. 1        &
                           .AND.         &
               CELL(j-1,k) .EQ. 1 ) THEN
             un(j,k) = uh(j,k)                                          &
                      -A1(j,k)*( q(j,k) +q(j-1,k) )*dt_dx /(csz(k)*2.)  &
                              -( q(j,k) -q(j-1,k) )*dt_dx /(csz(k)*2.)
            !un(j,k) = uh(j,k)                                          &
            !          -A1(j,k)*( q(j,k) +q(j-1,k) )*t / (rho*(hx*csz(k))) / 2  &
            !          -( q(j,k) -q(j-1,k) )*t / (rho*(hx*csz(k))) / 2
          END IF
        END DO
      END DO

! NOTE: code below Here is exact copy of XMOMENT_1ST

!-----------------------------------------------------------------------
! Space Filtering of un
!-----------------------------------------------------------------------
      utmp = un

      DO k=2,ke
        DO j=js+2,je-1
          un(j,k) = (1.-UALPHA)*utmp(j,k)               &
                       +UALPHA*0.25*(    utmp(j-1,k)    &
                                     +2.*utmp( j ,k)    &
                                        +utmp(j+1,k)  )
        END DO
      END DO

!-----------------------------------------------------------------------
! un Treatment at DRY-DRY CELL Interface
!-----------------------------------------------------------------------

      DO k=ks+1,ke-1

!     DRY - DRT CELL case
        DO j=js+1,je
          IF ( CELL( j ,k) .EQ. 0         &
                           .AND.          &
               CELL(j-1,k) .EQ. 0 ) THEN
            un(j,k) = 0.
          END IF
        END DO

!-----------------------------------------------------------------------
! un Treatment at DRY-WET CELL Interface
!-----------------------------------------------------------------------
!.. main treatment

!     WET - DRT CELL case
        DO j=js+1,je-1        ! CHANGE 1  TO 2 FOR RADIATION
          IF ( CELL( j ,k) .EQ. 1        &
                           .AND.         &
               CELL(j+1,k) .EQ. 0 ) THEN

            !     sea level - land height is (-)
            IF ( (zo(j,k) +H(j+1,k) -eto(j+1,k) ) .LE. EPSILON ) THEN
              un(j+1,k) = 0.
            END IF

          END IF

!     DRY - WET CELL case
          IF ( CELL(j-1,k) .EQ. 0        &
                          .AND.          &
               CELL( j ,k) .EQ. 1 ) THEN

            !     sea level - land height is (-)
            IF ( (zo(j,k) +H(j-1,k) -eto(j-1,k) ) .LE. EPSILON ) THEN
              un(j,k) = 0.
            END IF

          END IF
        END DO

      END DO

!      un( js, ks+1:ke-1 ) = un( js+1, ks+1:ke-1 )
!      un( je, ks+1:ke-1 ) = un( je-1, ks+1:ke-1 )
!      un( js:je, ks ) = un( js:je, ks+1 ) ! cover EAST corner
!      un( js:je, ke ) = un( js:je, ke-1 ) ! cover EAST corner

END SUBROUTINE

!-------------------------------------------------------------------------
SUBROUTINE YMOMENT_2ND ( vn,                          &
                         q, B1, zo, eto, H, Dlim,     &
                         CELL, dt_dy, js, je, ks, ke)
!-------------------------------------------------------------------------
!  2nd-half timestep y-Momentum Equation solver in Spherical coordinates
!-------------------------------------------------------------------------
!
!  This program calculates 2nd-half timestep y-direction velocity, un,
!  using the 2nd-half timestep y-momentum equation.
!
!
!   Z.Kowalik  2006 IMS,      University of Alaska Fairbanks
!   Y.Yamazaki 2007 ORE,      University of Hawaii Manoa
!------------------------------------------------------------------------
      !USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE

!  (OUT)...
      REAL, DIMENSION(js:je,ks:ke):: vn

!  (IN)...
      REAL(dp), DIMENSION(js:je,ks:ke):: q
      REAL(dp), DIMENSION(js:je,ks:ke):: B1
      REAL, DIMENSION(js:je,ks:ke):: zo
      REAL, DIMENSION(js:je,ks:ke):: eto
      REAL, DIMENSION(js:je,ks:ke):: H
      REAL, DIMENSION(js:je,ks:ke):: Dlim

      INTEGER*2, DIMENSION(js:je,ks:ke):: CELL

      REAL::    dt_dy

      INTEGER:: js, je, ks, ke

!  (INTERNAL)...
      REAL, DIMENSION(js:je,ks:ke):: vh
      REAL, DIMENSION(js:je,ks:ke):: vtmp
      INTEGER :: j, k

      vh(js:je,ks:ke) = vn(js:je,ks:ke)

      DO k = ks+1, ke-1
        DO j = js+1, je-1
          IF ( CELL(j, k ) .EQ. 1        &
                           .AND.         &
               CELL(j,k+1) .EQ. 1 ) THEN
            vn(j,k) = vh(j,k)                                 &
                     -B1(j,k)*( q(j,k+1) +q(j,k) )*dt_dy / 2. &
                             -( q(j,k+1) -q(j,k) )*dt_dy / 2.
           !vn(j,k) = vh(j,k)                                        &
           !         -B1(j,k)*( q(j,k+1) +q(j,k) )*t / (rho*hy) / 2 &
           !         -( q(j,k+1) -q(j,k) )*t / (rho*hy) / 2
          END IF
        END DO
      END DO

! NOTE: code below Here is exact copy of YMOMENT_1ST

!-----------------------------------------------------------------------
! Space Filtering of vn
!-----------------------------------------------------------------------

     vtmp=vn

     DO j = js, je
       DO k=ks+1,ke-1
         vn(j,k) = (1.-UALPHA)*vtmp(j,k)               &
                      +UALPHA*0.25*(    vtmp(j,k-1)    &
                                    +2.*vtmp(j, k )    &
                                      +vtmp(j,k+1)  )
       END DO
     END DO

!-----------------------------------------------------------------------
! vn Treatment at DRY-DRY CELL Interface
!-----------------------------------------------------------------------
      DO j=js,je

!     DRY - DRT CELL case
        DO k=ks+1,ke-1
          IF ( CELL(j,k  ) .EQ. 0        &
                           .AND.         &
               CELL(j,k+1) .EQ. 0) THEN
            vn(j,k) = 0.
          END IF
        END DO

!-----------------------------------------------------------------------
! vn Treatment at DRY-WET CELL Interface
!-----------------------------------------------------------------------

!.. main treatment

!     WET - DRT CELL case
        DO k=ks+1,ke-1
          IF ( CELL(j,k  ) .EQ. 1        &
                           .AND.         &
               CELL(j,k+1) .EQ. 0 ) THEN

           !    sea level - land height is (-)
            IF ((zo(j,k) +H(j,k+1) -eto(j,k+1) ) .LE. EPSILON ) THEN
              vn(j,k) = 0.   ! v=0 at TOP WALL
            END IF
          END IF

!     DRY - WET CELL case
          IF ( CELL(j,k-1) .EQ. 0        &
                           .AND.         &
               CELL(j,k  ) .EQ. 1 ) THEN
           !    sea level - land height is (-)
            IF ((zo(j,k) +H(j,k-1) -eto(j,k-1) ) .LE. EPSILON ) THEN
              vn(j,k-1) = 0. ! v=0 at BOTTOM WALL
            END IF
          END IF
        END DO
      END DO

!      vn( js, ks+1:ke-1 ) = vn( js+1, ks+1:ke-1 )
!      vn( je, ks+1:ke-1 ) = vn( je-1, ks+1:ke-1 )
!      vn( js:je, ks ) = vn( js:je, ks+1 ) ! cover EAST corner
!      vn( js:je, ke ) = vn( js:je, ke-1 ) ! cover EAST corner

END SUBROUTINE

!---------------------------------------------------------------------
SUBROUTINE NON_HYDRO_SIP_PIVOT ( q, An, Bn,                       &
                                 zo, un, vn, eto, wbo, wbn, wso,  &
                                 H, Dlim, CELL, csz, csv, dt,     &
                                 dx, dy, js, je, ks, ke, NJK )
!---------------------------------------------------------------------
!          Non-hydrostatic Pressure solver in Spherical coord.
!             with SIP solver with Neuman B.C. and Pivoting
!---------------------------------------------------------------------
!
!  This program calculates coefficients of the linear equation system
!  to obtain q using the Strongly Implicit Procedure (SIP) solver
!  for 5-diagonal, and both non-symmetric and symmetric matrices
!  (subroutine SIP2D) with Neuman B.C. and Pivoting.
!
!
!   Z.Kowalik  2006  IMS, University of Alaska Fairbanks
!   Y.Yamazaki 2007  ORE, University of Hawaii Manoa
!   J.Horrillo 2008  IMS, University of Alaska Fairbanks
!---------------------------------------------------------------------
!      USE ALL_INPUTS
      IMPLICIT NONE

!  (OUT)...
      REAL(dp), DIMENSION(js:je,ks:ke):: q
      REAL(dp), DIMENSION(js:je,ks:ke):: An, Bn

!  (IN)...
      REAL, DIMENSION(js:je,ks:ke):: zo
      REAL, DIMENSION(js:je,ks:ke):: un
      REAL, DIMENSION(js:je,ks:ke):: vn
      REAL, DIMENSION(js:je,ks:ke):: eto
      REAL, DIMENSION(js:je,ks:ke):: wbo, wbn, wso
      REAL, DIMENSION(js:je,ks:ke):: H
      REAL, DIMENSION(js:je,ks:ke):: Dlim

      INTEGER*2, DIMENSION(js:je,ks:ke):: CELL

      REAL, DIMENSION(ks:ke):: csz, csv

      REAL::    dt
      REAL::    dx, dy

      INTEGER:: js, je, ks, ke
      INTEGER:: NJK

!  (INTERNAL)...

  ! Coefficients declared in MODULE "ALL_INPUT_VARIABLES_DEC"
      REAL(dp), DIMENSION(:), ALLOCATABLE :: Phi   ! = q
      REAL(dp), DIMENSION(:), ALLOCATABLE :: AL, AR, AB, AT, AP, QP

      REAL(dp), PARAMETER:: AP_DRY  = 1.0d25            ! BEST for 15 sec

      REAL(dp), PARAMETER:: rho     = 1.0d0       ! rho(g/cm^3) : water density
     !REAL(dp), PARAMETER:: rho     = 1.0d3       ! rho(kg/m^3) : water density
      REAL(dp), DIMENSION( js:je, ks:ke ):: Ax, Ay
      REAL(dp), DIMENSION( js:je, ks:ke ):: qtmp    ! temp q for filtering

      REAL(dp):: RES0
      REAL(dp):: VOL, QUO

      REAL(dp):: Cx, Cy
      REAL(dp):: dAn, dBn


      INTEGER,  DIMENSION(1:je) :: LJ
      INTEGER :: j, k, jk
      INTEGER :: istat

      ALLOCATE( AL(NJK), AR(NJK), AB(NJK), AT(NJK), AP(NJK), QP(NJK), STAT=istat)
      ALLOCATE( Phi(NJK), STAT = istat)

      AR = NJK*0.0d0; AL = NJK*0.0d0
      AT = NJK*0.0d0; AB = NJK*0.0d0
      AP = NJK*0.0d0; QP = NJK*0.0d0

      Phi  = NJK*0.0d0

      q    = 0.0d0
      An   = 0.0d0
      Bn   = 0.0d0
      qtmp = 0.0d0

!.. convering 2D Array into 1D !
      DO j = js, je
        LJ(j) = (j-1)*(ke-ks+1)
      END DO

! NEW ////////////////////////////////////////////////////////////////////////////////
!!.. setting q = 0 on DRY cells
!      DO k = ks, ke
!        DO j = js, je
!           jk = LJ(j) + k
!           PHI(jk) = q(j,k)
!        END DO
!      END DO
! NEW END ////////////////////////////////////////////////////////////////////////////


!------------------------------------------------------------------------------
!  An,Coefficient (A1 in Kowalik 2006)
!------------------------------------------------------------------------------
      DO k = ks, ke
        DO j = js+1, je
          IF (CELL( j ,k) .EQ. 1       &
                          .AND.        &
              CELL(j-1,k) .EQ. 1 ) THEN
             An(j,k) = (                                       &
                          ( zo(j  ,k) -H(j  ,k) +eto(j  ,k) )  &
                         -( zo(j-1,k) -H(j-1,k) +eto(j-1,k) )  &
                         )                                      &
                          / ( Dlim(j,k) +Dlim(j-1,k) )
          END IF
        END DO
      END DO

!.. For LEFT Boundary (j = js)

      DO k = ks, ke
        IF (CELL( js ,k) .EQ. 1 ) THEN
          An(js,k) = An(js+1,k)
        END IF
      END DO


!------------------------------------------------------------------------------
!  Bn, Coefficient (B1 in Kowalik 2006)
!------------------------------------------------------------------------------
      DO k = ks, ke-1
        DO j = js, je
          IF (CELL(j, k ) .EQ. 1       &
                          .AND.        &
              CELL(j,k+1) .EQ. 1 ) THEN
             Bn(j,k) = (                                       &
                          ( zo(j,k+1) -H(j,k+1) +eto(j,k+1) )  &
                         -( zo(j,k)   -H(j,k)   +eto(j,k)   )  &
                         )                                      &
                          / ( Dlim(j,k+1) +Dlim(j,k) )
          END IF
        END DO
      END DO

!.. For TOP Boundary (k = ke-1)
      DO j = js, je
        IF (CELL( j ,ke) .EQ. 1 ) THEN
             Bn(j,ke) = Bn(j,ke-1)
        END IF
      END DO

!------------------------------------------------------------------------------
!  AL,AR,AT,AB,AP,QP Coefficients of Linear Equation System
!------------------------------------------------------------------------------
!      Cx = (0.5*dt)/(dx**2)
!      Cy = (0.5*dt)/(dy**2)
!
!           *  (dx**2) /(0.5*dt)
!
!      Cx = (0.5*dt)/(dx**2) * ( (dx**2)/(0.5*dt) )
!      Cy = (0.5*dt)/(dy**2) * ( (dx**2)/(0.5*dt) )
!         ||
      Cx = 1.0
      Cy = (dx**2)/(dy**2) !  1.0 for dx = dy

      DO k=ks+1,ke-1
        DO j=js+1,je-1

          jk = LJ(j) + k

          AL(jk) = (1. /(csz(k))**2 )*Cx*( -1. +An(j,k) ) 

          AR(jk) = (1. /(csz(k))**2 )*Cx*( -1. -An(j+1,k) ) 

          AB(jk) = ( csv(k-1) /csz(k) )*Cy*( -1. +Bn(j,k-1) )

          AT(jk) = ( csv( k ) /csz(k) )*Cy*( -1. -Bn(j,k) )

          AP(jk) = ( 1. /(csz(k))**2 )*Cx*( (1 +An(j,k)) +(1-An(j+1,k)) ) &
                  +( 1. /   csz(k)   )*Cy*(                               &
                                            csv( k )*( 1. -Bn(j, k ) )    &
                                           +csv(k-1)*( 1. +Bn(j,k-1) ) )  &
                  +4.0*(dx**2) /(Dlim(j,k)**2)
                 !+( (dx**2) /(0.5*dt) )*(2.0*dt) /(Dlim(j,k)**2)

          QP(jk) = ( -(        un(j+1,k)          -un(j, k ) ) /(dx*csz(k)) &
                     -( csv(k)*vn( j ,k) -csv(k-1)*vn(j,k-1) ) /(dy*csz(k)) &
                     -( wso(j,k) + wbo(j,k) -2.*wbn(j,k) )/Dlim(j,k)       )&
                   *( (dx**2) /(0.5*dt) )

                                   
        END DO
      END DO

!------------------------------------------------------------------------------
!  DRY CELL B.C. Version 1
!  set q=0.0 on DRY CELL and WET CELLs located next to DRY CELL
!------------------------------------------------------------------------------
      DO k = ks+1, ke-1
        DO j = js+1, je-1
          IF ( CELL(j,k) .EQ. 0) THEN
            jk = LJ(j) + k
            AP(jk)      = AP_DRY  !
            AP(jk+1)    = AP_DRY  ! TOP
            AP(jk-1)    = AP_DRY  ! BOTTOM
            AP(jk+ke)   = AP_DRY  ! RIGHT
            AP(jk-ke)   = AP_DRY  ! LEFT

            AP(jk+ke+1) = AP_DRY  ! TOP-RIGHT
            AP(jk+ke-1) = AP_DRY  ! BOTTOM-RIGHT
            AP(jk-ke+1) = AP_DRY  ! TOP-LEFT
            AP(jk-ke-1) = AP_DRY  ! BOTTOM-LEFT
         END IF
       END DO
     END DO

!------------------------------------------------------------------------------
!  Neuman B.C. for Computational Domain Boundary
!------------------------------------------------------------------------------

!..    LEFT Boundary
         j = js+1
         DO k = ks+1, ke-1
           jk = LJ(j) + k
           AP(jk) = AP(jk) + AL(jk)
           AL(jk) = 0.0d0
         END DO

!..    RIGHT Boundary
         j = je-1
         DO k = ks+1, ke-1
           jk = LJ(j) + k
           AP(jk) = AP(jk) + AR(jk)
           AR(jk) = 0.0d0
         END DO

!..    BOTTOM Boundary
         k = ks+1
         DO j = js+1, je-1
          jk = LJ(j) + k
           AP(jk) = AP(jk) + AB(jk)
           AB(jk) = 0.0d0
         END DO

!..    TOP Boundary
         k = ke-1
         DO j = js+1, je-1
           jk = LJ(j) + k
           AP(jk) = AP(jk) + AT(jk)
           AT(jk) = 0.0d0
        END DO

!      DEALLOCATE(AL, AR, AT, AB, AP, QP)
!      DEALLOCATE(Phi)
!--------------------------------------------------------------
!  Strongly Implicit Procedure (SIP) solver
!  solve q (=Phi) using CGSTAB solver
!--------------------------------------------------------------
      CALL SIP2D( NJK, LJ, AL, AR, AB, AT, AP, QP, Phi, js, je, ks, ke )

      DEALLOCATE(AL, AR, AT, AB, AP, QP)

!      ALLOCATE( Phi( NJK ))
!      IF ( MOD(m,mout) .EQ. 0 ) THEN
!           WRITE (80) ( ( Phi(LJ(j)+k), j = js,je ), k = ks,ke )
!      END IF

!.. convering 1D Array into 2D

!      ALLOCATE( Phi(NJK), STAT = istat)

      DO k = ks, ke
        DO j = js, je
           jk = LJ(j) + k
             q(j,k) = Phi(jk)
        END DO
      END DO

      DEALLOCATE(Phi)

      q( js, ks+1:ke-1 ) = q( js+1, ks+1:ke-1 )
      q( je, ks+1:ke-1 ) = q( je-1, ks+1:ke-1 )
      q( js:je, ks ) = q( js:je, ks+1 ) ! cover EAST corner
      q( js:je, ke ) = q( js:je, ke-1 ) ! cover EAST corner


!--------------------------------------------------------------
! Space Filtering of q
!--------------------------------------------------------------
      qtmp(js:je,ks:ke)=q(js:je,ks:ke)

      DO k = ks+1, ke-1
        DO j = js+1, je-1
!.. Kowalik and Murty 1993 p.147 Eq.3.145
         q(j,k) = (1. -QALPHA )*qtmp(j,k)                          &
                      +QALPHA*0.25*( qtmp(j-1, k ) +qtmp(j+1, k )  &
                                    +qtmp( j ,k-1) +qtmp( j ,k+1)  &
                                    -4*qtmp( j , k )  )
!!.. Kowalik and Murty 1993 p.145 Eq.3.135 (work!)
!          q(j,k) = qtmp(j,k) +0.5*QALPHA *(1.-QALPHA )                        &
!                   *(qtmp(j-1,k)+qtmp(j+1,k)+qtmp(j,k-1)+qtmp(j,k+1)-4*qtmp(j,k)) &
!                   +0.25*QALPHA **2*(qtmp(j+1,k-1)+qtmp(j-1,k-1)              &
!                   +qtmp(j-1,k+1)+qtmp(j+1,k+1)-4*qtmp(j,k))
        END DO
      END DO


      q( js, ks+1:ke-1 ) = q( js+1, ks+1:ke-1 )
      q( je, ks+1:ke-1 ) = q( je-1, ks+1:ke-1 )
      q( js:je, ks ) = q( js:je, ks+1 ) ! cover EAST corner
      q( js:je, ke ) = q( js:je, ke-1 ) ! cover EAST corner

END SUBROUTINE


!---------------------------------------------------------------------
SUBROUTINE NON_HYDRO_SIP_PIVOT_DC ( q, q_O, An, Bn, zo, un, vn,      &
                                    eto, wbo, wbn, wso, H, Dlim,     &
                                    CELL, csz, csv, dt,  dx, dy,     &
                                    js, je, ks, ke, NJK, mI, dm_out )
!---------------------------------------------------------------------
!          Non-hydrostatic Pressure solver in Spherical coord.
!             with SIP solver with Neuman B.C. and Pivoting
!---------------------------------------------------------------------
!
!  This program calculates coefficients of the linear equation system
!  to obtain q using the Strongly Implicit Procedure (SIP) solver
!  for 5-diagonal, and both non-symmetric and symmetric matrices
!  (subroutine SIP2D) with Neuman B.C. and Pivoting.
!
!
!   Z.Kowalik  2006  IMS, University of Alaska Fairbanks
!   Y.Yamazaki 2007  ORE, University of Hawaii Manoa
!   J.Horrillo 2008  IMS, University of Alaska Fairbanks
!---------------------------------------------------------------------
!      USE ALL_INPUTS
      IMPLICIT NONE

!  (OUT)...
      REAL(dp), DIMENSION(js:je,ks:ke):: q, q_O
      REAL(dp), DIMENSION(js:je,ks:ke):: An, Bn

!  (IN)...
      REAL, DIMENSION(js:je,ks:ke):: zo
      REAL, DIMENSION(js:je,ks:ke):: un
      REAL, DIMENSION(js:je,ks:ke):: vn
      REAL, DIMENSION(js:je,ks:ke):: eto
      REAL, DIMENSION(js:je,ks:ke):: wbo, wbn, wso
      REAL, DIMENSION(js:je,ks:ke):: H
      REAL, DIMENSION(js:je,ks:ke):: Dlim

      INTEGER*2, DIMENSION(js:je,ks:ke):: CELL

      REAL, DIMENSION(ks:ke):: csz, csv

      REAL::    dt
      REAL::    dx, dy

      INTEGER:: js, je, ks, ke
      INTEGER:: NJK
      INTEGER:: mI, dm_out

!  (INTERNAL)...

  ! Coefficients declared in MODULE "ALL_INPUT_VARIABLES_DEC"
      REAL(dp), DIMENSION(:), ALLOCATABLE :: Phi   ! = q
      REAL(dp), DIMENSION(:), ALLOCATABLE :: AL, AR, AB, AT, AP, QP

      REAL(dp), PARAMETER:: AP_DRY  = 1.0d25            ! BEST for 15 sec

      REAL(dp), PARAMETER:: rho     = 1.0d0       ! rho(g/cm^3) : water density
     !REAL(dp), PARAMETER:: rho     = 1.0d3       ! rho(kg/m^3) : water density
      REAL(dp), DIMENSION( js:je, ks:ke ):: Ax, Ay
      REAL(dp), DIMENSION( js:je, ks:ke ):: qtmp    ! temp q for filtering

      REAL(dp):: RES0
      REAL(dp):: VOL, QUO

      REAL(dp):: Cx, Cy
      REAL(dp):: dAn, dBn
      REAL(dp):: dq

      INTEGER,  DIMENSION(1:je) :: LJ
      INTEGER :: j, k, jk
      INTEGER :: istat

      ALLOCATE( AL(NJK), AR(NJK), AB(NJK), AT(NJK), AP(NJK), QP(NJK), STAT=istat)
      ALLOCATE( Phi(NJK), STAT = istat)

      AR = NJK*0.0d0; AL = NJK*0.0d0
      AT = NJK*0.0d0; AB = NJK*0.0d0
      AP = NJK*0.0d0; QP = NJK*0.0d0

      Phi  = NJK*0.0d0

      q    = 0.0d0
      An   = 0.0d0
      Bn   = 0.0d0
      qtmp = 0.0d0

!.. convering 2D Array into 1D !
      DO j = js, je
        LJ(j) = (j-1)*(ke-ks+1)
      END DO

!------------------------------------------------------------------------------
!  An,Coefficient (A1 in Kowalik 2006)
!------------------------------------------------------------------------------
      DO k = ks, ke
        DO j = js+1, je
          IF (CELL( j ,k) .EQ. 1       &
                          .AND.        &
              CELL(j-1,k) .EQ. 1 ) THEN
             An(j,k) = (                                       &
                          ( zo(j  ,k) -H(j  ,k) +eto(j  ,k) )  &
                         -( zo(j-1,k) -H(j-1,k) +eto(j-1,k) )  &
                         )                                      &
                          / ( Dlim(j,k) +Dlim(j-1,k) )
          END IF
        END DO
      END DO

!.. For LEFT Boundary (j = js)

      DO k = ks, ke
        IF (CELL( js ,k) .EQ. 1 ) THEN
          An(js,k) = An(js+1,k)
!          dAn = An(js+1,k) -An(js+2,k)
!          An(js,k) = An(js+1,k) +dAn
        END IF
      END DO


!------------------------------------------------------------------------------
!  Bn, Coefficient (B1 in Kowalik 2006)
!------------------------------------------------------------------------------
      DO k = ks, ke-1
        DO j = js, je
          IF (CELL(j, k ) .EQ. 1       &
                          .AND.        &
              CELL(j,k+1) .EQ. 1 ) THEN
             Bn(j,k) = (                                       &
                          ( zo(j,k+1) -H(j,k+1) +eto(j,k+1) )  &
                         -( zo(j,k)   -H(j,k)   +eto(j,k)   )  &
                         )                                      &
                          / ( Dlim(j,k+1) +Dlim(j,k) )
          END IF
        END DO
      END DO

!.. For TOP Boundary (k = ke-1)
      DO j = js, je
        IF (CELL( j ,ke) .EQ. 1 ) THEN
          Bn(j,ke) = Bn(j,ke-1)
!          dBn = Bn(j,ke-1) -Bn(j,ke-2)
!          Bn(j,ke) = Bn(j,ke-1) +dBn
        END IF
      END DO

!------------------------------------------------------------------------------
!  AL,AR,AT,AB,AP,QP Coefficients of Linear Equation System
!------------------------------------------------------------------------------
!      Cx = (0.5*dt)/(dx**2)
!      Cy = (0.5*dt)/(dy**2)
!
!           *  (dx**2) /(0.5*dt)
!
!      Cx = (0.5*dt)/(dx**2) * ( (dx**2)/(0.5*dt) )
!      Cy = (0.5*dt)/(dy**2) * ( (dx**2)/(0.5*dt) )
!         ||
      Cx = 1.0
      Cy = (dx**2)/(dy**2) !  1.0 for dx = dy

      DO k=ks+1,ke-1
        DO j=js+1,je-1

          jk = LJ(j) + k

          AL(jk) = (1. /(csz(k))**2 )*Cx*( -1. +An(j,k) ) 

          AR(jk) = (1. /(csz(k))**2 )*Cx*( -1. -An(j+1,k) ) 

          AB(jk) = ( csv(k-1) /csz(k) )*Cy*( -1. +Bn(j,k-1) )

          AT(jk) = ( csv( k ) /csz(k) )*Cy*( -1. -Bn(j,k) )

          AP(jk) = ( 1. /(csz(k))**2 )*Cx*( (1 +An(j,k)) +(1-An(j+1,k)) ) &
                  +( 1. /   csz(k)   )*Cy*(                               &
                                            csv( k )*( 1. -Bn(j, k ) )    &
                                           +csv(k-1)*( 1. +Bn(j,k-1) ) )  &
                  +4.0*(dx**2) /(Dlim(j,k)**2)
                 !+( (dx**2) /(0.5*dt) )*(2.0*dt) /(Dlim(j,k)**2)

          QP(jk) = ( -(        un(j+1,k)          -un(j, k ) ) /(dx*csz(k)) &
                     -( csv(k)*vn( j ,k) -csv(k-1)*vn(j,k-1) ) /(dy*csz(k)) &
                     -( wso(j,k) + wbo(j,k) -2.*wbn(j,k) )/Dlim(j,k)       )&
                   *( (dx**2) /(0.5*dt) )

                                   
        END DO
      END DO

!------------------------------------------------------------------------------
!  DRY CELL B.C. Version 1
!  set q=0.0 on DRY CELL and WET CELLs located next to DRY CELL
!------------------------------------------------------------------------------
      DO k = ks+1, ke-1
        DO j = js+1, je-1
          IF ( CELL(j,k) .EQ. 0) THEN
            jk = LJ(j) + k
            AP(jk)      = AP_DRY  !
            AP(jk+1)    = AP_DRY  ! TOP
            AP(jk-1)    = AP_DRY  ! BOTTOM
            AP(jk+ke)   = AP_DRY  ! RIGHT
            AP(jk-ke)   = AP_DRY  ! LEFT

            AP(jk+ke+1) = AP_DRY  ! TOP-RIGHT
            AP(jk+ke-1) = AP_DRY  ! BOTTOM-RIGHT
            AP(jk-ke+1) = AP_DRY  ! TOP-LEFT
            AP(jk-ke-1) = AP_DRY  ! BOTTOM-LEFT
         END IF
       END DO
     END DO


!------------------------------------------------------------------------------
!  Dirichlet B.C.
!------------------------------------------------------------------------------

! (1). set Phi

     !along computational boundaries

!... LEFT boundary
      j=js
      DO k=ks+1,ke-1
        jk = LJ(j) + k
        Phi(jk) = q_O(j,k)
      END DO

!... RIGHT boundary
      j=je
      DO k=ks+1,ke-1
        jk = LJ(j) + k
        Phi(jk) = q_O(j,k)
      END DO

!... BOTTOM boundary
      k=ks
      DO j=js,je
        jk = LJ(j) + k
        Phi(jk) = q_O(j,k)
      END DO

!... TOP boundary
      k=ke
      DO j=js,je
        jk = LJ(j) + k
        Phi(jk) = q_O(j,k)
      END DO


! (2). set coefficients for [A]

!      along computational boundaries

!... LEFT boundary
      j = js+1
      DO k=ks+1,ke-1
        jk = LJ(j) +k
        QP(jk) = QP(jk) -AL(jk)*Phi(jk-ke)
        AL(jk) = 0.0
      END DO

!... RIGHT boundary
      j = je-1
      DO k=ks+1,ke-1
        jk = LJ(j) +k
        QP(jk) = QP(jk) -AR(jk)*Phi(jk+ke)
        AR(jk) = 0.0
      END DO


!... BOTTOM boundary
      k = ks+1
      DO j=js+1,je-1
        jk = LJ(j) +k
        QP(jk) = QP(jk) -AB(jk)*Phi(jk-1)
        AB(jk) = 0.0
      END DO

!... TOP boundary
      k = ke-1
      DO j=js+1,je-1
        jk = LJ(j) +k
        QP(jk) = QP(jk) -AT(jk)*Phi(jk+1)
        AT(jk) = 0.0
      END DO


!      DEALLOCATE(AL, AR, AT, AB, AP, QP)
!      DEALLOCATE(Phi)
!--------------------------------------------------------------
!  Strongly Implicit Procedure (SIP) solver
!  solve q (=Phi) using CGSTAB solver
!--------------------------------------------------------------
      CALL SIP2D( NJK, LJ, AL, AR, AB, AT, AP, QP, Phi, js, je, ks, ke )

      DEALLOCATE(AL, AR, AT, AB, AP, QP)

!      ALLOCATE( Phi( NJK ))
!      IF ( MOD(m,mout) .EQ. 0 ) THEN
!           WRITE (80) ( ( Phi(LJ(j)+k), j = js,je ), k = ks,ke )
!      END IF

!.. convering 1D Array into 2D

!      ALLOCATE( Phi(NJK), STAT = istat)

      DO k = ks, ke
        DO j = js, je
           jk = LJ(j) + k
           q(j,k) = Phi(jk)
        END DO
      END DO

      DEALLOCATE(Phi)

!      q( js, ks+1:ke-1 ) = q_O( js, ks+1:ke-1 )
!      q( je, ks+1:ke-1 ) = q_O( je, ks+1:ke-1 )
!      q( js:je, ks ) = q_O( js:je, ks ) ! cover EAST corner
!      q( js:je, ke ) = q_O( js:je, ke ) ! cover EAST corner


!--------------------------------------------------------------
! Space Filtering of q
!--------------------------------------------------------------
      qtmp(js:je,ks:ke)=q(js:je,ks:ke)

      DO k = ks+1, ke-1
        DO j = js+1, je-1
!.. Kowalik and Murty 1993 p.147 Eq.3.145
         q(j,k) = (1. -QALPHA )*qtmp(j,k)                          &
                      +QALPHA*0.25*( qtmp(j-1, k ) +qtmp(j+1, k )  &
                                    +qtmp( j ,k-1) +qtmp( j ,k+1)  &
                                    -4*qtmp( j , k )  )
!!.. Kowalik and Murty 1993 p.145 Eq.3.135 (work!)
!          q(j,k) = qtmp(j,k) +0.5*QALPHA *(1.-QALPHA )                            &
!                   *(qtmp(j-1,k)+qtmp(j+1,k)+qtmp(j,k-1)+qtmp(j,k+1)-4*qtmp(j,k)) &
!                   +0.25*QALPHA **2*(qtmp(j+1,k-1)+qtmp(j-1,k-1)                  &
!                   +qtmp(j-1,k+1)+qtmp(j+1,k+1)-4*qtmp(j,k))
        END DO
      END DO

!      q( js, ks+1:ke-1 ) = q_O( js, ks+1:ke-1 )
!      q( je, ks+1:ke-1 ) = q_O( je, ks+1:ke-1 )
!      q( js:je, ks ) = q_O( js:je, ks ) ! cover EAST corner
!      q( js:je, ke ) = q_O( js:je, ke ) ! cover EAST corner


END SUBROUTINE



! linear algorithm solvers

!---------------------------------------------------------------------
SUBROUTINE SIP2D( NJK, LJ, AL, AR, AB, AT, AP, QP, Phi, &
                  js, je, ks, ke )
!---------------------------------------------------------------------
!  Strongly Implicit Procedure (Incomplete LU solver of Stone, 1968)
!                      for both Non-symmetric and Symmetric Matrices
!---------------------------------------------------------------------
!
!  This program calculates Phi of the linear equation system using
!  the strongly implicit procedure (SIP) for 5-diagonal, and both
!  non-symmetric and symmetric matrices.
!
!   The original code was written by
!        M. Peric, Institut fuer Schiffbau, Hamburg, 1995.
!   (See Ferzinger and Peric (2002) 5.3.4, P.101).
!
!
!   Y.Yamazaki 2008  ORE, University of Hawaii Manoa
!---------------------------------------------------------------------
!      USE ALL_INPUTS
      IMPLICIT NONE

!.. Setup Paramters
!---------------------------------------------------------------------

   ! Maximum number of allowed Iteration
      INTEGER,  PARAMETER:: MAXIT  = 2000     ! Yamazaki et al 2008
!      INTEGER,  PARAMETER:: MAXIT  = 200000
   ! Normalized Residuel Sum
      REAL(dp), PARAMETER:: RESMAX = 0.001d0  ! Yamazaki et al 2008
!      REAL(dp), PARAMETER:: RESMAX = 1.0d-6

      REAL(dp), PARAMETER:: EPS = 1.0d-20
!      REAL(dp), PARAMETER:: EPS = 1.0d-5

      REAL(dp), PARAMETER:: ALFA = 0.92d0
!---------------------------------------------------------------------
      INTEGER:: js, je, ks, ke
      INTEGER, DIMENSION(1:je):: LJ
      INTEGER:: NJK

  ! Coefficients declared in MODULE "ALL_INPUT_VARIABLES_DEC"
!      REAL(dp), DIMENSION(:), ALLOCATABLE :: Phi   ! = q
!      REAL(dp), DIMENSION(:), ALLOCATABLE :: AL, AR, AB, AT, AP, QP
      REAL(dp), DIMENSION(NJK) :: Phi   ! = q
      REAL(dp), DIMENSION(NJK) :: AL, AR, AB, AT, AP, QP

      REAL(dp), DIMENSION(:), ALLOCATABLE :: RES
      REAL(dp), DIMENSION(:), ALLOCATABLE :: MP

      REAL(dp), DIMENSION(:), ALLOCATABLE :: LL, LB ! for SIP preconditioner
      REAL(dp), DIMENSION(:), ALLOCATABLE :: UR, UT ! for SIP preconditioner

      REAL(dp):: P1, P2                           ! for SIP preconditioner
      REAL(dp):: RES0, RESN, RSM

      INTEGER :: itnum
      INTEGER :: j, k, jk, nit
      INTEGER :: istat

!      ALLOCATE( AL(NJK), AR(NJK), AB(NJK), AT(NJK), AP(NJK), QP(NJK), STAT=istat)
!      ALLOCATE( Phi(NJK), STAT = istat)
      ALLOCATE( RES(NJK), MP(NJK), STAT = istat )
      ALLOCATE( LL(NJK),  LB(NJK), STAT = istat )
      ALLOCATE( UR(NJK),  UT(NJK), STAT = istat )

      UR  = NJK*0.0d0; UT = NJK*0.0d0; RES = NJK*0.0d0

!.....CALCULATE ELEMENTS OF [L] AND [U] MATRICES

      DO j = js+1,je-1                             ! DO I=2,NIM
        DO jk = LJ(j)+ks+1, LJ(j)+ke-1             !   DO IJ=LI(I)+2,LI(I)+NJM
          LL(jk) = AL(jk) /(1. +ALFA*UT(jk-ke) )   !     LW(IJ)=AW(IJ)/(1.+ALFA*UN(IJ-NJ))
          LB(jk) = AB(jk) /(1. +ALFA*UR(jk-1) )    !     LS(IJ)=AS(IJ)/(1.+ALFA*UE(IJ-1))

          P1 = ALFA*LL(jk)*UT(jk-ke)               !     P1=ALFA*LW(IJ)*UN(IJ-NJ)
          P2 = ALFA*LB(jk)*UR(jk-1)                !     P2=ALFA*LS(IJ)*UE(IJ-1)

          MP(jk) = 1.0d0 /( AP(JK) +P1 +P2      &  !     LPR(IJ) = 1. /( AP(IJ) +P1 +P2    &
                          -LL(JK)*UR(jk-ke)     &  !                    -LW(IJ)*UE(IJ-NJ)  &
                          -LB(JK)*UT(jk-1)    )    !                    -LS(IJ)*UN(IJ-1)  )

          UT(jk) = ( AT(jk) -P1 )*MP(jk)           !     UN(IJ)=(AN(IJ)-P1)*LPR(IJ)
          UR(jk) = ( AR(jk) -P2 )*MP(jk)           !     UE(IJ)=(AE(IJ)-P2)*LPR(IJ)
        END DO
      END DO

!---------------------------------------------------------------------
! START ITERATION
!---------------------------------------------------------------------
      itnum=1

!.....CALCULATE RESIDUAL AND AUXILLIARY VECTORS; INNER ITERATION LOOP
      DO nit=1,MAXIT ! DO N=1,MAXIT
        itnum = itnum+1

        RESN=0.0d0

        DO j = js+1,je-1                             ! DO I=2,NIM
          DO jk = LJ(j)+ks+1, LJ(j)+ke-1             !   DO IJ=LI(I)+2,LI(I)+NJM
            RES(jk) = QP(jk)              &
                      -AP(jk)*Phi(jk)     &
                      -AT(jk)*Phi(jk+1)   &
                      -AB(jk)*Phi(jk-1)   &
                      -AR(jk)*Phi(jk+ke)  &
                      -AL(jk)*Phi(jk-ke)

            RESN = RESN+ABS(RES(jk))

            RES(jk) = ( RES(jk)                     &
                       -LB(jk)*RES(jk-1)            &
                       -LL(jk)*RES(jk-ke) )*MP(jk)

          END DO
        END DO


        IF( nit .EQ. 1 ) THEN
          RES0 = RESN                   ! IF(N.EQ.1) RES0 = RESN
        END IF

!.....CALCULATE INCREMENT AND CORRECT VARIABLE

        DO j = je-1, js+1, -1                 ! DO I=NIM,2,-1
          DO jk = LJ(j)+ke-1, LJ(j)+ks+1, -1  !   DO IJ=LI(I)+NJM,LI(I)+2,-1
            RES(jk) = RES(jk)             &
                      -UT(jk)*RES(jk+1)   &
                      -UR(jk)*RES(jk+ke)
            Phi(jk) = Phi(jk) +RES(jk)
          END DO
        END DO

!.....CONVERGENCE CHECK

        RSM = RESN /( RES0 +EPS )

        IF( RSM .LT. RESMAX ) THEN
          DEALLOCATE(RES, MP, LL, LB, UR, UT)
!          DEALLOCATE(AL, AR, AT, AB, AP, QP)
!          DEALLOCATE(Phi)
          RETURN
        END IF

      END DO


!---------------------------------------------------------------------
! END ITERATION
!---------------------------------------------------------------------

      DEALLOCATE(RES, MP, LL, LB, UR, UT)
!      DEALLOCATE(AL, AR, AT, AB, AP, QP)
!      DEALLOCATE(Phi)

END SUBROUTINE



! runup solvers

!---------------------------------------------------------------------
SUBROUTINE RUNUP ( CELL, D, uo, vo,  &
                   zo, H, eto,       &
                   js, je, ks, ke  )
!---------------------------------------------------------------------
!                              Runup solver
!                       Yamazaki et al 2008 version
!---------------------------------------------------------------------
!
!
!   Z.Kowalik  2006 IMS,      University of Alaska Fairbanks
!   W.Knight   2007 NOAA/NWS, West Coast & Alaska Tsunami Warning Center
!   Y.Yamazaki 2007 ORE,      University of Hawaii Manoa
!---------------------------------------------------------------------
!      USE ALL_INPUTS
      IMPLICIT NONE

!  (INOUT)...
      INTEGER*2, DIMENSION(js:je,ks:ke):: CELL

      REAL, DIMENSION(js:je,ks:ke):: D
      REAL, DIMENSION(js:je,ks:ke):: uo, vo

!  (IN)...
      REAL, DIMENSION(js:je,ks:ke):: zo
      REAL, DIMENSION(js:je,ks:ke):: H
      REAL, DIMENSION(js:je,ks:ke):: eto
      INTEGER:: js, je, ks, ke

!  (INTERNAL)...
      INTEGER*2, DIMENSION(js:je,ks:ke):: CELLX,CELLY
      REAL    :: DRtemp,DBtemp,DTtemp
      INTEGER :: j, k

      CELLX = 0
      CELLY = 0

!----------------------------------------------------------------------
! X direction RUNUP
!----------------------------------------------------------------------
      DO k=ks,ke
        DO j=js+1,je
!        DO j=js+1,je-1
!         (1). WET - DRY interface
!              j-1    j
          IF ( CELL(j-1,  k) .EQ. 1    &
                           .AND.       &
               CELL(j,k) .EQ. 0 ) THEN
            !     sea level - land height is (+)
            IF ( (zo(j-1,k) +H(j,k) -eto(j,k)) .GT. EPSILON ) THEN
              D(j,k)     = zo(j-1,k) + H(j,k)-eto(j,k)
              CELL(j,k)  = 1
              CELLX(j,k) = 1
 !            zo(j,k) = zo(j-1,k)         !

              IF ( j .GE. 3 ) THEN
                IF ( CELL(j-2,k) .EQ. 1. ) THEN
                  uo(j,k) = uo(j-1,k)
                END IF

                IF ( CELL(j-2,k) .EQ. 0. ) THEN  ! CELL (j,k) IS SURROUNDED
                  uo(j,k) = SQRT(g*D(j,k))         ! BY DRY CELLS
                END IF
              END IF

            END IF
          END IF
        END DO
      END DO

      DO k=ks,ke
        DO j=js+1,je-1


!         (2). WET - DRY - WET interface
!              j-1    j    j+1
!                   CELLX
!
!              check if water flow from right side of CELL'X' and correct D(j,k)

          IF ( CELLX( j ,k) .EQ. 1        &
                            .AND.         &
                CELL(j+1,k) .EQ. 1 ) THEN
            !     sea level - land height is (+)
            IF ( (zo(j+1,k) +H(j,k) -eto(j,k)) .GT. EPSILON ) THEN
              DRtemp= zo(j+1,k) + H(j,k)-eto(j,k)

              D(j,k) = 0.5*(DRtemp+D(j,k))

 !            zo(j,k) = zo(j+1,k)        !

              IF ( j .LE. je-2 ) THEN
                IF ( CELL(j+2,k) .EQ. 1 )THEN
                  uo(j+1,k)=uo(j+2,k)
                END IF

                IF ( CELL(j+2,k) .EQ. 0 ) THEN     ! CELL(j,k) SURROUNDER
                  uo(j+1,k)=-SQRT(g*D(j,k))        ! BY DRY CELLS
                END IF
              END IF

            END IF
          END IF

!         (3). DRY - WET interface
!               j    j+1
          IF ( CELL(j,k) .EQ. 0        &
                           .AND.         &
               CELL(j+1,  k) .EQ. 1 ) THEN
            !     sea level - land height is (+)
            IF ( (zo(j+1,k) +H(j,k) -eto(j,k)) .GT. EPSILON ) THEN
              D(j,k) = zo(j+1,k) + H(j,k)-eto(j,k)
              CELL(j,k) = 1
              CELLX(j,k) = 1
 !            zo(j,k) = zo(j+1,k)        !

              IF ( j .LE. je-2 ) THEN
                IF ( CELL(j+2,k) .EQ. 1 )THEN
                  uo(j+1,k)=uo(j+2,k)
                END IF

                IF ( CELL(j+2,k) .EQ. 0 ) THEN     ! CELL(j,k) SURROUNDER
                  uo(j+1,k) = -SQRT(g*D(j,k))        ! BY DRY CELLS
                END IF
              END IF

            END IF
          END IF

        END DO
      END DO


!----------------------------------------------------------------------
! Y direction RUNUP
!----------------------------------------------------------------------
      DO j=js,je
        DO k = ks+1, ke-1
!--------------------------------------------------------------------------

!         (1).  DRY(CELLX)  k
!                  |
!                 WET      k-1
!
!              check if water flow from BOTTOM side of CELL'X' and correct D(j,k)
          IF (  CELL(j,k-1) .EQ. 1        &
                            .AND.         &
               CELLX(j, k ) .EQ. 1 ) THEN

            IF ( (zo(j,k-1) +H(j,k) -eto(j,k)) .GT. EPSILON) THEN
              DBtemp= zo(j,k-1) + H(j,k)-eto(j,k)

              D(j,k) = 0.5*(DBtemp+D(j,k))
             ! CELL(j,k) = 1  !already defined as 1 in x sweep!!

!             zo(j,k+1) = zo(j,k)

              IF ( k .GE. 3 ) THEN
                IF ( CELL(j,k-2) .EQ. 1 ) THEN
                  vo(j,k-1)=vo(j,k-2)
                END IF

                IF ( CELL(j,k-2) .EQ. 0 ) THEN
                  vo(j,k-1)=SQRT(g*D(j,k))
                END IF
              END IF

            END IF
          END IF

!         (2).    WET      k+1
!                  |
!               DRY(CELLX)  k
!              check if water flow from TOP side of CELL'X' and correct D(j,k)
          IF ( CELLX(j, k ) .EQ. 1        &
                            .AND.         &
                CELL(j,k+1) .EQ. 1 ) THEN
            IF ( (zo(j,k+1) +H(j,k) -eto(j,k)) .GT. EPSILON) THEN

              DTtemp= zo(j,k+1) + H(j,k)-eto(j,k)

              D(j,k) = 0.5*( DTtemp +D(j,k) )

 !            zo(j,k-1) = zo(j,k)

              IF ( k .LE. ke-2) THEN
                IF ( CELL(j,k+2) .EQ. 1 ) THEN
                  vo(j,k)=vo(j,k+1)
                END IF

                IF ( CELL(j,k+2) .EQ. 0 ) THEN
                  vo(j,k)=-SQRT(g*D(j,k))
                END IF
              END IF
            END IF
          END IF

!         (3).    DRY   k
!                  |
!                 WET  k-1
!
          IF ( CELL(j,k-1) .EQ. 1        &
                           .AND.         &
               CELL(j, k ) .EQ. 0 ) THEN
            IF ( (zo(j,k-1) +H(j,k) -eto(j,k)) .GT. EPSILON) THEN
              D(j,k)= zo(j,k-1) + H(j,k)-eto(j,k)

              CELL(j,k) = 1
              CELLY(j,k) = 1
!             zo(j,k+1) = zo(j,k)

              IF ( k .GE. 3 ) THEN
                IF ( CELL(j,k-2) .EQ. 1 ) THEN
                  vo(j,k-1)=vo(j,k-2)
                END IF

                IF ( CELL(j,k-2) .EQ. 0 ) THEN
                  vo(j,k-1)=SQRT(g*D(j,k))
                END IF
              END IF

            END IF
          END IF
!--------------------------------------------------------------------------
        END DO
      END DO


      DO j=1,je
        DO k = ks+1, ke - 1

!         (4).    WET      k+1
!                  |
!               DRY(CELLY)  k
!
!              check if water flow from TOP side of CELL'Y' and correct D(j,k)
          IF ( CELLY(j, k ) .EQ. 1        &
                            .AND.         &
                CELL(j,k+1) .EQ. 1 ) THEN
            IF ( (zo(j,k+1) +H(j,k) -eto(j,k)) .GT. EPSILON) THEN



              DTtemp= zo(j,k+1) + H(j,k)-eto(j,k)

              D(j,k) = 0.5*( DTtemp +D(j,k) )
 !             CELL(j,k) = 1
 !            zo(j,k-1) = zo(j,k)

              IF ( k .LE. ke-2) THEN
                IF ( CELL(j,k+2) .EQ. 1 ) THEN
                  vo(j,k)=vo(j,k+1)
                END IF

                IF ( CELL(j,k+2) .EQ. 0 ) THEN
                  vo(j,k)=-SQRT(g*D(j,k))
                END IF

              END IF
            END IF
          END IF

!         (4).    WET  k+1
!                  |
!                 DRY   k
          IF ( CELL(j,k) .EQ. 0        &
                           .AND.         &
               CELL(j,k+1  ) .EQ. 1 ) THEN
            IF ( (zo(j,k+1) +H(j,k) -eto(j,k)) .GT. EPSILON) THEN

              D(j,k) = zo(j,k+1) + H(j,k)-eto(j,k)

              CELL(j,k) = 1
!             zo(j,k-1) = zo(j,k)

              IF ( k .LE. ke-2) THEN
                IF ( CELL(j,k+2) .EQ. 1 ) THEN
                  vo(j,k)=vo(j,k+1)
                END IF

                IF ( CELL(j,k+2) .EQ. 0 ) THEN
                  vo(j,k)=-SQRT(g*D(j,k))
                END IF
              END IF

            END IF
          END IF
!--------------------------------------------------------------------------
        END DO
      END DO

END SUBROUTINE

!---------------------------------------------------------------------
SUBROUTINE RUNUP_Kowalik ( CELL, D, uo, vo,  &
                           zo, H, eto,       &
                           js, je, ks, ke  )
!---------------------------------------------------------------------
!                              Runup solver
!                       Kowalik et al. 2005 version
!---------------------------------------------------------------------
!
!
!   Z.Kowalik  2006 IMS,      University of Alaska Fairbanks
!   W.Knight   2007 NOAA/NWS, West Coast & Alaska Tsunami Warning Center
!   Y.Yamazaki 2007 ORE,      University of Hawaii Manoa
!---------------------------------------------------------------------
!      USE ALL_INPUTS
      IMPLICIT NONE

!  (INOUT)...
      INTEGER*2, DIMENSION(js:je,ks:ke):: CELL

      REAL, DIMENSION(js:je,ks:ke):: D
      REAL, DIMENSION(js:je,ks:ke):: uo, vo

!  (IN)...
      REAL, DIMENSION(js:je,ks:ke):: zo
      REAL, DIMENSION(js:je,ks:ke):: H
      REAL, DIMENSION(js:je,ks:ke):: eto
      INTEGER:: js, je, ks, ke

!  (INTERNAL)...
      INTEGER*2, DIMENSION(js:je,ks:ke):: CELLX,CELLY
      REAL    :: DRtemp,DBtemp,DTtemp
      INTEGER :: j, k

      CELLX = 0
      CELLY = 0

!----------------------------------------------------------------------
! X direction RUNUP
!----------------------------------------------------------------------
      DO k=ks,ke
        DO j=js+1,je-1

!         (1). WET - DRY interface
!              j-1    j
          IF ( CELL(j-1,  k) .EQ. 1    &
                           .AND.       &
               CELL(j,k) .EQ. 0 ) THEN
            !     sea level - land height is (+)
            IF ( (zo(j-1,k) +H(j,k) -eto(j,k)) .GT. EPSILON ) THEN
              D(j,k)     = zo(j-1,k) + H(j,k)-eto(j,k)
              CELL(j,k)  = 1
              CELLX(j,k) = 1
 !            zo(j,k) = zo(j-1,k)         !

              IF ( CELL(j-2,k) .EQ. 1. ) THEN
                uo(j,k) = uo(j-1,k)
              END IF

              IF ( CELL(j-2,k) .EQ. 0. ) THEN  ! CELL (j,k) IS SURROUNDED
                uo(j,k) = SQRT(g*D(j,k))         ! BY DRY CELLS
              END IF
            END IF
          END IF
        END DO
      END DO

      DO k=ks,ke
        DO j=js+1,je-1


!         (2). WET - DRY - WET interface
!              j-1    j    j+1
!                   CELLX
!
!              check if water flow from right side of CELL'X' and correct D(j,k)

          IF ( CELLX( j ,k) .EQ. 1        &
                            .AND.         &
                CELL(j+1,k) .EQ. 1 ) THEN
            !     sea level - land height is (+)
            IF ( (zo(j+1,k) +H(j,k) -eto(j,k)) .GT. EPSILON ) THEN
              DRtemp= zo(j+1,k) + H(j,k)-eto(j,k)

              D(j,k) = 0.5*(DRtemp+D(j,k))

 !            zo(j,k) = zo(j+1,k)        !

              IF ( CELL(j+2,k) .EQ. 1 )THEN
                uo(j+1,k)=uo(j+2,k)
              END IF

              IF ( CELL(j+2,k) .EQ. 0 ) THEN     ! CELL(j,k) SURROUNDER
                uo(j+1,k)=-SQRT(g*D(j,k))        ! BY DRY CELLS
              END IF
            END IF
          END IF

!         (3). DRY - WET interface
!               j    j+1
          IF ( CELL(j,k) .EQ. 0        &
                           .AND.         &
               CELL(j+1,  k) .EQ. 1 ) THEN
            !     sea level - land height is (+)
            IF ( (zo(j+1,k) +H(j,k) -eto(j,k)) .GT. EPSILON ) THEN
              D(j,k) = zo(j+1,k) + H(j,k)-eto(j,k)
              CELL(j,k) = 1
              CELLX(j,k) = 1
 !            zo(j,k) = zo(j+1,k)        !

              IF ( CELL(j+2,k) .EQ. 1 )THEN
                uo(j+1,k)=uo(j+2,k)
              END IF

              IF ( CELL(j+2,k) .EQ. 0 ) THEN     ! CELL(j,k) SURROUNDER
                uo(j+1,k) = -SQRT(g*D(j,k))        ! BY DRY CELLS
              END IF

            END IF
          END IF

        END DO
      END DO


!----------------------------------------------------------------------
! Y direction RUNUP
!----------------------------------------------------------------------
      DO j=1,je
        DO k = ks+1, ke - 1
!--------------------------------------------------------------------------

!         (1).  DRY(CELLX)  k
!                  |
!                 WET      k-1
!
!              check if water flow from BOTTOM side of CELL'X' and correct D(j,k)
          IF (  CELL(j,k-1) .EQ. 1        &
                            .AND.         &
               CELLX(j, k ) .EQ. 1 ) THEN

            IF ( (zo(j,k-1) +H(j,k) -eto(j,k)) .GT. EPSILON) THEN
              DBtemp= zo(j,k-1) + H(j,k)-eto(j,k)

              D(j,k) = 0.5*(DBtemp+D(j,k))
             ! CELL(j,k) = 1  !already defined as 1 in x sweep!!

!             zo(j,k+1) = zo(j,k)

              IF ( CELL(j,k-2) .EQ. 1 ) THEN
                vo(j,k-1)=vo(j,k-2)
              END IF

              IF ( CELL(j,k-2) .EQ. 0 ) THEN
                vo(j,k-1)=SQRT(g*D(j,k))
              END IF

            END IF
          END IF

!         (2).    WET      k+1
!                  |
!               DRY(CELLX)  k
!              check if water flow from TOP side of CELL'X' and correct D(j,k)
          IF ( CELLX(j, k ) .EQ. 1        &
                            .AND.         &
                CELL(j,k+1) .EQ. 1 ) THEN
            IF ( (zo(j,k+1) +H(j,k) -eto(j,k)) .GT. EPSILON) THEN

              DTtemp= zo(j,k+1) + H(j,k)-eto(j,k)

              D(j,k) = 0.5*( DTtemp +D(j,k) )

 !            zo(j,k-1) = zo(j,k)

              IF ( CELL(j,k+2) .EQ. 1 ) THEN
                vo(j,k)=vo(j,k+1)
              END IF

              IF ( CELL(j,k+2) .EQ. 0 ) THEN
                vo(j,k)=-SQRT(g*D(j,k))
              END IF

            END IF
          END IF

!         (3).    DRY   k
!                  |
!                 WET  k-1
!
          IF ( CELL(j,k-1) .EQ. 1        &
                           .AND.         &
               CELL(j, k ) .EQ. 0 ) THEN
            IF ( (zo(j,k-1) +H(j,k) -eto(j,k)) .GT. EPSILON) THEN
              D(j,k)= zo(j,k-1) + H(j,k)-eto(j,k)

              CELL(j,k) = 1
              CELLY(j,k) = 1
!             zo(j,k+1) = zo(j,k)

              IF ( CELL(j,k-2) .EQ. 1 ) THEN
                vo(j,k-1)=vo(j,k-2)
              END IF

              IF ( CELL(j,k-2) .EQ. 0 ) THEN
                vo(j,k-1)=SQRT(g*D(j,k))
              END IF

            END IF
          END IF
!--------------------------------------------------------------------------
        END DO
      END DO


      DO j=1,je
        DO k = ks+1, ke - 1

!         (4).    WET      k+1
!                  |
!               DRY(CELLY)  k
!
!              check if water flow from TOP side of CELL'Y' and correct D(j,k)
          IF ( CELLY(j, k ) .EQ. 1        &
                            .AND.         &
                CELL(j,k+1) .EQ. 1 ) THEN
            IF ( (zo(j,k+1) +H(j,k) -eto(j,k)) .GT. EPSILON) THEN



              DTtemp= zo(j,k+1) + H(j,k)-eto(j,k)

              D(j,k) = 0.5*( DTtemp +D(j,k) )
 !             CELL(j,k) = 1
 !            zo(j,k-1) = zo(j,k)

              IF ( CELL(j,k+2) .EQ. 1 ) THEN
                vo(j,k)=vo(j,k+1)
              END IF

              IF ( CELL(j,k+2) .EQ. 0 ) THEN
                vo(j,k)=-SQRT(g*D(j,k))
              END IF

            END IF
          END IF

!         (4).    WET  k+1
!                  |
!                 DRY   k
          IF ( CELL(j,k) .EQ. 0        &
                           .AND.         &
               CELL(j,k+1  ) .EQ. 1 ) THEN
            IF ( (zo(j,k+1) +H(j,k) -eto(j,k)) .GT. EPSILON) THEN

              D(j,k) = zo(j,k+1) + H(j,k)-eto(j,k)

              CELL(j,k) = 1
 !            zo(j,k-1) = zo(j,k)

              IF ( CELL(j,k+2) .EQ. 1 ) THEN
                vo(j,k)=vo(j,k+1)
              END IF

              IF ( CELL(j,k+2) .EQ. 0 ) THEN
                vo(j,k)=-SQRT(g*D(j,k))
              END IF

            END IF
          END IF
!--------------------------------------------------------------------------
        END DO
      END DO

END SUBROUTINE



!------------------------------------------------------------------------
SUBROUTINE WET_DRY_UV_BC ( uo, vo,             &
                           zo, H, eto, CELL,   &
                           js, je, ks, ke    )
!------------------------------------------------------------------------
!           Velocity Boundary Condition on WET-DAY interface
!------------------------------------------------------------------------
!
!   This program defines u and v at interface of WEt-DRY boundary.
!
!
!   Z.Kowalik  2006 IMS      University of Alaska Fairbanks
!   W.Knight   2007 NOAA/NWS West Coast & Alaska Tsunami Warning Center
!   Y.Yamazaki 2007 ORE      University of Hawaii Manoa
!-------------------------------------------------------------------------
      !USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE

!  (INOUT)...

      REAL, DIMENSION(js:je,ks:ke):: uo, vo

!  (IN)...
      INTEGER*2, DIMENSION(js:je,ks:ke):: CELL
      REAL, DIMENSION(js:je,ks:ke):: zo
      REAL, DIMENSION(js:je,ks:ke):: H
      REAL, DIMENSION(js:je,ks:ke):: eto
      INTEGER:: js, je, ks, ke

!  (INTERNAL)...
      INTEGER :: j, k

!.. u evaluation along x direction

      DO k = ks+1, ke-1

        DO j = js+1, je-1
          IF ( CELL(j,k  ) .EQ. 0        &
                           .AND.         &
               CELL(j-1,k) .EQ. 0 ) THEN
            uo(j,k) = 0.0
          END IF
        END DO

        DO j = js+1, je-1        ! CHANGE 1  TO 2 FOR RADIATION
          IF ( CELL(j,  k) .EQ. 1        &
                           .AND.         &
               CELL(j+1,k) .EQ. 0 ) THEN

            IF ( (zo(j,k) + H(j+1,k)-eto(j+1,k)) .LE. EPSILON ) THEN
              uo(j+1,k) = 0.0
            END IF

          END IF

          IF ( CELL(j-1,k) .EQ. 0        &
                           .AND.         &
               CELL(j,  k) .EQ. 1 ) THEN

            IF ( (zo(j,k) +H(j-1,k)-eto(j-1,k)) .LE. EPSILON ) THEN
              uo(j,k) = 0.0
            END IF

          END IF
        END DO
      END DO

!.. v evaluation along x direction
      DO j = js,je-1
        DO k = ks+1, ke-1

          IF ( CELL(j,k  ) .EQ. 0        &
                           .AND.         &
               CELL(j,k+1) .EQ. 0 ) THEN
            vo(j,k) = 0.0
          END IF

        END DO

        DO k = ks+1, ke - 1
          IF ( CELL(j,k  ) .EQ. 1        &
                           .AND.         &
               CELL(j,k+1) .EQ. 0 ) THEN

            IF ( (zo(j,k) +H(j,k+1)-eto(j,k+1)) .LE. EPSILON ) THEN
              vo(j,k) = 0.0
            END IF

          END IF

          IF ( CELL(j,k-1) .EQ. 0        &
                           .AND.         &
               CELL(j,k  ) .EQ. 1 ) THEN

            IF ( (zo(j,k) +H(j,k-1)-eto(j,k-1)) .LE. EPSILON ) THEN
              vo(j,k-1) = 0.0
            END IF

          END IF
        END DO
      END DO

END SUBROUTINE

! variable calculation solvers
!------------------------------------------------------------------------
SUBROUTINE Hu_Hv_CAL ( Hu, Hv, H, js, je, ks, ke )
!------------------------------------------------------------------------
!           Still water depth at velocity calculation points
!------------------------------------------------------------------------
!
!
!
!   Y.Yamazaki 2007 ORE      University of Hawaii Manoa
!-------------------------------------------------------------------------

      !USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE

!  (IN)...
      REAL, DIMENSION(js:je,ks:ke):: H
      INTEGER:: js, je, ks, ke

!  (OUT)...
      REAL, DIMENSION(js:je,ks:ke):: Hu, Hv

!  (INTERNAL)...
      INTEGER:: j, k


!.. H at u cal points, Hu
      DO j=js+1,je
        DO k=1,ke
          Hu(j,k)=(H(j-1,k)+H(j,k))*0.5
        END DO
      END DO

      DO k=1,ke
        Hu(js,k) = H(js,k)
      END DO

!.. H at v cal points, Hv
      DO j=1,je
        DO k=ks,ke-1
          Hv(j,k)=(H(j,k+1)+H(j,k))*0.5
        END DO
      END DO

      DO j=1,je
!       Hv(j,ke)=Hv(j,ke-1) ! original
        Hv(j,ke)=H(j,ke)    ! but this would be right
      END DO

END SUBROUTINE



!------------------------------------------------------------------------
SUBROUTINE Dlim_Du_Dv_CAL ( Dlim, Du, Dv,            &
                            zo, eto, H, D, Hu, Hv,   &
                            js, je, ks, ke         )
!------------------------------------------------------------------------
!           Total water depth at velocity calculation points
!------------------------------------------------------------------------
!
!
!
!   Y.Yamazaki 2007 ORE      University of Hawaii Manoa
!-------------------------------------------------------------------------
      !USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE

!  (OUT)...
      REAL, DIMENSION(js:je,ks:ke):: Dlim, Du, Dv

!  (IN)...
      REAL, DIMENSION(js:je,ks:ke):: zo, eto
      REAL, DIMENSION(js:je,ks:ke):: H, D
      REAL, DIMENSION(js:je,ks:ke):: Hu, Hv
      INTEGER:: js, je, ks, ke

!  (INTERNAL)...
      INTEGER:: j, k


!------------------------------------------------------------------------------
!  Limited Total Water Depth, Dlim
!------------------------------------------------------------------------------
      DO k = ks, ke
        DO j = js, je
           Dlim(j,k)=  H(j,k) +zo(j,k)           ! try both !!
           !Dlim(j,k)=  H(j,k) +zo(j,k) -eto(j,k)
           IF ( ABS(Dlim(j,k)) .LT. Dmin ) THEN
              Dlim(j,k)=Dmin
           END IF
        END DO
      END DO




!.. D at u cal points, Du
      DO j=js+1,je
        DO k=ks,ke
          Du(j,k) = Hu(j,k) +0.5*(  zo(j-1,k)  +zo(j,k) )  &
                            -0.5*( eto(j-1,k) +eto(j,k) )
          IF( Du(j,k) .LE. Dmin ) THEN
            Du(j,k) = Dmin
          END IF
        END DO
      END DO

      DO k=ks,ke
        Du(js,k) = D(js,k)
      END DO

!.. D at v cal points, Dv
      DO j=js,je
        DO k=ks,ke-1
          Dv(j,k) = Hv(j,k) +0.5*(  zo(j,k)  +zo(j,k+1) ) &
                            -0.5*( eto(j,k) +eto(j,k+1) )
          IF( Dv(j,k) .LE. Dmin ) THEN
            Dv(j,k) = Dmin
          END IF
        END DO
      END DO

      DO j=js,je
!       Dv(j,ke)=Dv(j,ke-1) ! original
        Dv(j,ke)=D(j,ke)    ! but this would be right
      END DO

END SUBROUTINE


!------------------------------------------------------------------------
SUBROUTINE TRIGON ( siz, csz, siv, csv, ks, ke, dlat, latB )
!------------------------------------------------------------------------
! This program calculate the parameters used in the spherical coordinate
! system calculation
!
!
!   Z.Kowalik  2006 IMS      University of Alaska Fairbanks
!   W.Knight   2007 NOAA/NWS West Coast & Alaska Tsunami Warning Center
!   Y.Yamazaki 2007 ORE      University of Hawaii Manoa
!------------------------------------------------------------------------
      !USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE

!  (IN)...
      REAL   :: dlat, latB
      INTEGER:: ks, ke

!  (OUT)...
      REAL, DIMENSION(ks:ke):: siz,siv,csz,csv

!  (INTERNAL)...
      REAL   :: latz, latv
      INTEGER:: k

      DO k=ks,ke
        latz = latB +dlat*FLOAT(k-1) ! Lat(deg.) at the CENTER of CELL (z cal point)
        latv = latz +0.5*dlat        ! Lat(deg.) at the TOP of CELL    (v cal point)

        siz(k) = SIN(latz*rad)       ! sin for z cal point (use only for Coriolis term in un cal)
        csz(k) = COS(latz*rad)       ! cos for z cal point

        siv(k) = SIN(latv*rad)       ! sin for v cal point (use only for Coriolis term in vn cal)
        csv(k) = COS(latv*rad)       ! cos for v cal point
      END DO

END SUBROUTINE


! open boundary conditions
!------------------------------------------------------------------------
SUBROUTINE OPEN_BC_LR
!------------------------------------------------------------------------
!
!   Z.Kowalik  2006 IMS      University of Alaska Fairbanks
!   W.Knight   2007 NOAA/NWS West Coast & Alaska Tsunami Warning Center
!   Y.Yamazaki 2007 ORE      University of Hawaii Manoa
!------------------------------------------------------------------------
      !USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE

!  (OUT)...

!  (INOUT)...

!  (IN)...
!
!  (INTERNAL)...
      REAL    :: Dtmp
      INTEGER :: k


!... LEFT Open B.C.
      DO k=ks1,ke1
        Dtmp = H1(js1+1,k) -eto1(js1+1,k) +zo1(js1+1,k)
        IF( Dtmp .LT. 0. ) THEN
          Dtmp = 0.
        END IF
        zo1(js1,k) = -uo1(js1+1,k)*SQRT(Dtmp/g)
        zn1(js1,k) = zo1(js1,k)
      END DO

!... RIGHT Open B.C.
      DO k=ks1+1,ke1-1
        Dtmp= H1(je1-1,k) -eto1(je1-1,k) +zo1(je1-1,k)
        IF( Dtmp .LT. 0. ) THEN
          Dtmp=0.
        END IF
        zo1(je1,k) = uo1(je1,k)*SQRT(Dtmp/g)
        zn1(je1,k) = zo1(je1,k)
      END DO

END SUBROUTINE

!------------------------------------------------------------------------
SUBROUTINE OPEN_BC_BT
!------------------------------------------------------------------------
!
!   Z.Kowalik  2006 IMS      University of Alaska Fairbanks
!   W.Knight   2007 NOAA/NWS West Coast & Alaska Tsunami Warning Center
!   Y.Yamazaki 2007 ORE      University of Hawaii Manoa
!------------------------------------------------------------------------
      !USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE

      REAL    :: Dtmp
      INTEGER :: j

!... BOTTOMT Open B.C.
      DO j=js1,je1
       !Dtmp= H1(j,ks1+1)-eto1(j,ks1+1)+zo1(j,ks1+1)  ! Yoshiki
        Dtmp = H1(j,ks1) -eto1(j,ks1) +zo1(j,ks1)
        IF( Dtmp .LT. 0. ) THEN
          Dtmp=0.
        END IF
        zo1(j,ks1) = -vo1(j,ks1)*SQRT(Dtmp/g)
        zn1(j,ks1) = zo1(j,ks1)
      END DO

!... TOP Open B.C.
      DO j=js1,je1
       !Dtmp = H1(j,ke1-1)-eto1(j,ke1-1)+zo1(j,ke1-1)
        Dtmp = H1(j,ke1)  -eto1(j,ke1)  +zo1(j,ke1)
        IF( Dtmp .LT. 0. ) THEN
          Dtmp=0.
        END IF
        zo1(j,ke1) = vo1(j,ke1-1)*SQRT(Dtmp/g)
       !zo1(j,ke1) = zo1(j,ke1-1)
        zn1(j,ke1) = zo1(j,ke1)
      END DO
 
END SUBROUTINE

! input
!------------------------------------------------------------------------
SUBROUTINE INPUT_BATHYMETRY
!------------------------------------------------------------------------
      !USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE

      INTEGER:: j, k

!------------------------------------------------------------------------
!.. H, water depth at zeta cal point
!------------------------------------------------------------------------

! 1st Layer
!------------------------------------------------------------------------

! (1). Binary format
      OPEN ( UNIT=11,                              &
             FILE='ale2min.dat',                   &
             FORM='unformatted',STATUS='unknown')
      READ(11) ( (H1(j,k), j = js1,je1 ),k = ks1,ke1 )

! (2). ASCHI format
!      OPEN ( UNIT=11,                         &
!             FILE='samoa_30sec_G_rec.dat',    &
!             FILE='samoa_30sec_ori.dat',      &
!             STATUS='OLD' )
!      READ(11,*) ( (H1(j,k), j = js1,je1 ),k = ks1,ke1 )

!     WRITE (104) ( (H1(j,k), j=js1,je1 ) ,k=ks1,ke1 )  ! 'H.dat': NO cutoff topo

      DO j=js1,je1
        DO k=ks1,ke1
          H1(j,k) = -H1(j,k)*100.         !H(cm) NOTE: Wet +, Dry -
          IF( H1(j,k) .LT. -5000. ) THEN
            H1(j,k) = -5000.              ! cut land higher than 50 m
          END IF                          ! for bottom deformation case
        END DO
      END DO

      DO j=js1,je1
        DO k=ks1,ke1
          IF (H1(j,k).LT.100.AND.h1(j,k).GT.0.) THEN
            H1(j,k) = 400.             ! VERY SHALLOW MAKE IT 4M
          END IF
        END DO
      END DO
      WRITE (104) ( (H1(j,k), j=js1,je1 ) ,k=ks1,ke1 )  ! 'H.dat': cutoff topo


! 2nd Layer
!------------------------------------------------------------------------
!      OPEN ( UNIT=21,                        &
!             FILE='tutuila_3sec_G_rec.dat',  &
!             FILE='tutuila_3sec_ori.dat',   &
!             STATUS='OLD' )
         open(unit=21,file='crescent24sec.dat',status='unknown')

!      OPEN ( UNIT=21, FILE=L_FILE2, STATUS='OLD' )
     READ(21,*) ( (H2(j,k), j = js2,je2 ),k = ks2,ke2 )
!      WRITE (204) ( (H2(j,k), j=js2,je2 ) ,k=ks2,ke2 )  ! 'H.dat': NO cutoff topo

      DO j=js2,je2
        DO k=ks2,ke2
          H2(j,k) = -H2(j,k)*100.         !H(cm) NOTE: Wet +, Dry -
          IF( H2(j,k) .LT. -5000. ) THEN
            H2(j,k) = -5000.              ! cut land higher than 50 m
          END IF                          ! for bottom deformation case
        END DO
      END DO
      DO j=js2,je2
        DO k=ks2,ke2
          IF (H2(j,k).LT.100.AND.h2(j,k).GT.0.) THEN
            H2(j,k) = 400.             ! VERY SHALLOW MAKE IT 4M
          END IF
        END DO
      END DO

      WRITE (204) ( (H2(j,k), j=js2,je2 ) ,k=ks2,ke2 )  ! 'H.dat': cutoff topo

! Depth Transition B.C.
!---------------------------------------------------------------------
      CALL DC_BOUND_INTERP_H ( H2,H1,inc12x,inc12y,js1,je1,ks1,ke1,  &
                               js2,je2,ks2,ke2,js2o,je2o,ks2o,ke2o )



! 3rd Layer
!------------------------------------------------------------------------
!      OPEN ( UNIT=31,                               &
!             FILE='pagopago_0p3sec_G_rec.dat',        &
!             FILE='pagopago_0p3sec_ori.dat',        &
!             STATUS='OLD' )
         open(unit=31,file='crescent3sec.dat',status='unknown')
      READ(31,*) ( (H3(j,k), j = js3,je3 ),k = ks3,ke3 )
!      WRITE (304) ( (H3(j,k), j=js3,je3 ) ,k=ks3,ke3 )  ! 'H.dat': NO cutoff topo

      DO j=js3,je3
        DO k=ks3,ke3
          H3(j,k) = -H3(j,k)*100.         !H(cm) NOTE: Wet +, Dry -
          IF( H3(j,k) .LT. -5000. ) THEN
            H3(j,k) = -5000.              ! cut land higher than 50 m
          END IF                          ! for bottom deformation case
        END DO
      END DO

      DO j=js3,je3
        DO k=ks3,ke3
          IF( H3(j,k).LT.100.AND.H3(j,k).GT.0.) THEN
            H3(j,k) = 400.              ! VERY SHALLOW MAKE IT 4M
          END IF
        END DO
      END DO

      WRITE (304) ( (H3(j,k), j=js3,je3 ) ,k=ks3,ke3 )  ! 'H.dat': cutoff topo

! Depth Transition B.C.
!---------------------------------------------------------------------
      CALL DC_BOUND_INTERP_H ( H3,H2,inc23x,inc23y,js2,je2,ks2,ke2,  &
                               js3,je3,ks3,ke3,js3o,je3o,ks3o,ke3o )


! 4th Layer
!------------------------------------------------------------------------
!!      OPEN ( UNIT=41, FILE='hilo_0p3sec_Halpha2p0.dat', STATUS='OLD' )
      OPEN ( UNIT=41, FILE='cresport1s.dat', STATUS='OLD' )
      READ(41,*) ( (H4(j,k), j = js4,je4 ),k = ks4,ke4 )
!!      WRITE (404) ( (H4(j,k), j=js4,je4 ) ,k=ks4,ke4 )  ! 'H.dat': NO cutoff topo
!
      DO j=js4,je4
        DO k=ks4,ke4
          H4(j,k) = -H4(j,k)*100.         !H(cm) NOTE: Wet +, Dry -
          IF( H4(j,k) .LT. -5000. ) THEN
            H4(j,k) = -5000.              ! cut land higher than 50 m
          END IF                          ! for bottom deformation case
        END DO
      END DO
      WRITE (404) ( (H4(j,k), j=js4,je4 ) ,k=ks4,ke4 )  ! 'H.dat': cutoff topo

      CALL DC_BOUND_INTERP_H ( H4,H3,inc34x,inc34y,js3,je3,ks3,ke3,  &
                               js4,je4,ks4,ke4,js4o,je4o,ks4o,ke4o )


END SUBROUTINE


! output
!---------------------------------------------------------------------------------------
SUBROUTINE OPEN_OUTPUT_FILE_NAME
!---------------------------------------------------------------------------------------

!... OUTPUT file names
!------------------------------------------------------------------------------
! 1st Domain
!-------------------------------------------------------------------------------------------------
IF (.not.RESTART) THEN
      OPEN ( UNIT=101, FILE='z1.dat',   FORM='unformatted', STATUS='unknown')
!      OPEN ( UNIT=102, FILE='u1.dat',   FORM='unformatted', STATUS='unknown')
!      OPEN ( UNIT=103, FILE='v1.dat',   FORM='unformatted', STATUS='unknown')
      OPEN ( UNIT=104, FILE='H1.dat',   FORM='unformatted', STATUS='unknown')
!      OPEN ( UNIT=105, FILE='D1.dat',   FORM='unformatted', STATUS='unknown')
!      OPEN ( UNIT=106, FILE='eta1.dat', FORM='unformatted', STATUS='unknown')

!      OPEN ( UNIT=107, FILE='wb1.dat',  FORM='unformatted', STATUS='unknown')
!      OPEN ( UNIT=108, FILE='ws1.dat',  FORM='unformatted', STATUS='unknown')
!      OPEN ( UNIT=109, FILE='q1.dat',   FORM='unformatted', STATUS='unknown')
!      OPEN ( UNIT=110, FILE='Hb1.dat',  FORM='unformatted', STATUS='unknown')

! in output_zmax1.f90
!     OPEN ( UNIT=111, FILE='zmax1.dat', FORM='unformatted', STATUS='unknown')
! in neowave2D.f90
!      OPEN ( UNIT=151, FILE='z01.dat', FORM='unformatted', STATUS='unknown')

!      OPEN ( UNIT=151, FILE='disx.dat', FORM='unformatted', STATUS='unknown')
!      OPEN ( UNIT=152, FILE='disy.dat', FORM='unformatted', STATUS='unknown')
!      OPEN ( UNIT=153, FILE='disz.dat', FORM='unformatted', STATUS='unknown')

! 2nd Domain
!-------------------------------------------------------------------------------------------------
      OPEN ( UNIT=201, FILE='z2.dat',   FORM='unformatted', STATUS='unknown')
      OPEN ( UNIT=204, FILE='H2.dat',   FORM='unformatted', STATUS='unknown')

! in output_zmax2.f90
!     OPEN ( UNIT=211, FILE='zmax2.dat', FORM='unformatted', STATUS='unknown')

! in neowave2D.f90
!      OPEN ( UNIT=251, FILE='z02.dat', FORM='unformatted', STATUS='unknown')

! 3rd Domain
!-------------------------------------------------------------------------------------------------
      OPEN ( UNIT=301, FILE='z3.dat',   FORM='unformatted', STATUS='unknown')
      OPEN ( UNIT=304, FILE='H3.dat',   FORM='unformatted', STATUS='unknown')

! in output_zmax3.f90
!     OPEN ( UNIT=311, FILE='zmax3.dat', FORM='unformatted', STATUS='unknown')

! 4rd Domain
!-------------------------------------------------------------------------------------------------
      OPEN ( UNIT=401, FILE='z4.dat',   FORM='unformatted', STATUS='unknown')
      OPEN ( UNIT=404, FILE='H4.dat',   FORM='unformatted', STATUS='unknown')

! in output_zmax4.f90
!     OPEN ( UNIT=411, FILE='zmax4.dat', FORM='unformatted', STATUS='unknown')
ELSE
      OPEN ( UNIT=101, FILE='z1.dat',   FORM='unformatted', STATUS='unknown',POSITION='append')
!      OPEN ( UNIT=102, FILE='u1.dat',   FORM='unformatted', STATUS='unknown')
!      OPEN ( UNIT=103, FILE='v1.dat',   FORM='unformatted', STATUS='unknown')
      OPEN ( UNIT=104, FILE='H1.dat',   FORM='unformatted', STATUS='unknown',POSITION='append')
!      OPEN ( UNIT=105, FILE='D1.dat',   FORM='unformatted', STATUS='unknown')
!      OPEN ( UNIT=106, FILE='eta1.dat', FORM='unformatted', STATUS='unknown')

!      OPEN ( UNIT=107, FILE='wb1.dat',  FORM='unformatted', STATUS='unknown')
!      OPEN ( UNIT=108, FILE='ws1.dat',  FORM='unformatted', STATUS='unknown')
!      OPEN ( UNIT=109, FILE='q1.dat',   FORM='unformatted', STATUS='unknown')
!      OPEN ( UNIT=110, FILE='Hb1.dat',  FORM='unformatted', STATUS='unknown')

! in output_zmax1.f90
!     OPEN ( UNIT=111, FILE='zmax1.dat', FORM='unformatted', STATUS='unknown')
! in neowave2D.f90
!      OPEN ( UNIT=151, FILE='z01.dat', FORM='unformatted', STATUS='unknown')

!      OPEN ( UNIT=151, FILE='disx.dat', FORM='unformatted', STATUS='unknown')
!      OPEN ( UNIT=152, FILE='disy.dat', FORM='unformatted', STATUS='unknown')
!      OPEN ( UNIT=153, FILE='disz.dat', FORM='unformatted', STATUS='unknown')

! 2nd Domain
!-------------------------------------------------------------------------------------------------
      OPEN ( UNIT=201, FILE='z2.dat',   FORM='unformatted', STATUS='unknown',POSITION='append')
      OPEN ( UNIT=204, FILE='H2.dat',   FORM='unformatted', STATUS='unknown',POSITION='append')

! in output_zmax2.f90
!     OPEN ( UNIT=211, FILE='zmax2.dat', FORM='unformatted', STATUS='unknown')

! in neowave2D.f90
!      OPEN ( UNIT=251, FILE='z02.dat', FORM='unformatted', STATUS='unknown')

! 3rd Domain
!-------------------------------------------------------------------------------------------------
      OPEN ( UNIT=301, FILE='z3.dat',   FORM='unformatted', STATUS='unknown',POSITION='append')
      OPEN ( UNIT=304, FILE='H3.dat',   FORM='unformatted', STATUS='unknown',POSITION='append')

! in output_zmax3.f90
!     OPEN ( UNIT=311, FILE='zmax3.dat', FORM='unformatted', STATUS='unknown')

! 4rd Domain
!-------------------------------------------------------------------------------------------------
      OPEN ( UNIT=401, FILE='z4.dat',   FORM='unformatted', STATUS='unknown',POSITION='append')
      OPEN ( UNIT=404, FILE='H4.dat',   FORM='unformatted', STATUS='unknown',POSITION='append')

! in output_zmax4.f90
!     OPEN ( UNIT=411, FILE='zmax4.dat', FORM='unformatted', STATUS='unknown')
END IF


END SUBROUTINE

!---------------------------------------------------------------------------------------
SUBROUTINE OUTPUT_INIT_TIME_FRAME
!---------------------------------------------------------------------------------------
      !USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE

      REAL, DIMENSION(js1:je1,ks1:ke1)::  zexp1 ! z1 export
      REAL, DIMENSION(js2:je2,ks2:ke2)::  zexp2 ! z2 export
      REAL, DIMENSION(js3:je3,ks3:ke3)::  zexp3 ! z3 export
      REAL, DIMENSION(js4:je4,ks4:ke4)::  zexp4 ! z4 export
      INTEGER:: j, k, j1, k1, j2, k2, j3, k3, j4, k4
      zexp1 = 0.0
      zexp2 = 0.0
      zexp3 = 0.0
      zexp4 = 0.0

      IF ( m .EQ. 1  ) THEN

        DO k = ks1, ke1
          DO j = js1, je1
            zexp1(j,k) = zo1(j,k)
          !(1). set z = -1.0cm for land part
            IF( CELL1(j,k).EQ.0.) THEN ! research setup
              zexp1(j,k) = -1.
            END IF
          END DO
        END DO

        DO k = ks2, ke2
          DO j = js2, je2
            zexp2(j,k) = zo2(j,k)
          !(1). set z = -1.0cm for land part
            IF( CELL2(j,k).EQ.0.) THEN ! research setup
              zexp2(j,k) = -1.
            END IF
          END DO
        END DO

        DO k = ks3, ke3
          DO j = js3, je3
            zexp3(j,k) = zo3(j,k)
          !(1). set z = -1.0cm for land part
            IF( CELL3(j,k).EQ.0.) THEN ! research setup
              zexp3(j,k) = -1.
            END IF
          END DO
        END DO

        DO k = ks4, ke4
          DO j = js4, je4
            zexp4(j,k) = zo4(j,k)
          !(1). set z = -1.0cm for land part
            IF( CELL4(j,k).EQ.0.) THEN ! research setup
              zexp4(j,k) = -1.
            END IF
          END DO
        END DO

        WRITE (101) ( ( zexp1(j,k),j=js1,je1 ), k=ks1,ke1  )
!        WRITE (101) ( ( zexp1(j,k),j=js1,je1,4 ), k=ks1,ke1,4  )
!        WRITE (101) ( ( zo1(j,k)  ,j=js1,je1,4  ), k=ks1,ke1,4  )
!        WRITE (102) ( ( uo1(j,k)  ,j=js1,je1,4  ), k=ks1,ke1,4  )
!        WRITE (103) ( ( vo1(j,k)  ,j=js1,je1,4  ), k=ks1,ke1,4  )
!        WRITE (106) ( ( eto1(j,k) ,j=js1,je1,4  ), k=ks1,ke1,4  )
!        WRITE (109) ( ( q1(j,k)  ,j=js1,je1,4  ), k=ks1,ke1,4  )

        WRITE (201) ( ( zexp2(j,k),j=js2,je2 ), k=ks2,ke2 )
!        WRITE (201) ( ( zo2(j,k)  ,j=js2,je2 ), k=ks2,ke2 )
!        WRITE (202) ( ( uo2(j,k)  ,j=js2,je2 ), k=ks2,ke2 )
!        WRITE (203) ( ( vo2(j,k)  ,j=js2,je2 ), k=ks2,ke2 )
!        WRITE (209) ( ( q2(j,k)  ,j=js2,je2 ), k=ks2,ke2 )

        WRITE (301) ( ( zexp3(j,k),j=js3,je3 ), k=ks3,ke3 )
!        WRITE (301) ( ( zo3(j,k)  ,j=js3,je3 ), k=ks3,ke3 )
!        WRITE (302) ( ( uo3(j,k)  ,j=js3,je3 ), k=ks3,ke3 )
!        WRITE (303) ( ( vo3(j,k)  ,j=js3,je3 ), k=ks3,ke3 )
!        WRITE (309) ( ( q3(j,k)  ,j=js3,je3 ), k=ks3,ke3 )

        WRITE (401) ( ( zexp4(j,k),j=js4,je4 ), k=ks4,ke4 )
!        WRITE (401) ( ( zo4(j,k)  ,j=js4,je4 ), k=ks4,ke4 )
!        WRITE (402) ( ( uo4(j,k)  ,j=js4,je4 ), k=ks4,ke4 )
!        WRITE (403) ( ( vo4(j,k)  ,j=js4,je4 ), k=ks4,ke4 )
!        WRITE (409) ( ( q4(j,k)  ,j=js4,je4 ), k=ks4,ke4 )
      END IF



END SUBROUTINE

!---------------------------------------------------------------------------------------
SUBROUTINE OUTPUT_TIME_FRAME
!---------------------------------------------------------------------------------------
      !USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE
      REAL, DIMENSION(js1:je1,ks1:ke1)::  zexp1 ! z1 export
      REAL, DIMENSION(js2:je2,ks2:ke2)::  zexp2 ! z2 export
      REAL, DIMENSION(js3:je3,ks3:ke3)::  zexp3 ! z3 export
      REAL, DIMENSION(js4:je4,ks4:ke4)::  zexp4 ! z4 export
      INTEGER:: j, k, j1, k1, j2, k2, j3, k3, j4, k4
      zexp1 = 0.0
      zexp2 = 0.0
      zexp3 = 0.0
      zexp4 = 0.0

!      IF ( m               .GE. mout_start        &
!                           .AND.                  &
!           MOD( m, mout1 ) .EQ. 0          ) THEN
      IF ( MOD( m, mout1 ) .EQ. 0 ) THEN

        DO k = ks1, ke1
          DO j = js1, je1
            zexp1(j,k) = zo1(j,k)
            IF( CELL1(j,k).EQ.0.) THEN       ! research setup
              zexp1(j,k) = -1.
            END IF
          END DO
        END DO

        WRITE (101) ( ( zexp1(j,k),j=js1,je1  ), k=ks1,ke1  )
!        WRITE (101) ( ( zexp1(j,k),j=js1,je1,4  ), k=ks1,ke1,4  )
!        WRITE (101) ( ( zo1(j,k)  ,j=js1,je1,4  ), k=ks1,ke1,4  )
!        WRITE (102) ( ( uo1(j,k)  ,j=js1,je1,4  ), k=ks1,ke1,4  )
!        WRITE (103) ( ( vo1(j,k)  ,j=js1,je1,4  ), k=ks1,ke1,4  )
!        WRITE (106) ( ( eto1(j,k) ,j=js1,je1,4  ), k=ks1,ke1,4  )
!        WRITE (109) ( ( q1(j,k)  ,j=js1,je1,4  ), k=ks1,ke1,4  )

      END IF




!      IF ( m               .GE. mout_start        &
!                           .AND.                  &
!           MOD( m, mout2 ) .EQ. 0          ) THEN
      IF ( MOD( m, mout2 ) .EQ. 0 ) THEN

       DO k = ks2, ke2
          DO j = js2, je2
            zexp2(j,k) = zo2(j,k)
          !(1). set z = -1.0cm for land part
            IF( CELL2(j,k).EQ.0.) THEN ! research setup
              zexp2(j,k) = -1.
            END IF
          END DO
        END DO
        

        WRITE (201) ( ( zexp2(j,k),j=js2,je2 ), k=ks2,ke2 )
!        WRITE (201) ( ( zo2(j,k)  ,j=js2,je2 ), k=ks2,ke2 )
!        WRITE (202) ( ( uo2(j,k)  ,j=js2,je2 ), k=ks2,ke2 )
!        WRITE (203) ( ( vo2(j,k)  ,j=js2,je2 ), k=ks2,ke2 )
!        WRITE (209) ( ( q2(j,k)  ,j=js2,je2 ), k=ks2,ke2 )

      END IF


!      IF ( m               .GE. mout_start        &
!                           .AND.                  &
!           MOD( m, mout3 ) .EQ. 0          ) THEN
	IF(M.GT.m0DC)THEN         !write when double couple is called
      IF ( MOD( m, mout3 ) .EQ. 0 ) THEN

        DO k = ks3, ke3
          DO j = js3, je3
            zexp3(j,k) = zo3(j,k)
          !(1). set z = -1.0cm for land part
            IF( CELL3(j,k).EQ.0.) THEN ! research setup
              zexp3(j,k) = -1.
            END IF
          END DO
        END DO

        WRITE (301) ( ( zexp3(j,k),j=js3,je3 ), k=ks3,ke3 )
!        WRITE (301) ( ( zo3(j,k)  ,j=js3,je3 ), k=ks3,ke3 )
!        WRITE (302) ( ( uo3(j,k)  ,j=js3,je3 ), k=ks3,ke3 )
!        WRITE (303) ( ( vo3(j,k)  ,j=js3,je3 ), k=ks3,ke3 )
!        WRITE (309) ( ( q3(j,k)  ,j=js3,je3 ), k=ks3,ke3 )
      END IF
        END IF


!      IF ( m               .GE. mout_start        &
!                           .AND.                  &
!           MOD( m, mout4 ) .EQ. 0          ) THEN

 	IF(M.GT.m0DC)THEN         !write when double couple is called

      IF ( MOD( m, mout4 ) .EQ. 0 ) THEN

        DO k = ks4, ke4
          DO j = js4, je4
            zexp4(j,k) = zo4(j,k)
          !(1). set z = -1.0cm for land part
            IF( CELL4(j,k).EQ.0.) THEN ! research setup
              zexp4(j,k) = -1.
            END IF
          END DO
        END DO

        WRITE (401) ( ( zexp4(j,k),j=js4,je4 ), k=ks4,ke4 )
!        WRITE (401) ( ( zo4(j,k)  ,j=js4,je4 ), k=ks4,ke4 )
!        WRITE (402) ( ( uo4(j,k)  ,j=js4,je4 ), k=ks4,ke4 )
!        WRITE (403) ( ( vo4(j,k)  ,j=js4,je4 ), k=ks4,ke4 )
!        WRITE (409) ( ( q4(j,k)  ,j=js4,je4 ), k=ks4,ke4 )
      END IF
           END IF

END SUBROUTINE

!---------------------------------------------------------------------------------------
SUBROUTINE ZMAX_SEARCH ( zmax, z_max, jmax, kmax, zo, CELL, js, je, ks, ke )
!---------------------------------------------------------------------------------------
!      USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE

!  (OUT)...
      INTEGER:: jmax, kmax

!  (INOUT)...
      REAL, DIMENSION(js:je,ks:ke):: zmax
      REAL::  z_max

!  (IN)...
      REAL, DIMENSION(js:je,ks:ke):: zo
      INTEGER*2, DIMENSION(js:je,ks:ke):: CELL
      INTEGER:: js, je, ks, ke

!  (INTERNAL)...
      INTEGER:: j, k

      DO k=ks,ke
        DO j=js,je

          IF(zo(j,k)   .GE. zmax(j,k)         &
                       .AND.                  &
             CELL(j,k) .EQ. 1          ) THEN

            zmax(j,k) = zo(j,k)

          END IF

          IF(zmax(j,k).GT.z_max)THEN
            z_max = zmax(j,k)
            jmax  = j
            kmax  = k
          END IF

        END do
      END DO

END SUBROUTINE

!---------------------------------------------------------------------------------------
SUBROUTINE OUTPUT_ZMAX1
!---------------------------------------------------------------------------------------
!      USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE
      INTEGER:: j, k

!... save final maximum zeta, zmax(j,k)

      OPEN ( UNIT=111, FILE='zmax1.dat', FORM='unformatted', STATUS='unknown')
      WRITE (111) ( (zmax1(j,k), j=js1,je1 ), k=ks1,ke1 )


END SUBROUTINE

!---------------------------------------------------------------------------------------
SUBROUTINE OUTPUT_ZMAX2
!---------------------------------------------------------------------------------------
!      USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE
      INTEGER:: j, k

!... save final maximum zeta, zmax(j,k)

      OPEN ( UNIT=211, FILE='zmax2.dat', FORM='unformatted', STATUS='unknown')
      WRITE (211) ( (zmax2(j,k), j=js2,je2 ), k=ks2,ke2 )

END SUBROUTINE

!---------------------------------------------------------------------------------------
SUBROUTINE OUTPUT_ZMAX3
!---------------------------------------------------------------------------------------
      !USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE
      INTEGER:: j, k

!... save final maximum zeta, zmax(j,k)

      OPEN ( UNIT=311, FILE='zmax3.dat', FORM='unformatted', STATUS='unknown')
      WRITE (311) ( (zmax3(j,k), j=js3,je3 ), k=ks3,ke3 )

END SUBROUTINE

!---------------------------------------------------------------------------------------
SUBROUTINE OUTPUT_ZMAX4
!---------------------------------------------------------------------------------------
      !USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE
      INTEGER:: j, k

!... save final maximum zeta, zmax(j,k)

      OPEN ( UNIT=411, FILE='zmax4.dat', FORM='unformatted', STATUS='unknown')
      WRITE (411) ( (zmax4(j,k), j=js4,je4 ), k=ks4,ke4 )

END SUBROUTINE

!---------------------------------------------------------------------------------------
SUBROUTINE OPEN_WATER_LEVEL_STATIONS_UAF
!---------------------------------------------------------------------------------------
! This program open the water level data output files
! In this program the total number of water level stations, obsnum, are defined
! in "ALLINPUT" block.
!
!   Z.Kowalik  2007 IMS,      University of Alaska Fairbanks
!   J.Horrillo 2007 IMS,      University of Alaska Fairbanks
!---------------------------------------------------------------------------------------
!      USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE

      INTEGER                        :: units
      INTEGER                        :: i

      INTEGER,PARAMETER:: obsnum=36  ! up to 31 in the first grid

      INTEGER,DIMENSION(obsnum):: nums         !, jobs, kobs
      REAL,DIMENSION(obsnum):: longs, lats
      CHARACTER*14,DIMENSION(obsnum):: obsname
      CHARACTER*16 names



      OPEN ( UNIT=51, FILE='station_al.dat', STATUS='unknown')

      DO i=1,obsnum
        READ(51,*) nums(i), jobs(i), kobs(i), longs(i), lats(i), obsname(i)
      ENDDO

      PRINT*,nums(33), jobs(33),  obsname(33)
      PRINT*,nums(36), jobs(36),  obsname(36)

      DO i=1,obsnum
        units = 700 +i
        WRITE(names,'(A7,I2.2,A4)')'station',i,'.dat'
        IF(.not.RESTART)then
          OPEN ( UNIT=units, FILE=names, STATUS='unknown')
        else
          OPEN ( UNIT=units, FILE=names, STATUS='unknown',POSITION='append')
        END if
      END DO

END SUBROUTINE

!---------------------------------------------------------------------
SUBROUTINE OPEN_WATER_LEVEL_STATIONS_AVERAGED
!---------------------------------------------------------------------
!             OPEN Water Level Stations OUTPUT Files
!                 for Averaged Water Level Records
!---------------------------------------------------------------------
!
!  This program opens the OUTPUT files for the averaged water level
!  records and calculates the j,k at the water level stations, jobs_A
!  and kobs_A, and gives ID number for each station, obsGrid_A.
!
!  ID number, obsGrid_A, tells which grid's j,k is used to output
!  water level. For example, if obsGrid_A=3, jobs_A and kobs_A use
!  the 3rd grid's j and k (j3 and k3) to output the water level.
!
!
!  NOTE: need to comment out the IF statement in the first LOOP
!        for the grids that are NOT used in computation.
!
!   Z.Kowalik  2006  IMS, University of Alaska Fairbanks
!   Y.Yamazaki 2007  ORE, University of Hawaii Manoa
!---------------------------------------------------------------------
!      USE ALL_INPUTS

      IMPLICIT NONE

      INTEGER                          :: units
      INTEGER                          :: i
     !INTEGER, DIMENSION(nobsmax_A)    :: jobs_A, kobs_A ! j,k for water level stations
     !INTEGER, DIMENSION(nobsmax_A)    :: obsGrid_A    ! ID to know Grid (Doamin) where the
                                                   ! water level stations are located

      REAL,DIMENSION(nobsmax_A)        :: xobs, yobs
      CHARACTER*20,DIMENSION(nobsmax_A):: obsname
      CHARACTER*16 names
      CHARACTER*16 header

      OPEN ( UNIT=52,                                   &
             FILE='water_level_stations_averaged.txt',  &
             STATUS='unknown')
      READ(52,*) ! header line 1
      READ(52,*) ! header line 2
      READ(52,*) ! header line 3
      READ(52,*) ! header line 4
      READ(52,*) ! header line 5
      READ(52,'(28x,i5)') obsnum_A  ! read total number of station in line 6
      READ(52,*) ! header line 7
      READ(52,*) ! header line 8
      READ(52,*) ! header line 9
      READ(52,*) ! header line 10


      DO i=1,obsnum_A
        READ(52,*) xobs(i), yobs(i), obsname(i)
        obsname(i) = TRIM( ADJUSTL( obsname(i) ) )//"_A.dat"


        obsGrid_A(i) = 1
        jobs_A(i) = NINT( ( ( xobs(i) -lonL1 ) )/dlon1 ) +1  ! xobs is in (deg)
        kobs_A(i) = NINT( ( ( yobs(i) -latB1 ) )/dlat1 ) +1  ! yobs is in (deh)

        IF ( xobs(i) .GE. lonL2 .AND. xobs(i) .LE. lonR2          &
                                .AND.                             &
             yobs(i) .GE. latB2 .AND. yobs(i) .LE. latT2  ) THEN
          obsGrid_A(i) = 2
          jobs_A(i) = NINT( ( ( xobs(i) -lonL2 ) )/dlon2 ) +1  ! xobs is in (deg)
          kobs_A(i) = NINT( ( ( yobs(i) -latB2 ) )/dlat2 ) +1  ! yobs is in (deh)

          IF ( xobs(i) .GE. lonL3 .AND. xobs(i) .LE. lonR3        &
                                  .AND.                           &
               yobs(i) .GE. latB3 .AND. yobs(i) .LE. latT3  ) THEN
            obsGrid_A(i) = 3
            jobs_A(i) = NINT( ( ( xobs(i) -lonL3 ) )/dlon3 ) +1  ! xobs is in (deg)
            kobs_A(i) = NINT( ( ( yobs(i) -latB3 ) )/dlat3 ) +1  ! yobs is in (deh)

            IF ( xobs(i) .GE. lonL4 .AND. xobs(i) .LE. lonR4        &
                                    .AND.                           &
                 yobs(i) .GE. latB4 .AND. yobs(i) .LE. latT4  ) THEN
              obsGrid_A(i) = 4
              jobs_A(i) = NINT( ( ( xobs(i) -lonL4 ) )/dlon4 ) +1  ! xobs is in (deg)
              kobs_A(i) = NINT( ( ( yobs(i) -latB4 ) )/dlat4 ) +1  ! yobs is in (deh)
            END IF

          END IF

        END IF

        PRINT*,obsname(i),': j=',jobs_A(i),'k=',kobs_A(i),'obsGrid_A=',obsGrid_A(i)

      ENDDO

      CLOSE(52)

      DO i=1,obsnum_A
        units = 800 +i
       !WRITE(names,'(A6,A2,A4)')'gauge_',obsname(i),'.dat'
        OPEN ( UNIT=units, FILE=obsname(i), STATUS='unknown')
       !OPEN ( UNIT=units, FILE=obsname(i), STATUS='unknown',POSITION='append')
      END DO


END SUBROUTINE



!---------------------------------------------------------------------------------------
SUBROUTINE OUTPUT_WATER_LEVEL_STATION
!---------------------------------------------------------------------------------------
! this program search j,k at the water level station and write the sea level
! at each gauge output time step (mgout)
!
!---------------------------------------------------------------------------------------
!      USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE

      INTEGER:: units
      INTEGER:: i
      INTEGER,PARAMETER:: obsnum=36  ! up to 31 in the first grid
      IF ( MOD(m,mgout) .EQ. 0 ) THEN
        DO i=1,obsnum-5
          zobs(i) = zo1( jobs(i), kobs(i) )  !first domain
          units   = 700 +i
          WRITE(units,*) zobs(i)   ! write gauge records
        END DO
         DO i=obsnum-4,obsnum-3
          zobs(i) = zo2( jobs(i), kobs(i) ) !second domain
          units   = 700 +i
          WRITE(units,*) zobs(i)   ! write gauge records
        END DO
         DO i=obsnum-2,obsnum-1
          zobs(i) = zo3( jobs(i), kobs(i) ) !third domain
          units   = 700 +i
          WRITE(units,*) zobs(i)   ! write gauge records
        END DO

         DO i=obsnum,obsnum
          zobs(i) = zo4( jobs(i), kobs(i) ) !fourth domain
          units   = 700 +i
          WRITE(units,*) zobs(i)   ! write gauge records
        END DO
      END IF

END SUBROUTINE

!---------------------------------------------------------------------------------------
SUBROUTINE OUTPUT_WATER_LEVEL_STATION_AVERAGED
!---------------------------------------------------------------------------------------
! This program averages the sea level at each water level station for Tavg (s) search
! and output every Tavg (s).
!
!   Z.Kowalik  2007 IMS,      University of Alaska Fairbanks
!   J.Horrillo 2007 IMS,      University of Alaska Fairbanks
!---------------------------------------------------------------------------------------
!      USE ALL_INPUT_VARIABLES_DEC
      IMPLICIT NONE

      REAL,PARAMETER::    Tavg = 60.0       ! (sec) averaged period
      INTEGER,PARAMETER:: mavg = Tavg /dt1

      INTEGER:: units
      INTEGER:: i

      IF(MOD(m,mavg).EQ.0) THEN
     !average the sum of zo when m reaches MOD(m,mavg).EQ.0
        DO i=1,obsnum
          zobs(i) = zo1( jobs(i), kobs(i) ) +zobs(i)

!... cal. of averaged sea level
          zobs(i) = zobs(i) /FLOAT(mavg)

          units   = 700 +i
          WRITE(units,*) zobs(i)   ! write gauge records
          zobs(i) = 0.0

        END DO

      ELSE
     !sum zo till m reaches MOD(m,mavg).EQ.0
        DO i=1,obsnum
          zobs(i) = zo1( jobs(i), kobs(i) ) +zobs(i)
        END DO

      END IF


END SUBROUTINE

!---------------------------------------------------------------------
SUBROUTINE OUTPUT_WATER_LEVEL_AVERAGED_INIT_GRID_1
!---------------------------------------------------------------------
!         OUTPUT Initial Water Level at Stations in 1st Grid
!---------------------------------------------------------------------
!
!  This program outputs the initial water level for the averaged water
!  level records at the water level stations located in the 1st grid
!  (domain).
!
!
!   Z.Kowalik  2006  IMS, University of Alaska Fairbanks
!   Y.Yamazaki 2007  ORE, University of Hawaii Manoa
!---------------------------------------------------------------------
!      USE ALL_INPUTS
!      USE ALL_VARIABLES_GRID_1

      IMPLICIT NONE

      INTEGER:: units
      INTEGER:: i

      DO i=1,obsnum_A
        IF ( obsGrid_A(i) .EQ. 1 ) THEN
          zobs_A(i) = zo1( jobs_A(i), kobs_A(i) )
          units   = 800 +i
          WRITE(units,*) zobs_A(i)                ! write zo at each station
          zobs_A(i) = 0.0                         ! reset zobs_A ar 0.0
        END IF
      END DO

END SUBROUTINE

!---------------------------------------------------------------------
SUBROUTINE OUTPUT_WATER_LEVEL_AVERAGED_INIT_GRID_2
!---------------------------------------------------------------------
!         OUTPUT Initial Water Level at Stations in 2nd Grid
!---------------------------------------------------------------------
!
!  This program outputs the initial water level for the averaged water
!  level records at the water level stations located in the 2nd grid
!  (domain).
!
!
!   Z.Kowalik  2006  IMS, University of Alaska Fairbanks
!   Y.Yamazaki 2007  ORE, University of Hawaii Manoa
!---------------------------------------------------------------------
!      USE ALL_INPUTS
!      USE ALL_VARIABLES_GRID_2

      IMPLICIT NONE

      INTEGER:: units
      INTEGER:: i

      DO i=1,obsnum_A
        IF ( obsGrid_A(i) .EQ. 2 ) THEN

          zobs_A(i) = zo2( jobs_A(i), kobs_A(i) )
          units   = 800 +i
          WRITE(units,*) zobs_A(i)                ! write zo at each station
          zobs_A(i) = 0.0                         ! reset zobs_A ar 0.0
        END IF
      END DO

END SUBROUTINE

!---------------------------------------------------------------------
SUBROUTINE OUTPUT_WATER_LEVEL_AVERAGED_INIT_GRID_3
!---------------------------------------------------------------------
!         OUTPUT Initial Water Level at Stations in 3rd Grid
!---------------------------------------------------------------------
!
!  This program outputs the initial water level for the averaged water
!  level records at the water level stations located in the 3rd Grid
!  (domain).
!
!
!   Z.Kowalik  2006  IMS, University of Alaska Fairbanks
!   Y.Yamazaki 2007  ORE, University of Hawaii Manoa
!---------------------------------------------------------------------
!      USE ALL_INPUTS
!      USE ALL_VARIABLES_GRID_3

      IMPLICIT NONE

      INTEGER:: units
      INTEGER:: i

      DO i=1,obsnum_A
        IF ( obsGrid_A(i) .EQ. 3 ) THEN
          zobs_A(i) = zo3( jobs_A(i), kobs_A(i) )
          units   = 800 +i
          WRITE(units,*) zobs_A(i)                ! write zo at each station
          zobs_A(i) = 0.0                         ! reset zobs_A ar 0.0
        END IF
      END DO


END SUBROUTINE

!---------------------------------------------------------------------
SUBROUTINE OUTPUT_WATER_LEVEL_AVERAGED_INIT_GRID_4
!---------------------------------------------------------------------
!         OUTPUT Initial Water Level at Stations in 4th Grid
!---------------------------------------------------------------------
!
!  This program outputs the initial water level for the averaged water
!  level records at the water level stations located in the 4th Grid
!  (domain).
!
!
!   Z.Kowalik  2006  IMS, University of Alaska Fairbanks
!   Y.Yamazaki 2007  ORE, University of Hawaii Manoa
!---------------------------------------------------------------------
!      USE ALL_INPUTS
!      USE ALL_VARIABLES_GRID_4

      IMPLICIT NONE

      INTEGER:: units
      INTEGER:: i

      DO i=1,obsnum_A
        IF ( obsGrid_A(i) .EQ. 4 ) THEN
          zobs_A(i) = zo4( jobs_A(i), kobs_A(i) )
          units   = 800 +i
          WRITE(units,*) zobs_A(i)                ! write zo at each station
          zobs_A(i) = 0.0                         ! reset zobs_A ar 0.0
        END IF
      END DO

END SUBROUTINE

!---------------------------------------------------------------------
SUBROUTINE OUTPUT_WATER_LEVEL_AVERAGED_GRID_1
!---------------------------------------------------------------------
!           OUTPUT Water Level at Stations in 1st Grid
!---------------------------------------------------------------------
!
!  This program outputs the averaged water levels at the water level
!  stations located in the 1st grid (domain)
!
!
!   Z.Kowalik  2006  IMS, University of Alaska Fairbanks
!   Y.Yamazaki 2007  ORE, University of Hawaii Manoa
!---------------------------------------------------------------------
!      USE ALL_INPUTS
!      USE ALL_VARIABLES_GRID_1

      IMPLICIT NONE

     !REAL,   PARAMETER:: dt_Agout  = 60.0            ! in all_inputs
     !INTEGER,PARAMETER:: dm_Agout1 = dt_Agout /dt1   ! in all_inputs
      INTEGER:: units
      INTEGER:: i

      IF(MOD(m,dm_Agout1).EQ.0) THEN
     ! average the sum of zo when m reaches MOD(m,dm_Agout1).EQ.0
        DO i=1,obsnum_A
          IF ( obsGrid_A(i) .EQ. 1 ) THEN

            zobs_A(i) = zo1( jobs_A(i), kobs_A(i) ) +zobs_A(i)

            zobs_A(i) = zobs_A(i) /FLOAT(dm_Agout1) ! cal. of averaged sea level

            units   = 800 +i
            WRITE(units,*) zobs_A(i)                ! write zo at each station

            zobs_A(i) = 0.0                         ! reset zobs_A ar 0.0

          END IF
        END DO
      ELSE
     ! sum zo till m reaches MOD(m,dm_Agout1).EQ.0
        DO i=1,obsnum_A
          IF ( obsGrid_A(i) .EQ. 1 ) THEN

            zobs_A(i) = zo1( jobs_A(i), kobs_A(i) ) +zobs_A(i)

          END IF
        END DO
      END IF


END SUBROUTINE

!---------------------------------------------------------------------
SUBROUTINE OUTPUT_WATER_LEVEL_AVERAGED_GRID_2
!---------------------------------------------------------------------
!           OUTPUT Water Level at Stations in 2nd Grid
!---------------------------------------------------------------------
!
!  This program outputs the averaged water levels at the water level
!  stations located in the 2nd grid (domain)
!
!
!   Z.Kowalik  2006  IMS, University of Alaska Fairbanks
!   Y.Yamazaki 2007  ORE, University of Hawaii Manoa
!---------------------------------------------------------------------
!      USE ALL_INPUTS
!      USE ALL_VARIABLES_GRID_2

      IMPLICIT NONE

     !REAL,   PARAMETER:: dt_Agout  = 60.0            ! in all_inputs
     !INTEGER,PARAMETER:: dm_Agout2 = dt_Agout /dt2   ! in all_inputs
      INTEGER:: units
      INTEGER:: i

      IF(MOD(m2,dm_Agout2).EQ.0) THEN
     ! average the sum of zo when m reaches MOD(m,dm_Agout2).EQ.0
        DO i=1,obsnum_A
          IF ( obsGrid_A(i) .EQ. 2 ) THEN

            zobs_A(i) = zo2( jobs_A(i), kobs_A(i) ) +zobs_A(i)

            zobs_A(i) = zobs_A(i) /FLOAT(dm_Agout2) ! cal. of averaged sea level

            units   = 800 +i
            WRITE(units,*) zobs_A(i)                ! write zo at each station

            zobs_A(i) = 0.0                         ! reset zobs_A ar 0.0

          END IF
        END DO
      ELSE
     ! sum zo till m reaches MOD(m,dm_Agout2).EQ.0
        DO i=1,obsnum_A
          IF ( obsGrid_A(i) .EQ. 2 ) THEN

            zobs_A(i) = zo2( jobs_A(i), kobs_A(i) ) +zobs_A(i)

          END IF
        END DO
      END IF


END SUBROUTINE

!---------------------------------------------------------------------
SUBROUTINE OUTPUT_WATER_LEVEL_AVERAGED_GRID_3
!---------------------------------------------------------------------
!           OUTPUT Water Level at Stations in 3rd Grid
!---------------------------------------------------------------------
!
!  This program outputs the averaged water levels at the water level
!  stations located in the 3rd grid (domain)
!
!
!   Z.Kowalik  2006  IMS, University of Alaska Fairbanks
!   Y.Yamazaki 2007  ORE, University of Hawaii Manoa
!---------------------------------------------------------------------
!      USE ALL_INPUTS
!      USE ALL_VARIABLES_GRID_3

      IMPLICIT NONE

     !REAL,   PARAMETER:: dt_Agout  = 60.0            ! in all_inputs
     !INTEGER,PARAMETER:: dm_Agout3 = dt_Agout /dt3   ! in all_inputs
      INTEGER:: units
      INTEGER:: i

      IF(MOD(m3,dm_Agout3).EQ.0) THEN
     ! average the sum of zo when m reaches MOD(m,dm_Agout3).EQ.0
        DO i=1,obsnum_A
          IF ( obsGrid_A(i) .EQ. 3 ) THEN

            zobs_A(i) = zo3( jobs_A(i), kobs_A(i) ) +zobs_A(i)

            zobs_A(i) = zobs_A(i) /FLOAT(dm_Agout3) ! cal. of averaged sea level


            units   = 800 +i
            WRITE(units,*) zobs_A(i)                ! write zo at each station

            zobs_A(i) = 0.0                         ! reset zobs_A ar 0.0

          END IF
        END DO
      ELSE
     ! sum zo till m reaches MOD(m,dm_Agout3).EQ.0
        DO i=1,obsnum_A
          IF ( obsGrid_A(i) .EQ. 3 ) THEN

            zobs_A(i) = zo3( jobs_A(i), kobs_A(i) ) +zobs_A(i)

          END IF
        END DO
      END IF


END SUBROUTINE

!---------------------------------------------------------------------
SUBROUTINE OUTPUT_WATER_LEVEL_AVERAGED_GRID_4
!---------------------------------------------------------------------
!           OUTPUT Water Level at Stations in 4th Grid
!---------------------------------------------------------------------
!
!  This program outputs the averaged water levels at the water level
!  stations located in the 4th grid (domain)
!
!
!   Z.Kowalik  2006  IMS, University of Alaska Fairbanks
!   Y.Yamazaki 2007  ORE, University of Hawaii Manoa
!---------------------------------------------------------------------
!      USE ALL_INPUTS
!      USE ALL_VARIABLES_GRID_4

      IMPLICIT NONE

     !REAL,   PARAMETER:: dt_Agout  = 60.0            ! in all_inputs
     !INTEGER,PARAMETER:: dm_Agout4 = dt_Agout /dt4   ! in all_inputs
      INTEGER:: units
      INTEGER:: i

      IF(MOD(m4,dm_Agout4).EQ.0) THEN
     ! average the sum of zo when m reaches MOD(m,dm_Agout4).EQ.0
        DO i=1,obsnum_A
          IF ( obsGrid_A(i) .EQ. 4 ) THEN

            zobs_A(i) = zo4( jobs_A(i), kobs_A(i) ) +zobs_A(i)

            zobs_A(i) = zobs_A(i) /FLOAT(dm_Agout4) ! cal. of averaged sea level



            units   = 800 +i
            WRITE(units,*) zobs_A(i)                ! write zo at each station

            zobs_A(i) = 0.0                         ! reset zobs_A ar 0.0

          END IF
        END DO
      ELSE
     ! sum zo till m reaches MOD(m,dm_Agout4).EQ.0
        DO i=1,obsnum_A
          IF ( obsGrid_A(i) .EQ. 4 ) THEN

            zobs_A(i) = zo4( jobs_A(i), kobs_A(i) ) +zobs_A(i)

          END IF
        END DO
      END IF


END SUBROUTINE





!initial condition solvers
!---------------------------------------------------------------------
SUBROUTINE SURFACE_OKADA_REC_1 ( zo, zn, H, dx, dy, csz,    &
                                 lonL, latB, dlon, dlat,    &
                                 js, je, ks, ke, nsmooth )

!---------------------------------------------------------------------
!
!  This program sets up the initial codition of sea surface
!  deformation at t =0 sec using the Okada solution of rectanglar
!  fault (1985) .
!
!   Z.Kowalik  2006  IMS, University of Alaska Fairbanks
!   Y.Yamazaki 2007  ORE, University of Hawaii Manoa
!---------------------------------------------------------------------
!      USE ALL_INPUTS
!      IMPLICIT NONE

!      INTEGER,PARAMETER:: nsmooth = 50       ! (-)

!  (INOUT)...
      REAL, DIMENSION(js:je,ks:ke):: zo, zn

!  (IN)...
      REAL, DIMENSION(js:je,ks:ke):: H

      REAL, DIMENSION(ks:ke):: csz
      REAL:: dx, dy
      REAL:: dlon, dlat
      REAL:: lonL, latB
      INTEGER:: js, je, ks, ke
      INTEGER:: nsmooth

!  (INTERNAL)...
      REAL:: Trise
      REAL(dp):: D0
      REAL(dp):: L, W
      REAL(dp):: lonR, latR, dR
      REAL(dp):: strike, dip, rake
      REAL:: xdisL, xdisR, ydisB, ydisT

      REAL(dp), DIMENSION(js:je,ks:ke)::disx, disy, disz

      NAMELIST/ fault_paramter / Trise, D0,         &
                                 lonR, latR, dR,    &
                                 L, W,              &
                                 strike, dip, rake

      NAMELIST/ dis_area / xdisL, xdisR, ydisB, ydisT

      INTEGER:: j, k, i
      INTEGER:: jdisL, jdisR, kdisB, kdisT


!     Open input file fault1.data and read the corresponding data.

      OPEN( UNIT=61, FILE = 'fault_2006_USGS.txt' )
      READ( 61, NML  = fault_paramter )
      READ( 61, NML  = dis_area )
      CLOSE(61)

      PRINT *, ' - Fault 1 ------------------------------------------ '
      PRINT *, ' '
      PRINT *, '  rise time =',Trise,'(s)'
      PRINT *, ' '
      PRINT *, ' Reference Point'
      PRINT *, '             ',latR,'(deg N)'
      PRINT *, '             ',lonR,'(deg E)'
      PRINT *, ' Fault Dimension'
      PRINT *, '   L        =', L/1000.,  '(km)'
      PRINT *, '   W        =', W/1000.,   '(km)'
      PRINT *, ' '
      PRINT *, '  depth     =',dR/1000.,'(km)'
      PRINT *, ' '
      PRINT *, '  slip      =', D0,       '(m)'
      PRINT *, ' '
      PRINT *, '  strike    =', strike,'(deg)'
      PRINT *, '  dip       =', dip,   '(deg)'
      PRINT *, '  rake      =', rake,  '(deg)'
      PRINT *, ' '
      PRINT *, ' ----------------------------------------------------- '

      strike = strike*rad  !phi    (rad):strike angle
      dip    = dip*rad     !delta  (rad):dip angle
      rake   = rake*rad    !lambda (rad):rake angle

      CALL OKADA3D_REC( js, je, ks, ke,       &
                        dx, dy, csz,          &
                        lonL, latB,           &
                        lonR, latR, dR,       &
                        L, W, D0,             &
                        strike, dip,  rake,   &
                        disx,   disy, disz  )

!      WRITE (151) ( (disx(j,k) ,j=js,je ), k=ks,ke )
!      WRITE (152) ( (disy(j,k) ,j=js,je ), k=ks,ke )
!      WRITE (153) ( (disz(j,k) ,j=js,je ), k=ks,ke )

      jdisL  = (xdisL-lonL)/dlon+1   ! j at LEFT   edge of OKADA Deformation applied area
      jdisR  = (xdisR-lonL)/dlon+1   ! j at RIGHT  edge of OKADA Deformation applied area
      kdisB  = (ydisB-latB)/dlat+1   ! k at BOTTOM edge of OKADA Deformation applied area
      kdisT  = (ydisT-latB)/dlat+1   ! k at TOP    edge of OKADA Deformation applied area

      PRINT *, '  jdisL    =', jdisL
      PRINT *, '  jdisR    =', jdisR
      PRINT *, '  kdisB    =', kdisB
      PRINT *, '  kdisT    =', kdisT

!... input sea surface deformation within the area, (jdisL:jdisR,kdisB:kdisT)

!      disz = 200.0 ! for Boundary TEST
      DO k =kdisB,kdisT
        DO j=jdisL,jdisR
          IF (H(j,k) .GT. EPSILON ) THEN
            zo(j,k) = disz(j,k)
          ENDIF
        END DO
      END DO
        GOTO  300
!... smooth the "nsmooth" points from the boundaries of the sea surface
!    deformation applied area, (jdisL:jdisR,kdisB:kdisT)
      DO i =1, nsmooth
        k =kdisB-i
        DO j=jdisL-i,jdisR+i
          IF (H(j,k) .GT. EPSILON ) THEN
            zo(j,k)=FLOAT(nsmooth -i)/FLOAT(nsmooth)*disz(j,k)
          ENDIF
        END DO

        k =kdisT+i
        DO j=jdisL-i,jdisR+i
          IF (H(j,k) .GT. EPSILON ) THEN
            zo(j,k)=FLOAT(nsmooth -i)/FLOAT(nsmooth)*disz(j,k)
          ENDIF
        END DO

        j=jdisL-i
        DO k =kdisB-i+1,kdisT+i-1
          IF (H(j,k) .GT. EPSILON ) THEN
            zo(j,k)=FLOAT(nsmooth -i)/FLOAT(nsmooth)*disz(j,k)
          ENDIF
        END DO

        j=jdisR+i
        DO k =kdisB-i+1,kdisT+i-1
          IF (H(j,k) .GT. EPSILON ) THEN
            zo(j,k)=FLOAT(nsmooth -i)/FLOAT(nsmooth)*disz(j,k)
          ENDIF
        END DO
      END DO
300      continue
      DO j=js,je
        DO k=ks,ke
! or choose instead of 5.
          IF (Abs (zo(j,k)) .LT. 5.)zo(j,k)=0.

        END DO
      END DO

      zn=zo

END SUBROUTINE

!---------------------------------------------------------------------
SUBROUTINE SURFACE_OKADA_REC_2 ( zo, zn, H, dx, dy, csz,    &
                                 lonL, latB, dlon, dlat,    &
                                 js, je, ks, ke, nsmooth )

!---------------------------------------------------------------------
!
!  This program sets up the initial codition of sea surface
!  deformation at t =0 sec using the Okada solution of rectanglar
!  fault (1985) .
!
!   Z.Kowalik  2006  IMS, University of Alaska Fairbanks
!   Y.Yamazaki 2007  ORE, University of Hawaii Manoa
!---------------------------------------------------------------------
!      USE ALL_INPUTS
      IMPLICIT NONE

!      INTEGER,PARAMETER:: nsmooth = 50       ! (-)

!  (INOUT)...
      REAL, DIMENSION(js:je,ks:ke):: zo, zn

!  (IN)...
      REAL, DIMENSION(js:je,ks:ke):: H

      REAL, DIMENSION(ks:ke):: csz
      REAL:: dx, dy
      REAL:: dlon, dlat
      REAL:: lonL, latB
      INTEGER:: js, je, ks, ke
      INTEGER:: nsmooth

!  (INTERNAL)...
      REAL:: Trise
      REAL(dp):: D0
      REAL(dp):: L, W
      REAL(dp):: lonR, latR, dR
      REAL(dp):: strike, dip, rake
      REAL:: xdisL, xdisR, ydisB, ydisT

      REAL(dp), DIMENSION(js:je,ks:ke)::disx, disy, disz

      NAMELIST/ fault_paramter / Trise, D0,         &
                                 lonR, latR, dR,    &
                                 L, W,              &
                                 strike, dip, rake

      NAMELIST/ dis_area / xdisL, xdisR, ydisB, ydisT

      INTEGER:: j, k, i
      INTEGER:: jdisL, jdisR, kdisB, kdisT


!     Open input file fault1.data and read the corresponding data.

      OPEN( UNIT=61, FILE = '../input/fault_2009_ORE3_NO2.txt' )
      READ( 61, NML  = fault_paramter )
      READ( 61, NML  = dis_area )
      CLOSE(61)

      PRINT *, ' - Fault 1 ------------------------------------------ '
      PRINT *, ' '
      PRINT *, '  rise time =',Trise,'(s)'
      PRINT *, ' '
      PRINT *, ' Reference Point'
      PRINT *, '             ',latR,'(deg N)'
      PRINT *, '             ',lonR,'(deg E)'
      PRINT *, ' Fault Dimension'
      PRINT *, '   L        =', L/1000.,  '(km)'
      PRINT *, '   W        =', W/1000.,   '(km)'
      PRINT *, ' '
      PRINT *, '  depth     =',dR/1000.,'(km)'
      PRINT *, ' '
      PRINT *, '  slip      =', D0,       '(m)'
      PRINT *, ' '
      PRINT *, '  strike    =', strike,'(deg)'
      PRINT *, '  dip       =', dip,   '(deg)'
      PRINT *, '  rake      =', rake,  '(deg)'
      PRINT *, ' '
      PRINT *, ' ----------------------------------------------------- '

      strike = strike*rad  !phi    (rad):strike angle
      dip    = dip*rad     !delta  (rad):dip angle
      rake   = rake*rad    !lambda (rad):rake angle

      CALL OKADA3D_REC( js, je, ks, ke,       &
                        dx, dy, csz,          &
                        lonL, latB,           &
                        lonR, latR, dR,       &
                        L, W, D0,             &
                        strike, dip,  rake,   &
                        disx,   disy, disz  )

!      WRITE (151) ( (disx(j,k) ,j=js,je ), k=ks,ke )
!      WRITE (152) ( (disy(j,k) ,j=js,je ), k=ks,ke )
!      WRITE (153) ( (disz(j,k) ,j=js,je ), k=ks,ke )

      jdisL  = (xdisL-lonL)/dlon+1   ! j at LEFT   edge of OKADA Deformation applied area
      jdisR  = (xdisR-lonL)/dlon+1   ! j at RIGHT  edge of OKADA Deformation applied area
      kdisB  = (ydisB-latB)/dlat+1   ! k at BOTTOM edge of OKADA Deformation applied area
      kdisT  = (ydisT-latB)/dlat+1   ! k at TOP    edge of OKADA Deformation applied area

      PRINT *, '  jdisL    =', jdisL
      PRINT *, '  jdisR    =', jdisR
      PRINT *, '  kdisB    =', kdisB
      PRINT *, '  kdisT    =', kdisT

!... input sea surface deformation within the area, (jdisL:jdisR,kdisB:kdisT)

!      disz = 200.0 ! for Boundary TEST
      DO k =kdisB,kdisT
        DO j=jdisL,jdisR
          IF (H(j,k) .GT. EPSILON ) THEN
            zo(j,k) = disz(j,k)
          ENDIF
        END DO
      END DO

!... smooth the "nsmooth" points from the boundaries of the sea surface
!    deformation applied area, (jdisL:jdisR,kdisB:kdisT)
      DO i =1, nsmooth
        k =kdisB-i
        DO j=jdisL-i,jdisR+i
          IF (H(j,k) .GT. EPSILON ) THEN
            zo(j,k)=FLOAT(nsmooth -i)/FLOAT(nsmooth)*disz(j,k)
          ENDIF
        END DO

        k =kdisT+i
        DO j=jdisL-i,jdisR+i
          IF (H(j,k) .GT. EPSILON ) THEN
            zo(j,k)=FLOAT(nsmooth -i)/FLOAT(nsmooth)*disz(j,k)
          ENDIF
        END DO

        j=jdisL-i
        DO k =kdisB-i+1,kdisT+i-1
          IF (H(j,k) .GT. EPSILON ) THEN
            zo(j,k)=FLOAT(nsmooth -i)/FLOAT(nsmooth)*disz(j,k)
          ENDIF
        END DO

        j=jdisR+i
        DO k =kdisB-i+1,kdisT+i-1
          IF (H(j,k) .GT. EPSILON ) THEN
            zo(j,k)=FLOAT(nsmooth -i)/FLOAT(nsmooth)*disz(j,k)
          ENDIF
        END DO
      END DO

      zn=zo

END SUBROUTINE

!---------------------------------------------------------------------
SUBROUTINE SURFACE_OKADA_REC_3 ( zo, zn, H, dx, dy, csz,    &
                                 lonL, latB, dlon, dlat,    &
                                 js, je, ks, ke, nsmooth )

!---------------------------------------------------------------------
!
!  This program sets up the initial codition of sea surface
!  deformation at t =0 sec using the Okada solution of rectanglar
!  fault (1985) .
!
!   Z.Kowalik  2006  IMS, University of Alaska Fairbanks
!   Y.Yamazaki 2007  ORE, University of Hawaii Manoa
!---------------------------------------------------------------------
!      USE ALL_INPUTS
      IMPLICIT NONE

!      INTEGER,PARAMETER:: nsmooth = 50       ! (-)

!  (INOUT)...
      REAL, DIMENSION(js:je,ks:ke):: zo, zn

!  (IN)...
      REAL, DIMENSION(js:je,ks:ke):: H

      REAL, DIMENSION(ks:ke):: csz
      REAL:: dx, dy
      REAL:: dlon, dlat
      REAL:: lonL, latB
      INTEGER:: js, je, ks, ke
      INTEGER:: nsmooth

!  (INTERNAL)...
      REAL:: Trise
      REAL(dp):: D0
      REAL(dp):: L, W
      REAL(dp):: lonR, latR, dR
      REAL(dp):: strike, dip, rake
      REAL:: xdisL, xdisR, ydisB, ydisT

      REAL(dp), DIMENSION(js:je,ks:ke)::disx, disy, disz

      NAMELIST/ fault_paramter / Trise, D0,         &
                                 lonR, latR, dR,    &
                                 L, W,              &
                                 strike, dip, rake

      NAMELIST/ dis_area / xdisL, xdisR, ydisB, ydisT

      INTEGER:: j, k, i
      INTEGER:: jdisL, jdisR, kdisB, kdisT


!     Open input file fault1.data and read the corresponding data.

      OPEN( UNIT=61, FILE = '../input/fault_2009_ORE3_NO3.txt' )
      READ( 61, NML  = fault_paramter )
      READ( 61, NML  = dis_area )
      CLOSE(61)

      PRINT *, ' - Fault 1 ------------------------------------------ '
      PRINT *, ' '
      PRINT *, '  rise time =',Trise,'(s)'
      PRINT *, ' '
      PRINT *, ' Reference Point'
      PRINT *, '             ',latR,'(deg N)'
      PRINT *, '             ',lonR,'(deg E)'
      PRINT *, ' Fault Dimension'
      PRINT *, '   L        =', L/1000.,  '(km)'
      PRINT *, '   W        =', W/1000.,   '(km)'
      PRINT *, ' '
      PRINT *, '  depth     =',dR/1000.,'(km)'
      PRINT *, ' '
      PRINT *, '  slip      =', D0,       '(m)'
      PRINT *, ' '
      PRINT *, '  strike    =', strike,'(deg)'
      PRINT *, '  dip       =', dip,   '(deg)'
      PRINT *, '  rake      =', rake,  '(deg)'
      PRINT *, ' '
      PRINT *, ' ----------------------------------------------------- '

      strike = strike*rad  !phi    (rad):strike angle
      dip    = dip*rad     !delta  (rad):dip angle
      rake   = rake*rad    !lambda (rad):rake angle

      CALL OKADA3D_REC( js, je, ks, ke,       &
                        dx, dy, csz,          &
                        lonL, latB,           &
                        lonR, latR, dR,       &
                        L, W, D0,             &
                        strike, dip,  rake,   &
                        disx,   disy, disz  )

!      WRITE (151) ( (disx(j,k) ,j=js,je ), k=ks,ke )
!      WRITE (152) ( (disy(j,k) ,j=js,je ), k=ks,ke )
!      WRITE (153) ( (disz(j,k) ,j=js,je ), k=ks,ke )

      jdisL  = (xdisL-lonL)/dlon+1   ! j at LEFT   edge of OKADA Deformation applied area
      jdisR  = (xdisR-lonL)/dlon+1   ! j at RIGHT  edge of OKADA Deformation applied area
      kdisB  = (ydisB-latB)/dlat+1   ! k at BOTTOM edge of OKADA Deformation applied area
      kdisT  = (ydisT-latB)/dlat+1   ! k at TOP    edge of OKADA Deformation applied area

      PRINT *, '  jdisL    =', jdisL
      PRINT *, '  jdisR    =', jdisR
      PRINT *, '  kdisB    =', kdisB
      PRINT *, '  kdisT    =', kdisT

!... input sea surface deformation within the area, (jdisL:jdisR,kdisB:kdisT)

!      disz = 200.0 ! for Boundary TEST
      DO k =kdisB,kdisT
        DO j=jdisL,jdisR
          IF (H(j,k) .GT. EPSILON ) THEN
            zo(j,k) = disz(j,k)
          ENDIF
        END DO
      END DO

!... smooth the "nsmooth" points from the boundaries of the sea surface
!    deformation applied area, (jdisL:jdisR,kdisB:kdisT)
      DO i =1, nsmooth
        k =kdisB-i
        DO j=jdisL-i,jdisR+i
          IF (H(j,k) .GT. EPSILON ) THEN
            zo(j,k)=FLOAT(nsmooth -i)/FLOAT(nsmooth)*disz(j,k)
          ENDIF
        END DO

        k =kdisT+i
        DO j=jdisL-i,jdisR+i
          IF (H(j,k) .GT. EPSILON ) THEN
            zo(j,k)=FLOAT(nsmooth -i)/FLOAT(nsmooth)*disz(j,k)
          ENDIF
        END DO

        j=jdisL-i
        DO k =kdisB-i+1,kdisT+i-1
          IF (H(j,k) .GT. EPSILON ) THEN
            zo(j,k)=FLOAT(nsmooth -i)/FLOAT(nsmooth)*disz(j,k)
          ENDIF
        END DO

        j=jdisR+i
        DO k =kdisB-i+1,kdisT+i-1
          IF (H(j,k) .GT. EPSILON ) THEN
            zo(j,k)=FLOAT(nsmooth -i)/FLOAT(nsmooth)*disz(j,k)
          ENDIF
        END DO
      END DO

      zn=zo

END SUBROUTINE

!---------------------------------------------------------------------
SUBROUTINE OKADA3D_REC( js, je, ks, ke,       &
                        dx, dy, csz,          &
                        lonL, latB,           &
                        lonR, latR, dR,       &
                        L, W, D0,             &
                        strike, dip,  rake,   &
                        disx,   disy, disz  )
!---------------------------------------------------------------------
!        3D Half-Space Static Surface Displacement Solution
!                       for Rectanglar Fault
!---------------------------------------------------------------------
!  This subroutine solves the analytical solution of Okada (1985)
!  for 3D half-space static surface displacement due to rectanglar
!  fault based on
!
!    Okada, Y. 1985. Surface deformation due to shear and
!                    tensile faults in a half space.
!                    Bulletin of the Seismological Society
!                    of America,75(4) 1135-1154.
!
!   Y.Yong     2006  ORE, University of Hawaii Manoa
!   A.Sanchez  2007  ORE, University of Hawaii Manoa
!   Y.Yamazaki 2007  ORE, University of Hawaii Manoa
!---------------------------------------------------------------------
!      USE ALL_INPUTS
      IMPLICIT NONE

!  (IN)...
      REAL(dp):: D0
      REAL(dp):: L, W
      REAL(dp):: lonR, latR, dR
      REAL(dp):: strike, dip, rake

      REAL, DIMENSION(ks:ke):: csz
      REAL:: dx, dy
      REAL:: lonL, latB
      INTEGER:: js, je, ks, ke

!  (OUT)...
      REAL(dp), DIMENSION(js:je,ks:ke)::disx, disy, disz

!  (INTERNAL)...
      REAL(dp):: sins,coss
      REAL(dp):: sind,cosd
      REAL(dp):: sinr,cosr

      REAL(dp)::xj,yk,zl
      REAL(dp)::D0s,D0d
      REAL(dp)::pk, qk
      REAL(dp)::xjS,xjE,pkS,pkE

      REAL(dp)::fsx1,fsy1,fsz1
      REAL(dp)::fsx2,fsy2,fsz2
      REAL(dp)::fsx3,fsy3,fsz3
      REAL(dp)::fsx4,fsy4,fsz4

      REAL(dp)::fdx1,fdy1,fdz1
      REAL(dp)::fdx2,fdy2,fdz2
      REAL(dp)::fdx3,fdy3,fdz3
      REAL(dp)::fdx4,fdy4,fdz4

      REAL(dp)::dsx,dsy,dsz
      REAL(dp)::ddx,ddy,ddz

      REAL(dp), DIMENSION(js:je):: x_a
      REAL(dp), DIMENSION(ks:ke):: y_a

      INTEGER:: j, k

!      PRINT *, ' Reference Point'
!      PRINT *, '             ',latR,'(deg N)'
!      PRINT *, '             ',lonR,'(deg E)'
!      PRINT *, ' Fault Dimension'
!      PRINT *, '   L        =', L/1000.,  '(km)'
!      PRINT *, '   W        =', W/1000.,   '(km)'
!      PRINT *, ' '
!      PRINT *, '  depth     =',dR/1000.,'(km)'
!      PRINT *, ' '
!      PRINT *, '  slip      =', D0,       '(m)'
!      PRINT *, ' '
!      PRINT *, '  strike    =', strike,'(deg)'
!      PRINT *, '  dip       =', dip,   '(deg)'
!      PRINT *, '  rake      =', rake,  '(deg)'


! sine & cosine of DIP angle
    sind=SIN(dip)
    cosd=COS(dip)
    IF(ABS(cosd).LT.1.0d-4) THEN
      cosd=0.0d0
      IF(sind.GT.0.) sind= 1.0d0
      IF(sind.LT.0.) sind=-1.0d0
    ENDIF

! sine & cosine of RAKE angle
    sinr=SIN(rake)
    cosr=COS(rake)
    IF(ABS(cosr).LT.1.0d-4) THEN
      cosr=0.0d0
      IF(sinr.GT.0.) sinr= 1.0d0
      IF(sinr.LT.0.) sinr=-1.0d0
    ENDIF

! sine & cosine of STRIKE angle
    sins=SIN(strike)
    coss=COS(strike)
    IF(ABS(coss).LT.1.0d-4) THEN
      coss=0.0d0
      IF(sins.GT.0.) sins= 1.0d0
      IF(sins.LT.0.) sins=-1.0d0
    ENDIF

! strike and dip components of dislocation (slip)
    D0s = D0*cosr !strike-component
    D0d = D0*sinr !dip-component

    DO k = ks, ke
      DO j = js, je

! set reference point (lonR,latR) to (0,0) to calculate distances of
! all grid points relative to reference point
       x_a(j) = (float(j)-1)*dx*COS(latR*RAD)         -R0*(lonR-lonL)*RAD*COS(latR*RAD)  ! new 0805
                                                                                         ! match exactly with Alex matlab code
                                                                                         ! but need more TEST wirh Dr. Kowalik's
                                                                                         ! csz(k)
!      x_a(j) = (float(j)-1)*dx*csz(k)*COS(latR*RAD)  -R0*(lonR-lonL)*RAD*COS(latR*RAD)  ! Alex version
!      x_a(j) = (float(j)-1)*dx*csz(k)                -R0*(lonR-lonL)*RAD*COS(latB*RAD)  ! old COMCOT

       y_a(k) = (float(k)-1)*dy                       -R0*(latR-latB)*RAD

! x-axis in Okada 1985
! (strike direction axis)
       xj = x_a(j)*sins +y_a(k)*coss
!        xj = x_a(j)

! y-axis in Okada 1985
! (axis normal to strike direction)
       yk = -x_a(j)*coss +y_a(k)*sins
!       yk = y_a(k)

! z-axis in Okada 1985
! (vertical axis)
       zl = 0.0     ! at free surface (seabed)

! p-axis in Okada 1985
! (dip direction axis)
       pk = yk*COSD + dR*SIND

! q-axis in Okada 1985
! (axis normal to dip direction)
       qk = yk*SIND - dR*COSD

       xjS = xj     ! = x
       xjE = xj -L  ! = x -L
       pkS = pk     ! = p
       pkE = pk +W  ! = p +W

! --------------------------------------------------------------------
!  cal. of STRIKE-slip component DISLOCATION, dsx, dsy, dsz
! --------------------------------------------------------------------
      CALL STRIKEDIS(qk,xjS,pkE,fsx1,fsy1,fsz1,sind,cosd) ! f( x ,p+W)
      CALL STRIKEDIS(qk,xjS,pkS,fsx2,fsy2,fsz2,sind,cosd) ! f( x , p )
      CALL STRIKEDIS(qk,xjE,pkE,fsx3,fsy3,fsz3,sind,cosd) ! f(x-L,p+W)
      CALL STRIKEDIS(qk,xjE,pkS,fsx4,fsy4,fsz4,sind,cosd) ! f(x-L, p )

! f(x,p+W) - f(x,p) - f(x-L,p+W) + f(x-L,p)
      dsx=D0s*(fsx1-fsx2-fsx3+fsx4)
      dsy=D0s*(fsy1-fsy2-fsy3+fsy4)
      dsz=D0s*(fsz1-fsz2-fsz3+fsz4)

! --------------------------------------------------------------------
!  cal. of DIP-slip component DISLOCATION, ddx, ddy, ddz
! --------------------------------------------------------------------
      CALL DIPDIS(qk,xjS,pkE,fdx1,fdy1,fdz1,sind,cosd) ! f( x ,p+W)
      CALL DIPDIS(qk,xjS,pkS,fdx2,fdy2,fdz2,sind,cosd) ! f( x , p )
      CALL DIPDIS(qk,xjE,pkE,fdx3,fdy3,fdz3,sind,cosd) ! f(x-L,p+W)
      CALL DIPDIS(qk,xjE,pkS,fdx4,fdy4,fdz4,sind,cosd) ! f(x-L, p )

! f(x,p+W) - f(x,p) - f(x-L,p+W) + f(x-L,p)
      ddx=D0d*(fdx1-fdx2-fdx3+fdx4)
      ddy=D0d*(fdy1-fdy2-fdy3+fdy4)
      ddz=D0d*(fdz1-fdz2-fdz3+fdz4)

! --------------------------------------------------------------------
!  TOTAL DISLOCATION, hdx, hdy, vdz
! --------------------------------------------------------------------

      disx(j,k) = dsx+ddx
      disy(j,k) = dsy+ddy
      disz(j,k) = dsz+ddz

      END DO
    END DO


END SUBROUTINE

!---------------------------------------------------------------------
SUBROUTINE DIPDIS (qj,xix,etap,fdx,fdy,fdz,sind,cosd)
!---------------------------------------------------------------------
!                  Dip-slip Displacement solver
!                       for Rectanglar Fault
!---------------------------------------------------------------------
!
!  This prgram solves the dip-slip component of Okada analytical
!  solution (1985) based on
!
!    Okada, Y. 1985. Surface deformation due to shear and
!                    tensile faults in a half space.
!                    Bulletin of the Seismological Society
!                    of America,75(4) 1135-1154.
!
!   Y.Yong     2006  ORE, University of Hawaii Manoa
!   Y.Yamazaki 2007  ORE, University of Hawaii Manoa
!---------------------------------------------------------------------
!      USE ALL_INPUTS
      IMPLICIT NONE

      REAL(dp):: sind,cosd
      REAL(dp):: qj,xix,etap,fdx,fdy,fdz
      REAL(dp):: I1, I2, I3, I4, I5
      REAL(dp):: ybar, dbar
      REAL(dp):: Xa, Ra
      REAL(dp):: Aa, Ba

      ybar = etap*cosd + qj*sind
      dbar = etap*sind - qj*cosd

      Ra = SQRT(xix**2 + etap**2 + qj**2)
      Xa = SQRT(xix**2 + qj**2)

      Aa = nu/(lamda+nu)
      Ba = ATAN(xix*etap/(qj*Ra))

      IF (qj.EQ.0.) Ba = 0.
!      IF (Ra+etap.EQ.0.) Ra = -etap
      IF (cosd.EQ.0.) THEN
        I1 = -Aa/2*(xix*qj/((Ra+dbar)**2))
        I3 =  Aa/2*(etap/(Ra+dbar) + (ybar*qj)/((Ra+dbar)**2) - LOG(Ra+etap))
        I4 = -Aa*qj/(Ra+dbar)
        I5 = -Aa*xix*sind/(Ra+dbar)
      ELSE
!       IF (Ra+etap.eq.0.) then
!         I4 = Aa/cosd*(log(Ra+dbar)+sind*log(Ra-etap))
!         I5 = 2.*Aa/cosd*atan((etap*(Xa+qj*cosd)+Xa*(Ra+Xa)*sind)/(xix*(Ra+Xa)*cosd))
!       ELSE
          I4 = Aa/cosd*(LOG(Ra+dbar) - sind*LOG(Ra+etap))
          I5 = 2.*Aa/cosd*ATAN((etap*(Xa+qj*cosd)+Xa*(Ra+Xa)*sind)/(xix*(Ra+Xa)*cosd))
          I1 = Aa*((-1/cosd)*(xix/(Ra+dbar))) - sind/cosd*I5
          I3 = Aa*((1/cosd)*(ybar/(Ra+dbar)) - LOG(Ra+etap))+sind/cosd*I4
          I2 = Aa*(-LOG(Ra+etap)) - I3
!       ENDIF
      ENDIF
      IF (xix.EQ.0.) I5 = 0.

      fdx = -1/2./pi*( qj/Ra - I3*sind*cosd )                           ! udx/U2

      fdy = -1/2./pi*( ybar*qj/(Ra*(Ra+xix)) + cosd*Ba - I1*sind*cosd ) ! udx/U2

      fdz = -1/2./pi*( dbar*qj/(Ra*(Ra+xix)) + sind*Ba - I5*sind*cosd ) ! udx/U2

      RETURN
END SUBROUTINE

!---------------------------------------------------------------------
SUBROUTINE STRIKEDIS(qj,xix,etap,fsx,fsy,fsz,sind,cosd)
!---------------------------------------------------------------------
!                  Strike-slip Displacement solver
!                       for Rectanglar Fault
!---------------------------------------------------------------------
!
!  This prgram solves the strike-slip component of Okada analytical
!  solution (1985) based on
!
!    Okada, Y. 1985. Surface deformation due to shear and
!                    tensile faults in a half space.
!                    Bulletin of the Seismological Society
!                    of America,75(4) 1135-1154.
!
!   Y.Yong     2006  ORE, University of Hawaii Manoa
!   Y.Yamazaki 2007  ORE, University of Hawaii Manoa
!---------------------------------------------------------------------
!      USE ALL_INPUTS
      IMPLICIT NONE

      REAL(dp):: sind,cosd
      REAL(dp):: qj, xix, etap, fsx, fsy, fsz
      REAL(dp):: I1, I2, I3, I4, I5
      REAL(dp):: ybar, dbar
      REAL(dp):: Xa, Ra
      REAL(dp):: Aa, Ba


      ybar = etap*cosd + qj*sind
      dbar = etap*sind - qj*cosd

      Ra = SQRT(xix**2 + etap**2 + qj**2)
      Xa = SQRT(xix**2 + qj**2)

      Aa = nu/(lamda+nu)
      Ba = ATAN(xix*etap/(qj*Ra))

      IF (qj.EQ.0.) Ba=0
!      IF (Ra+etap.EQ.0.) Ra = -etap
      IF (cosd.EQ.0.) THEN
         I1 = -Aa/2*(xix*qj/((Ra+dbar)**2))
         I3 =  Aa/2*(etap/(Ra+dbar)+(ybar*qj)/((Ra+dbar)**2) - LOG(Ra+etap))
         I4 = -Aa*qj/(Ra+dbar)
         I5 = -Aa*xix*sind/(Ra+dbar)
      ELSE
!         IF (Ra+etap.EQ.0.) THEN
!            I4 = Aa/cosd*(log(Ra+dbar)+sind*log(Ra-etap))
!            I5 = 2.*Aa/cosd*atan((etap*(Xa+qj*cosd)+Xa*(Ra+Xa)*sind)/(xix*(Ra+Xa)*cosd))
!         ELSE
            I4 = Aa/cosd*(LOG(Ra+dbar) - sind*LOG(Ra+etap))
            I5 = 2.*Aa/cosd*ATAN((etap*(Xa+qj*cosd)+Xa*(Ra+Xa)*sind)/(xix*(Ra+Xa)*cosd))
            I1 = Aa*((-1/cosd)*(xix/(Ra+dbar))) - sind/cosd*I5
            I3 = Aa*((1/cosd)*(ybar/(Ra+dbar)) - LOG(Ra+etap))+sind/cosd*I4
            I2 = Aa*(-LOG(Ra+etap)) - I3

!         ENDIF
      ENDIF
      IF (xix.EQ.0.) I5 = 0.

!      IF (Ra+etap.EQ.0.) THEN
!        fsx = -1./2./pi*I1*sind
!      ELSE
         fsx = -1./2./pi*( xix*qj/(Ra*(Ra+etap)) + Ba + I1*sind)                ! usx/U1
         fsy = -1./2./pi*(ybar*qj/(Ra*(Ra+etap)) + qj*cosd/(Ra+etap) + I2*sind) ! usy/U1
         fsz = -1./2./pi*(dbar*qj/(Ra*(Ra+etap)) + qj*sind/(Ra+etap) + I4*sind) ! usz/U1
!      ENDIF
     
      RETURN
END SUBROUTINE

END PROGRAM NEOWAVE2D
!END PROGRAM

